##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.CommandHistory import CommandHistory
from gSculpt.CommandHistory.CommandTracker import CommandTracker

import GSGraph




class GSGraphSinkAddInputCommand (CommandHistory.Command):
	def __init__(self, sink, source):
		super( GSGraphSinkAddInputCommand, self ).__init__()
		self._sink = sink
		self._source = source


	def execute(self):
		self._sink.addInput( self._source )

	def unexecute(self):
		self._sink.removeInput( self._source )




class GSGraphSinkRemoveInputCommand (CommandHistory.Command):
	def __init__(self, sink, source):
		super( GSGraphSinkRemoveInputCommand, self ).__init__()
		self._sink = sink
		self._source = source
		index = sink.index( source )
		self._subsequentSources = [ sink.inputSource( i )   for i in xrange( index + 1, len( sink ) ) ]


	def execute(self):
		self._sink.removeInput( self._source )

	def unexecute(self):
		for source in reversed( self._subsequentSources ):
			self._sink.removeInput( source )
		self._sink.addInput( self._source )
		for source in self._subsequentSources:
			self._sink.addInput( source )





class GSGraphSinkCommandTracker (CommandTracker):
	def track(self, sink):
		super( GSGraphSinkCommandTracker, self ).track( sink )
		assert isinstance( sink, GSGraph.GSGraphSink )


	def stopTracking(self, sink):
		assert isinstance( sink, GSGraph.GSGraphSink )

		super( GSGraphSinkCommandTracker, self ).stopTracking( sink )



	def _f_onSinkInputAdded(self, sink, source, link):
		self._commandHistory.addCommand( GSGraphSinkAddInputCommand( sink, source ) )


	def _f_onSinkInputRemoved(self, sink, source, link):
		self._commandHistory.addCommand( GSGraphSinkRemoveInputCommand( sink, source ) )





class GSGraphAddNodeCommand (CommandHistory.Command):
	def __init__(self, graph, node):
		super( GSGraphAddNodeCommand, self ).__init__()
		self._graph = graph
		self._node = node


	def execute(self):
		self._graph.addNode( self._node )

	def unexecute(self):
		self._graph.removeNode( self._node )




class GSGraphRemoveNodeCommand (CommandHistory.Command):
	def __init__(self, graph, node):
		super( GSGraphRemoveNodeCommand, self ).__init__()
		self._graph = graph
		self._node = node



	def execute(self):
		self._graph.removeNode( self._node )

	def unexecute(self):
		self._graph.addNode( self._node )









class GSGraphCommandTracker (CommandTracker):
	"""GSGraph command tracker
	@bTrackNodeList - track the node list by responding to the addition and removal of nodes; this can be disabled, so that another module may handle this"""
	bTrackNodeList = False

	def __init__(self, commandHistory):
		"""Constructor
		@commandHistory - a CommandHistory that is to receive commands"""
		super( GSGraphCommandTracker, self ).__init__( commandHistory )
		self._bTrackNodeList = GSGraphCommandTracker.bTrackNodeList



	def track(self, graph):
		super( GSGraphCommandTracker, self ).track( graph )
		assert isinstance( graph, GSGraph.GSGraph )

		for node in graph.nodes:
			for sink in node.sinks:
				self._commandHistory.track( sink )


	def stopTracking(self, graph):
		assert isinstance( graph, GSGraph.GSGraph )

		for node in graph.nodes:
			for sink in node.sinks:
				self._commandHistory.stopTracking( sink )

		super( GSGraphCommandTracker, self ).stopTracking( graph )



	def _f_onGraphNodeAdded(self, graph, node):
		for sink in node.sinks:
			self._commandHistory.track( sink )
		if self._bTrackNodeList:
			self._commandHistory.addCommand( GSGraphAddNodeCommand( graph, node ) )


	def _f_onGraphNodeRemoved(self, graph, node):
		if self._bTrackNodeList:
			self._commandHistory.addCommand( GSGraphRemoveNodeCommand( graph, node ) )
		for sink in node.sinks:
			self._commandHistory.stopTracking( sink )



	def _f_onSinkInputAdded(self, sink, source, link):
		self._commandHistory.addCommand( GSGraphSinkAddInputCommand( sink, source ) )


	def _f_onSinkInputRemoved(self, sink, source, link):
		self._commandHistory.addCommand( GSGraphSinkRemoveInputCommand( sink, source ) )
