##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.Product import GSProduct
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Model.Model import GSProductModelBlank



class GSProductField (ObjectField):
	valueClass = GSProduct
	defaultValue = None

registerFieldType( GSProduct, GSProductField )




class ProcImportedModel (GSProcedure):
	description = 'Imported model'

	model = FieldColumn( GSProduct, None )

	inputClass = GSProductModelBlank


	def getOutputClass(self, inputClass):
		if self.model is None:
			return GSProductModelBlank
		else:
			return type( self.model )

	def procedureInvoke(self, inputObject, errorHandler):
		if self.model is None:
			return GSProductModelBlank()
		else:
			# It is *VERY* important that a *COPY* of the model is returned, otherwise
			# the imported model can get altered by further operations.
			return deepcopy( self.model )



class ToolImportedModel (GSProcedureTool):
	row = RowRefColumn( ProcImportedModel )

	functionClass = ProcImportedModel
	pageTitle = 'Import'
	titleText = 'Imported Model'


	layout = FormLayoutFactory()
	layout.row()
	layout.finalise()


