##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
import pygtk
pygtk.require( '2.0' )
import gtk

from gSculpt.Math.Math import Axis, Point3, Quaternion, Vector3

from gSculpt.VarEdit import VarEdit

from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor





class BoolColumnEditCheck (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, labelText):
		super( BoolColumnEditCheck, self ).__init__( [ 'check' ] )
		self._fieldColumn = fieldColumn
		self._labelText = labelText


	def createElement(self, rowEditContext):
		return VarEdit.BoolVarEditCheck( self._labelText )

	def initialiseElement(self, element, rowEditContext):
		super( BoolColumnEditCheck, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( BoolColumnEditCheck, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()




class IntColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page):
		super( IntColumnEditEntry, self ).__init__( [ 'entry' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page


	def createElement(self, rowEditContext):
		return VarEdit.IntVarEditEntry( self._minValue, self._maxValue, self._step, self._page )

	def initialiseElement(self, element, rowEditContext):
		super( IntColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( IntColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class IntColumnEditSlider (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page):
		super( IntColumnEditSlider, self ).__init__( [ 'entry' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page


	def createElement(self, rowEditContext):
		return VarEdit.IntVarEditSlider( self._minValue, self._maxValue, self._step, self._page )

	def initialiseElement(self, element, rowEditContext):
		super( IntColumnEditSlider, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( IntColumnEditSlider, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()




class FloatColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page, numDigits):
		super( FloatColumnEditEntry, self ).__init__( [ 'entry' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page
		self._numDigits = numDigits

	def createElement(self, rowEditContext):
		return VarEdit.FloatVarEditEntry( self._minValue, self._maxValue, self._step, self._page, self._numDigits )

	def initialiseElement(self, element, rowEditContext):
		super( FloatColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( FloatColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class FloatColumnEditSlider (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page):
		super( FloatColumnEditSlider, self ).__init__( [ 'slider' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page

	def createElement(self, rowEditContext):
		return VarEdit.FloatVarEditSlider( self._minValue, self._maxValue, self._step, self._page )

	def initialiseElement(self, element, rowEditContext):
		super( FloatColumnEditSlider, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( FloatColumnEditSlider, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class FloatColumnEditSlider (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page):
		super( FloatColumnEditSlider, self ).__init__( [ 'slider' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page

	def createElement(self, rowEditContext):
		return VarEdit.FloatVarEditSlider( self._minValue, self._maxValue, self._step, self._page )

	def initialiseElement(self, element, rowEditContext):
		super( FloatColumnEditSlider, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( FloatColumnEditSlider, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()








class StringColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( StringColumnEditEntry, self ).__init__( [ 'entry' ] )
		self._fieldColumn = fieldColumn

	def createElement(self, rowEditContext):
		return VarEdit.StringVarEditEntry()

	def initialiseElement(self, element, rowEditContext):
		super( StringColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( StringColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()




class StringColumnViewLabel (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( StringColumnViewLabel, self ).__init__( [ 'label' ] )
		self._fieldColumn = fieldColumn

	def createElement(self, rowEditContext):
		return VarEdit.StringVarViewLabel()

	def initialiseElement(self, element, rowEditContext):
		super( StringColumnViewLabel, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( StringColumnViewLabel, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()




class StringColumnEditAccelEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( StringColumnEditAccelEntry, self ).__init__( [ 'entry' ] )
		self._fieldColumn = fieldColumn

	def createElement(self, rowEditContext):
		return VarEdit.StringVarEditAccelEntry()

	def initialiseElement(self, element, rowEditContext):
		super( StringColumnEditAccelEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( StringColumnEditAccelEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()








class Vector2ColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page, numDigits):
		super( Vector2ColumnEditEntry, self ).__init__( [ 'x', 'y' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page
		self._numDigits = numDigits

	def createElement(self, rowEditContext):
		return VarEdit.Vector2VarEditEntry( self._minValue, self._maxValue, self._step, self._page, self._numDigits )

	def initialiseElement(self, element, rowEditContext):
		super( Vector2ColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( Vector2ColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class Point2ColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page, numDigits):
		super( Point2ColumnEditEntry, self ).__init__( [ 'x', 'y' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page
		self._numDigits = numDigits

	def createElement(self, rowEditContext):
		return VarEdit.Point2VarEditEntry( self._minValue, self._maxValue, self._step, self._page, self._numDigits )

	def initialiseElement(self, element, rowEditContext):
		super( Point2ColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( Point2ColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()





class Vector3ColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page, numDigits):
		super( Vector3ColumnEditEntry, self ).__init__( [ 'x', 'y', 'z' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page
		self._numDigits = numDigits

	def createElement(self, rowEditContext):
		return VarEdit.Vector3VarEditEntry( self._minValue, self._maxValue, self._step, self._page, self._numDigits )

	def initialiseElement(self, element, rowEditContext):
		super( Vector3ColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( Vector3ColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class Point3ColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, minValue, maxValue, step, page, numDigits):
		super( Point3ColumnEditEntry, self ).__init__( [ 'x', 'y', 'z' ] )
		self._fieldColumn = fieldColumn
		self._minValue = minValue
		self._maxValue = maxValue
		self._step = step
		self._page = page
		self._numDigits = numDigits

	def createElement(self, rowEditContext):
		return VarEdit.Point3VarEditEntry( self._minValue, self._maxValue, self._step, self._page, self._numDigits )

	def initialiseElement(self, element, rowEditContext):
		super( Point3ColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( Point3ColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()




class QuaternionColumnEditEntry (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( QuaternionColumnEditEntry, self ).__init__( [ 'x', 'y', 'z' ] )
		self._fieldColumn = fieldColumn

	def createElement(self, rowEditContext):
		return VarEdit.QuaternionVarEditEntry()

	def initialiseElement(self, element, rowEditContext):
		super( QuaternionColumnEditEntry, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( QuaternionColumnEditEntry, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class QuaternionColumnEditDirectionPopup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn, labelText):
		super( QuaternionColumnEditDirectionPopup, self ).__init__( [ 'popup' ] )
		self._fieldColumn = fieldColumn
		self._labelText = labelText

	def createElement(self, rowEditContext):
		return VarEdit.QuaternionVarEditDirectionPopup( rowEditContext.view, self._labelText )

	def initialiseElement(self, element, rowEditContext):
		super( QuaternionColumnEditDirectionPopup, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( QuaternionColumnEditDirectionPopup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()




class Colour3fColumnEditColourButton (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( Colour3fColumnEditColourButton, self ).__init__( [ 'colourButton' ] )
		self._fieldColumn = fieldColumn

	def createElement(self, rowEditContext):
		return VarEdit.Colour3fVarEditColourButton()

	def initialiseElement(self, element, rowEditContext):
		super( Colour3fColumnEditColourButton, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( Colour3fColumnEditColourButton, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()



class AxisColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( AxisColumnEditRadioGroup, self ).__init__( [ 'x', 'y', 'z', 'negativeX', 'negativeY', 'negativeZ' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return VarEdit.AxisVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( AxisColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )

	def shutdownElement(self, element, rowEditContext):
		super( AxisColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()





if __name__ == '__main__':
	import pygtk
	pygtk.require( '2.0' )
	import gtk


	from gSculpt.Kernel.DB import FieldColumn, Row, RowRefColumn, FieldProxyColumn
	from gSculpt.Kernel.DBEdit import RowEditor
	from gSculpt.Kernel.ColumnEditors import IntColumnEditEntry
	from gSculpt.UI.FormLayout import FormLayoutFactory
	from gSculpt.CommandHistory.CommandHistory import CommandHistory
	from gSculpt.Editor.EditorSettings import EditorSettings


	class ExampleRow (Row):
		a = FieldColumn( int, 1 )

	class ExampleRow2 (Row):
		b = FieldColumn( int, 2 )

	class ExampleRow3 (Row):
		x = RowRefColumn( ExampleRow )
		y = RowRefColumn( ExampleRow2 )
		a = FieldProxyColumn( x.a )
		b = FieldProxyColumn( y.b )



	class ExampleEditor (RowEditor):
		rowClass = ExampleRow
		aEdit = IntColumnEditEntry( ExampleRow.a, 0, 1048576, 1, 10 )

		layout = FormLayoutFactory()
		layout.row()  <<  'A:'  <<  aEdit
		layout.row()
		layout.finalise()

	class Example2Editor (RowEditor):
		rowClass = ExampleRow2
		bEdit = IntColumnEditEntry( ExampleRow2.b, 0, 1048576, 1, 10 )

		layout = FormLayoutFactory()
		layout.row()  <<  'B:'  <<  bEdit
		layout.row()
		layout.finalise()

	class Example3Editor (RowEditor):
		rowClass = ExampleRow3
		aEdit = IntColumnEditEntry( ExampleRow3.a, 0, 1048576, 1, 10 )
		bEdit = IntColumnEditEntry( ExampleRow3.b, 0, 1048576, 1, 10 )

		layout = FormLayoutFactory()
		layout.row()  <<  'A:'  <<  aEdit
		layout.row()  <<  'B:'  <<  bEdit
		layout.row()
		layout.finalise()


	def onDelete(widget, event):
		gtk.main_quit()


	def makeEditorWindow(editor, row):
		window = gtk.Window( gtk.WINDOW_TOPLEVEL )
		window.connect( 'delete-event', onDelete )
		window.show()
		window.add( editor.getRowEditorWidget() )

		editor.attachRow( row, view, editorSettings, window, commandHistory )
		editor.createRowEditorUI()
		editor.initialiseRowEditorUI()


	commandHistory = CommandHistory()
	view = None
	editorSettings = None


	row = ExampleRow()
	editor = ExampleEditor()
	makeEditorWindow( editor, row )

	row2 = ExampleRow2()
	editor2 = Example2Editor()
	makeEditorWindow( editor2, row2 )

	row3 = ExampleRow3()
	editor3 = Example3Editor()
	row3.x = row
	row3.y = row2
	makeEditorWindow( editor3, row3 )




	gtk.main()

