##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.CommandHistory import CommandHistory
from gSculpt.CommandHistory.CommandTracker import CommandTracker




class RowFieldValueCommand (CommandHistory.Command):
	def __init__(self, row, fieldName, oldValue, newValue):
		super( RowFieldValueCommand, self ).__init__()
		self._row = row
		self._valueTable = { fieldName : [ oldValue, newValue ] }


	def execute(self):
		for fieldName, ( oldValue, newValue )  in  self._valueTable.items():
			self._row.getFieldByName( fieldName ).setValue( deepcopy( newValue ) )

	def unexecute(self):
		for fieldName, ( oldValue, newValue )  in  self._valueTable.items():
			self._row.getFieldByName( fieldName ).setValue( deepcopy( oldValue ) )


	def canJoinWith(self, command):
		return isinstance( command, RowFieldValueCommand )  and  self._row is command._row  and  not self._bFinished

	def joinWith(self, command):
		for fieldName, ( oldValue, newValue )  in  command._valueTable.items():
			if self._valueTable.has_key( fieldName ):
				self._valueTable[fieldName][1] = newValue
			else:
				self._valueTable[fieldName] = [ oldValue, newValue ]






class RowCommandTracker (CommandTracker):
	def __init__(self, commandHistory):
		super( RowCommandTracker, self ).__init__( commandHistory )
		self._oldValue = None


	def track(self, row):
		super( RowCommandTracker, self ).track( row )

		compositeFields = row._f_getCompositeFields()
		for field in compositeFields:
			self._commandHistory.track( field.getValue() )


	def stopTracking(self, row):
		compositeFields = row._f_getCompositeFields()
		for field in compositeFields:
			self._commandHistory.stopTracking( field.getValue() )

		super( RowCommandTracker, self ).stopTracking( row )



	def _f_onRowChildFieldPreModify(self, row, fieldName):
		field = row.getFieldByName( fieldName )
		self._oldValue = deepcopy( field.value )

	def _f_onRowChildFieldModified(self, row, fieldName):
		field = row.getFieldByName( fieldName )
		self._commandHistory.addCommand( RowFieldValueCommand( row, fieldName, self._oldValue, deepcopy( field.value ) ) )




