##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Util.SignalSlot import ClassSignal
from gSculpt.Kernel.ObjectSource import ObjectSource



#//////////////////////////////////////////////////////////////////////////////
#		Object Source Proxy
#//////////////////////////////////////////////////////////////////////////////

class ObjectSourceProxy (ObjectSource):
	"""Variable proxy class; forwards signals from the target variable, and forwards calls to the target variable"""
	def __init__(self, target=None):
		"""Constructor
		@target - target variable interface"""
		super( ObjectSourceProxy, self ).__init__()
		self._target = None
		self._o_initTarget( target )


	def isValid(self):
		"""Determine if the variable is valid"""
		if self._target is not None:
			return self._target.isValid()
		else:
			return False

	def getValue(self):
		"""Get the variable value"""
		if self._target is not None:
			return self._target.getValue()
		else:
			return None


	def getTarget(self):
		"""Get the target variable interface"""
		return self._target

	def setTarget(self, target):
		"""Set the target variable interface
		@target - the target"""
		if target is not self._target:
			# Keep the old target
			oldTarget = self._target
			# Record the validity
			bWasValid = self.isValid()

			# Disconnect signals
			if self._target is not None:
				self._target.validitySignal.chainDisconnect( self.validitySignal )
				self._target.changedSignal.chainDisconnect( self.changedSignal )

			# Set the target
			self._target = target

			bIsValid = self.isValid()

			# If the variable is becoming invalid, then emit the validity signal BEFORE emitting the changed signal
			if bWasValid and not bIsValid:
				self.validitySignal.emit( False )

			# Notify of value change
			self.changedSignal.emit()

			# Connect signals
			if self._target is not None:
				self._target.validitySignal.chainConnect( self.validitySignal )
				self._target.changedSignal.chainConnect( self.changedSignal )

			# If the variable is becoming valid, emit the validity signal AFTER emitting the changed signal
			if not bWasValid and bIsValid:
				self.validitySignal.emit( True )


	def _o_initTarget(self, target):
		"""Set the target variable interface
		@target - the target"""
		if target is not None:
			# Set the target
			self._target = target

			# Connect signals
			self._target.validitySignal.chainConnect( self.validitySignal )
			self._target.changedSignal.chainConnect( self.changedSignal )

			self.validitySignal.emit( True )




	value = property( getValue )
	target = property( getTarget, setTarget )
