##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
import os

from gSculpt.FileIO.IOXml import *

from gSculpt.Config import GSConfig

from gSculpt.UI.UIAction import UIAction

from gSculpt.Kernel.DB import *



class Shortcut (Row):
	path = FieldColumn( str, '' )
	accelString = FieldColumn( str, '' )



	def __init__(self):
		super( Shortcut, self ).__init__()

		self.fields.accelString.changedSignal.connect( self._p_updateAction )
		self.bModified = False
		self.defaultAccelString = self.accelString


	def reset(self):
		self.accelString = self.defaultAccelString
		self.bModified = False



	def _p_updateAction(self):
		self.bModified = True
		if self.path != '':
			try:
				action = UIAction._pathToActionTable[self.path]
			except KeyError:
				pass
			else:
				action.accelerator = self.accelString


	def _f_copyFrom(self, shortcut):
		self.accelString = shortcut.accelString





class ShortcutTable (object):
	def __init__(self):
		self._pathToShortcut = {}

		self._p_populate()



	def getShortcuts(self):
		return self._pathToShortcut.values()



	def _p_populate(self):
		for action in UIAction._actions:
			shortcut = self._p_newShortcut( action )
			self._pathToShortcut[shortcut.path] = shortcut

	def _p_clear(self):
		self._pathToShortcut = {}
		self._p_populate()



	def __readxml__(self, xmlNode):
		for node in xmlNode.childrenNamed( 'shortcut' ):
			shortcut = Shortcut()
			node  >>  shortcut

			try:
				existingShortcut = self._pathToShortcut[shortcut.path]
			except KeyError:
				pass
			else:
				existingShortcut._f_copyFrom( shortcut )


	def __writexml__(self, xmlNode):
		for shortcut in self._pathToShortcut.values():
			if shortcut.bModified:
				xmlNode.addChild( 'shortcut' )  <<  shortcut



	def _p_newShortcut(self, action):
		shortcut = Shortcut()
		shortcut.accelString = action.accelerator
		shortcut.defaultAccelString = shortcut.accelString
		shortcut.bModified = False
		shortcut.path = action.path
		return shortcut


ioObjectFactoryRegister( 'ShortcutTable', ShortcutTable )




def readShortcuts(shortcutTable, filename):
	if os.path.exists( filename ):
		f = open( filename, 'r' )
		if f is not None:
			doc = InputXmlDocument()
			doc.parseFile( f )
			contentNode = doc.getContentNode()
			if contentNode.isValid():
				contentNode  >>  shortcutTable
	else:
		raise IOError, 'Shortcuts file \'%s\' does not exist'  %  ( filename, )



def writeShortcuts(shortcutTable, filename=None):
	doc = OutputXmlDocument()
	doc.getContentNode()  <<  shortcutTable
	f = open( filename, 'w' )
	if f is not None:
		doc.writeFile( f )
		f.close()




def readSystemShortcuts(shortcutTable):
	try:
		readShortcuts( shortcutTable, GSConfig.shortcutsConfigPath )
	except IOError:
		print 'Could not read system shortcuts file'



def writeSystemShortcuts(shortcutTable):
	try:
		writeShortcuts( shortcutTable, GSConfig.shortcutsConfigPath )
	except IOError:
		print 'Could not write system shortcuts file'

