##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Point3, Quaternion, Vector3

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import ColumnEditor, ColumnWidgetEditor, ColumnEditorComponent

import ScrollManipulator, Point3Manipulator, TranslationManipulator3d, ScaleManipulator3d, RotationManipulator, RealManipulator, ManipulatorSwitch






class ColumnManipulatorSnapping (object):
	def __init__(self, attrName):
		super( ColumnManipulatorSnapping, self ).__init__()
		self._attrName = attrName

	def getManipulatorSnapping(self, rowEditContext):
		return getattr( rowEditContext.editorSettings.snapSettings, self._attrName )


class ColumnManipulatorSnappingDistance (ColumnManipulatorSnapping):
	def __init__(self):
		super( ColumnManipulatorSnappingDistance, self ).__init__( 'distance' )

class ColumnManipulatorSnappingFraction (ColumnManipulatorSnapping):
	def __init__(self):
		super( ColumnManipulatorSnappingFraction, self ).__init__( 'fraction' )

class ColumnManipulatorSnappingAngle (ColumnManipulatorSnapping):
	def __init__(self):
		super( ColumnManipulatorSnappingAngle, self ).__init__( 'angle' )





class ColumnManipulator (ColumnEditor):
	def __init__(self):
		super( ColumnManipulator, self ).__init__()
		self._bEnabled = False

	def enable(self):
		assert not self._f_metaMember_isInitialised(), 'cannot enable; column manipulator has already been initialised'
		self._bEnabled = True

	def disable(self):
		assert not self._f_metaMember_isInitialised(), 'cannot disable; column manipulator has already been initialised'
		self._bEnabled = False


	def _o_getManipulatorSnapping(self, columnManipulatorSnapping, rowEditContext):
		if columnManipulatorSnapping is not None:
			return columnManipulatorSnapping.getManipulatorSnapping( rowEditContext )
		else:
			return None




class ColumnManipulatorSwitch (ColumnManipulator):
	def __init__(self, columnManipulators):
		super( ColumnManipulatorSwitch, self ).__init__()
		self._columnManipulators = columnManipulators
		for columnManipulator in columnManipulators:
			self._o_addDependency( columnManipulator )


	def createElement(self, rowEditContext):
		manipulators = [ rowEditContext.rowEditor._p_getUIElementForColumnEditor( columnManip )  for columnManip in self._columnManipulators ]
		return ManipulatorSwitch.ManipulatorSwitch( manipulators )

	def initialiseElement(self, element, rowEditContext):
		super( ColumnManipulatorSwitch, self ).initialiseElement( element, rowEditContext )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( ColumnManipulatorSwitch, self ).shutdownElement( element, rowEditContext )
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()




class ColumnManipulatorSwitchRadioGroup (ColumnWidgetEditor):
	class ChoiceComponent (ColumnEditorComponent):
		def __init__(self, columnManipSwitchRadioGroup, index):
			super( ColumnManipulatorSwitchRadioGroup.ChoiceComponent, self ).__init__()
			self._columnManipSwitchRadioGroup = columnManipSwitchRadioGroup
			self._index = index

		def getFactory(self):
			"Get the factory that owns this factory component"
			return self._columnManipSwitchRadioGroup

		def getElementWidget(self, element):
			"""Get the widget of @element that is represented by this factory component
			@element - the UI element"""
			return element.getButtonByIndex( self._index )


	def __init__(self, columnManipSwitch):
		super( ColumnManipulatorSwitchRadioGroup, self ).__init__()
		self._columnManipSwitch = columnManipSwitch
		self._choices = []



	def addChoice(self, columnManipulator, action):
		index = len( self._choices )
		self._o_addDependency( columnManipulator )
		self._choices.append( ( columnManipulator, action ) )
		return ColumnManipulatorSwitchRadioGroup.ChoiceComponent( self, index )


	def createElement(self, rowEditContext):
		element = ManipulatorSwitch.ManipulatorSwitchRadioGroup()
		manipSwitch = self._columnManipSwitch._f_getVarEdit( rowEditContext.rowEditor )
		element.attachManipulatorSwitch( manipSwitch )
		for columnManip, action in self._choices:
			manip = columnManip._f_getVarEdit( rowEditContext.rowEditor )
			element.addChoice( manip, action )
		return element

	def destroyElement(self, element, rowEditContext):
		super( ColumnManipulatorSwitchRadioGroup, self ).destroyElement( element, rowEditContext )
		element.detachManipulatorSwitch()


	def installElementAccelerators(self, element, accels, rowEditContext):
		super( ColumnManipulatorSwitchRadioGroup, self ).installElementAccelerators( element, accels, rowEditContext )
		element.installAccelerators( accels )

	def uninstallElementAccelerators(self, element, accels, rowEditContext):
		super( ColumnManipulatorSwitchRadioGroup, self ).uninstallElementAccelerators( element, accels, rowEditContext )
		element.uninstallAccelerators( accels )





class IntColumnScrollManipulator (ColumnManipulator):
	def __init__(self, fieldColumn, lower, upper, rate=1):
		super( IntColumnScrollManipulator, self ).__init__()
		self._fieldColumn = fieldColumn
		self._lower = lower
		self._upper = upper
		self._rate = rate


	def createElement(self, rowEditContext):
		return ScrollManipulator.ScrollManipulator( self._lower, self._upper, self._rate )

	def initialiseElement(self, element, rowEditContext):
		super( IntColumnScrollManipulator, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( IntColumnScrollManipulator, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()




class Point3ColumnManipulator (ColumnManipulator):
	def __init__(self, fieldColumn, snapping=None):
		super( Point3ColumnManipulator, self ).__init__()
		self._fieldColumn = fieldColumn
		self._snapping = snapping


	def createElement(self, rowEditContext):
		return Point3Manipulator.Point3Manipulator( snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( Point3ColumnManipulator, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( Point3ColumnManipulator, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()




class PositionedColumnManipulator3d (ColumnManipulator):
	def __init__(self, position=None):
		super( PositionedColumnManipulator3d, self ).__init__()
		if position is not None:
			self._position = position
		else:
			self._position = Point3()


	def initialiseElement(self, element, rowEditContext):
		super( PositionedColumnManipulator3d, self ).initialiseElement( element, rowEditContext )
		if self._position is not None:
			if isinstance( self._position, Point3 ):
				element.position = self._position
			elif isinstance( self._position, ObjectSourceColumnInterface ):
				element.position = self._p_getObjectSource( self._position, rowEditContext )

	def shutdownElement(self, element, rowEditContext):
		super( PositionedColumnManipulator3d, self ).shutdownElement( element, rowEditContext )
		element.position = None



class TransformedColumnManipulator3d (PositionedColumnManipulator3d):
	def __init__(self, position=None, spaceRotation=None):
		super( TransformedColumnManipulator3d, self ).__init__( position )
		if spaceRotation is not None:
			self._spaceRotation = spaceRotation
		else:
			self._spaceRotation = Quaternion()


	def initialiseElement(self, element, rowEditContext):
		super( TransformedColumnManipulator3d, self ).initialiseElement( element, rowEditContext )
		if self._spaceRotation is not None:
			if isinstance( self._spaceRotation, Quaternion ):
				element.spaceRotation = self._spaceRotation
			elif isinstance( self._spaceRotation, ObjectSourceColumnInterface ):
				element.spaceRotation = self._p_getObjectSource( self._spaceRotation, rowEditContext )

	def shutdownElement(self, element, rowEditContext):
		super( TransformedColumnManipulator3d, self ).shutdownElement( element, rowEditContext )
		element.spaceRotation = None



class Translation2ColumnManipulator3d (TransformedColumnManipulator3d):
	def __init__(self, fieldColumn, position, spaceRotation=None, snapping=None):
		super( Translation2ColumnManipulator3d, self ).__init__( position, spaceRotation )
		self._fieldColumn = fieldColumn
		self._snapping = snapping


	def createElement(self, rowEditContext):
		return TranslationManipulator3d.Translation2Manipulator3d( snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( Translation2ColumnManipulator3d, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( Translation2ColumnManipulator3d, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()




class Translation3ColumnManipulator3d (TransformedColumnManipulator3d):
	def __init__(self, fieldColumn, position, spaceRotation=None, snapping=None):
		super( Translation3ColumnManipulator3d, self ).__init__( position, spaceRotation )
		self._fieldColumn = fieldColumn
		self._snapping = snapping


	def createElement(self, rowEditContext):
		return TranslationManipulator3d.Translation3Manipulator3d( snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( Translation3ColumnManipulator3d, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( Translation3ColumnManipulator3d, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()




class Scale2ColumnManipulator3d (TransformedColumnManipulator3d):
	def __init__(self, fieldColumn, position, spaceRotation=None, snapping=None):
		super( Scale2ColumnManipulator3d, self ).__init__( position, spaceRotation )
		self._fieldColumn = fieldColumn
		self._snapping = snapping


	def createElement(self, rowEditContext):
		return ScaleManipulator3d.Scale2Manipulator3d( snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( Scale2ColumnManipulator3d, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( Scale2ColumnManipulator3d, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()





class Scale3ColumnManipulator3d (TransformedColumnManipulator3d):
	def __init__(self, fieldColumn, position, spaceRotation=None, snapping=None):
		super( Scale3ColumnManipulator3d, self ).__init__( position, spaceRotation )
		self._fieldColumn = fieldColumn
		self._snapping = snapping


	def createElement(self, rowEditContext):
		return ScaleManipulator3d.Scale3Manipulator3d( snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( Scale3ColumnManipulator3d, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( Scale3ColumnManipulator3d, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()





class RotationColumnManipulator (TransformedColumnManipulator3d):
	def __init__(self, fieldColumn, position, spaceRotation=None, snapping=None):
		super( RotationColumnManipulator, self ).__init__( position, spaceRotation )
		self._fieldColumn = fieldColumn
		self._snapping = snapping


	def createElement(self, rowEditContext):
		return RotationManipulator.RotationManipulator( snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( RotationColumnManipulator, self ).initialiseElement( element, rowEditContext )
		var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( RotationColumnManipulator, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()





class RealColumnManipulator (ColumnManipulator):
	class WorldSpaceScaleFactory (object):
		def __init__(self, pointSourceDecl):
			self._pointSourceDecl = pointSourceDecl

		def createScale(self, rowEditContext):
			if isinstance( self._pointSourceDecl, ObjectSourceColumnInterface ):
				pointSource = ColumnEditor._p_getObjectSource( self._pointSourceDecl, rowEditContext )
			return RealManipulator.RealManipulator.WorldSpaceScale( pointSource )





class Real1ColumnManipulator (RealColumnManipulator):
	class WorldSpaceScaleFactory (object):
		def __init__(self, focalPointSourceColumn):
			self._focalPointSourceColumn = focalPointSourceColumn

		def createScale(self, rowEditContext):
			focalPointSource = self._focalPointSourceColumn._f_getObjectSourceFromInstance( rowEditContext.rowEditor )
			return RealManipulator.RealManipulator.WorldSpaceScale( focalPointSource )


	def __init__(self, fieldColumn, lower, upper, scale, snapping=None):
		super( Real1ColumnManipulator, self ).__init__()
		assert scale is not None, 'must supply a scale'
		self._fieldColumn = fieldColumn
		self._lower = lower
		self._upper = upper
		self._scale = scale
		self._snapping = snapping


	def createElement(self, rowEditContext):
		scale = self._scale
		if scale is not None  and  isinstance( scale, RealColumnManipulator.WorldSpaceScaleFactory ):
			scale = scale.createScale( rowEditContext )
		return RealManipulator.Real1Manipulator( self._lower, self._upper, scale, snapping=self._o_getManipulatorSnapping( self._snapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( RealColumnManipulator, self ).initialiseElement( element, rowEditContext )
		var = None
		if self._fieldColumn is not None:
			var = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( var )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( RealColumnManipulator, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()






class Real2ColumnManipulator (RealColumnManipulator):
	def __init__(self, xFieldColumn, xLower, xUpper, xScale, yFieldColumn, yLower, yUpper, yScale, xSnapping=None, ySnapping=None):
		super( RealColumnManipulator, self ).__init__()
		assert xScale is not None, 'must supply an x-scale'
		assert yScale is not None, 'must supply an y-scale'
		self._xFieldColumn = xFieldColumn
		self._xLower = xLower
		self._xUpper = xUpper
		self._xScale = xScale
		self._yFieldColumn = yFieldColumn
		self._yLower = yLower
		self._yUpper = yUpper
		self._yScale = yScale
		self._xSnapping = xSnapping
		self._ySnapping = ySnapping


	def createElement(self, rowEditContext):
		xScale = self._xScale
		yScale = self._yScale
		if xScale is not None  and  isinstance( xScale, RealColumnManipulator.WorldSpaceScaleFactory ):
			xScale = xScale.createScale( rowEditContext )
		if yScale is not None  and  isinstance( yScale, RealColumnManipulator.WorldSpaceScaleFactory ):
			yScale = yScale.createScale( rowEditContext )
		return RealManipulator.Real2Manipulator( self._xLower, self._xUpper, xScale, self._yLower, self._yUpper, yScale,
										self._o_getManipulatorSnapping( self._xSnapping, rowEditContext ), self._o_getManipulatorSnapping( self._ySnapping, rowEditContext ) )

	def initialiseElement(self, element, rowEditContext):
		super( RealColumnManipulator, self ).initialiseElement( element, rowEditContext )
		xVar = None
		yVar = None
		if self._xFieldColumn is not None:
			xVar = self._p_getVar( self._xFieldColumn, rowEditContext )
		if self._yFieldColumn is not None:
			yVar = self._p_getVar( self._yFieldColumn, rowEditContext )
		element.attachVariables( xVar, yVar )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		super( RealColumnManipulator, self ).shutdownElement( element, rowEditContext )
		element.detachVariables()
		element.commandHistory = None

	def installElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		if self._bEnabled:
			element.manipDetachFromView()

