##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Point3, Quaternion

from gSculpt.Kernel.ObjectSource import ObjectSource

from gSculpt.VarEdit.VarEdit import VarEdit

from gSculpt.Event.QueuedEvent import queueEvent



class Manipulator (VarEdit):
	def __init__(self):
		super( Manipulator, self ).__init__()
		self.commandHistory = None
		self._view = None


	def manipAttachToView(self, view):
		assert self._view is None, 'already attached to a view'
		self._view = view

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		self._view = None


	def _o_viewPostRedraw(self):
		if self._view is not None:
			self._view.postRedraw()


	def _p_commandHistoryBreak(self):
		if self.commandHistory is not None:
			self.commandHistory.finishCommand()




class ManipulatorParameter (object):
	def __init__(self, valueType):
		self._value = None
		self._source = None
		self._valueType = valueType
		self.onRefreshed = None
		self.onValidValue = None


	def refresh(self):
		if self._source is not None:
			self._value = self._source.getValue()
			assert self._value is None  or  isinstance( self._value, self._valueType )


		if self._value is not None  and  self.onValidValue is not None:
			self.onValidValue()

		if self.onRefreshed is not None:
			self.onRefreshed()



	def isValid(self):
		return self._value is not None


	def _p_onSourceValueChanged(self):
		queueEvent( self.refresh )


	def _p_getValue(self):
		return self._value

	def _p_setValue(self, val):
		if self._source is not None:
			self._source.changedSignal.disconnect( self._p_onSourceValueChanged )

		self._value = None
		self._source = None

		if val is not None:
			if isinstance( val, self._valueType ):
				self._value = val
			elif isinstance( val, ObjectSource ):
				self._source = val
			else:
				raise TypeError, '@val is not an ObjectSource, or one of the acceptable value types'

		if self._source is not None:
			self._source.changedSignal.connect( self._p_onSourceValueChanged )

		self.refresh()


	value = property( _p_getValue, _p_setValue )




class PositionedManipulator3d (Manipulator):
	def __init__(self):
		super( PositionedManipulator3d, self ).__init__()
		self._position = ManipulatorParameter( Point3 )
		self._position.onValidValue = self._o_setHandlePosition
		self._position.onRefreshed = self._p_onParamRefreshed


	def _p_onParamRefreshed(self):
		self._o_refreshSensitivity()
		self._o_viewPostRedraw()



	def _o_setHandlePosition(self):
		assert False, 'abstract'


	def _p_getPosition(self):
		return self._position.value

	def _p_setPosition(self, pos):
		self._position.value = pos


	position = property( _p_getPosition, _p_setPosition )




class TransformedManipulator3d (PositionedManipulator3d):
	def __init__(self):
		super( TransformedManipulator3d, self ).__init__()
		self._spaceRotation = ManipulatorParameter( Quaternion )
		self._spaceRotation.onValidValue = self._o_setHandleSpaceRotation
		self._spaceRotation.onRefreshed = self._p_onParamRefreshed


	def _o_setHandleSpaceRotation(self):
		assert False, 'abstract'


	def _p_getSpaceRotation(self):
		return self._spaceRotation.value

	def _p_setSpaceRotation(self, rot):
		self._spaceRotation.value = rot


	spaceRotation = property( _p_getSpaceRotation, _p_setSpaceRotation )

