##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Kernel.Variable import IntVar

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.UI.UIAction import UIActionInstanceSignal

from gSculpt.Manipulator.Manipulator import Manipulator



class ManipulatorSwitch (Manipulator):
	def __init__(self, manipulators):
		super( ManipulatorSwitch, self ).__init__()
		self._manipulators = manipulators
		self.switchIndex = IntVar( 0 )
		self.switchIndex.preChangeSignal.connect( self._p_onPreSwitch )
		self.switchIndex.valueSignal.connect( self._p_onSwitch )
		self._currentManipulator = self._manipulators[self.switchIndex.value]


	def manipAttachToView(self, view):
		super( ManipulatorSwitch, self ).manipAttachToView( view )
		if self._currentManipulator is not None:
			self._currentManipulator.manipAttachToView( view )

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		if self._currentManipulator is not None:
			self._currentManipulator.manipDetachFromView()
		super( ManipulatorSwitch, self ).manipDetachFromView()


	def index(self, manipulator):
		return self._manipulators.index( manipulator )


	def _o_setUISensitivity(self, bSensitive):
		pass


	def _p_onPreSwitch(self):
		if self._currentManipulator is not None:
			if self._view is not None:
				self._currentManipulator.manipDetachFromView()
			self._currentManipulator = None

	def _p_onSwitch(self, value):
		self._currentManipulator = self._manipulators[value]
		if self._view is not None:
			self._currentManipulator.manipAttachToView( self._view )




class ManipulatorSwitchRadioGroup (object):
	"""Manipulator switch radio group"""
	def __init__(self):
		super( ManipulatorSwitchRadioGroup, self ).__init__()
		self._manipSwitch = None
		self._varEdit = ChoiceVarEditRadioGroup()
		self._actions = []
		self._actionToIndex = {}


	def attachManipulatorSwitch(self, switch):
		assert self._manipSwitch is None, 'manipulator switch already attached'
		self._manipSwitch = switch
		self._varEdit.attachVariable( self._manipSwitch.switchIndex )

	def detachManipulatorSwitch(self):
		assert self._manipSwitch is not None, 'no manipulator switch attached'
		self._varEdit.detachVariable( self._manipSwitch.switchIndex )
		self._manipSwitch = None


	def addChoice(self, manipulator, action):
		actionInstance = action.instance( UIActionInstanceSignal )
		actionInstance.actionSignal.connect( self._p_onAction )
		index = self._manipSwitch.index( manipulator )
		self._actionToIndex[actionInstance] = index
		self._actions.append( actionInstance )
		return self._varEdit.addChoice( index, contents=action.createButtonContentsWidget() )


	def show(self):
		self._varEdit.show()

	def hide(self):
		self._varEdit.hide()


	def setSensitive(self, bSensitive):
		self._varEdit.setSensitive( bSensitive )


	def installAccelerators(self, accels):
		for action in self._actions:
			action.installAccel( accels )

	def uninstallAccelerators(self, accels):
		for action in self._actions:
			action.uninstallAccel( accels )


	def getButtonForManipulator(self, choice):
		return self._varEdit.getButtonForChoice( choice )

	def getButtonByIndex(self, index):
		return self._varEdit.getButtonByIndex( index )


	def _p_onAction(self, action):
		index = self._actionToIndex[action]
		self._manipSwitch.switchIndex.value = index

