##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Util.SignalSlot import ClassSignal

from gSculpt.Math.Math import Point3

from TranslationHandle3d import Translation3Handle3d

from gSculpt.Manipulator.Manipulator import Manipulator




class Point3Manipulator (Manipulator):
	startSignal = ClassSignal()
	manipulateSignal = ClassSignal()
	finishSignal = ClassSignal()


	def __init__(self, snapping=None):
		super( Point3Manipulator, self ).__init__()
		self._var = None
		self._handle = Translation3Handle3d()
		self._initialValue = None
		self.snapping = snapping

		self._o_setUISensitivity( False )


	def manipAttachToView(self, view):
		super( Point3Manipulator, self ).manipAttachToView( view )
		if self._var is not None:
			self._handle.attachToView( self._view )

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		if self._var is not None:
			self._handle.detachFromView()
		super( Point3Manipulator, self ).manipDetachFromView()


	def attachVariable(self, var):
		assert self._var is None, 'already attached to a variable'
		self._var = var
		self._var.valueSignal.connect( self._p_onVar )

		self._o_watchVarValidity( self._var )
		self._o_refreshSensitivity()

		self._handle.startSignal.connect( self._p_onStart )
		self._handle.translateSignal.connect( self._p_onTranslate )
		self._handle.finishSignal.connect( self._p_onFinish )

		if self._view is not None:
			self._handle.attachToView( self._view )

		if self._var.isValid():
			self._p_onVar( self._var.value )

	def detachVariable(self):
		assert self._var is not None, 'not attached to a variable'
		self._var.valueSignal.disconnect( self._p_onVar )

		self._o_stopWatchingVarValidity( self._var )

		self._handle.startSignal.disconnect( self._p_onStart )
		self._handle.translateSignal.disconnect( self._p_onTranslate )
		self._handle.finishSignal.disconnect( self._p_onFinish )

		if self._view is not None:
			self._handle.detachFromView()
		self._var = None
		self._o_refreshSensitivity()



	def _o_setUISensitivity(self, bSensitive):
		self._handle.setSensitive( bSensitive )

	def _o_computeSensitivity(self):
		if self._var is not None:
			return self._var.isValid()
		else:
			return False


	def _p_onVar(self, value):
		if value is not None:
			self._handle.position = value
		else:
			self._handle.position = Point3()


	def _p_onStart(self, handle, viewport):
		self._initialValue = self._var.value
		self._p_commandHistoryBreak()
		self.startSignal.emit( self, viewport )

	def _p_onTranslate(self, handle, viewport, translation, constraints):
		value = self._initialValue + translation
		if self.snapping is not None:
			value = self.snapping.snapPoint3( value, constraints, viewport.pointerStatus.bControlKey, viewport.pointerStatus.bShiftKey )
		self._var.value = value
		self.manipulateSignal.emit( self, viewport )

	def _p_onFinish(self, handle, viewport):
		self._initialValue = None
		self._p_commandHistoryBreak()
		self.finishSignal.emit( self, viewport )

