##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Vector3, Quaternion, Matrix4, Axes3

from gSculpt.Util.SignalSlot import ClassSignal

from RotationHandle import RotationHandle

from gSculpt.Manipulator.Manipulator import TransformedManipulator3d




class RotationManipulator (TransformedManipulator3d):
	startSignal = ClassSignal()
	manipulateSignal = ClassSignal()
	finishSignal = ClassSignal()


	def __init__(self, snapping=None):
		super( RotationManipulator, self ).__init__()
		self._var = None
		self._handle = RotationHandle( snapping )
		self._initialValue = None

		self._o_setUISensitivity( False )


	def manipAttachToView(self, view):
		super( RotationManipulator, self ).manipAttachToView( view )
		if self._var is not None:
			self._handle.attachToView( self._view )

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		if self._var is not None:
			self._handle.detachFromView()
		super( RotationManipulator, self ).manipDetachFromView()


	def attachVariable(self, var):
		assert self._var is None, 'already attached to a variable'
		self._var = var

		self._var.valueSignal.connect( self._p_onVar )

		self._o_watchVarValidity( self._var )
		self._o_refreshSensitivity()

		self._handle.startSignal.connect( self._p_onStart )
		self._handle.rotateSignal.connect( self._p_onRotate )
		self._handle.finishSignal.connect( self._p_onFinish )

		if self._view is not None:
			self._handle.attachToView( self._view )

		if self._var.isValid():
			self._p_onVar( var.value )

	def detachVariable(self):
		assert self._var is not None, 'not attached to a variable'

		self._var.valueSignal.disconnect( self._p_onVar )

		self._o_stopWatchingVarValidity( self._var )

		self._handle.startSignal.disconnect( self._p_onStart )
		self._handle.rotateSignal.disconnect( self._p_onRotate )
		self._handle.finishSignal.disconnect( self._p_onFinish )

		if self._view is not None:
			self._handle.detachFromView()
		self._var = None
		self._o_refreshSensitivity()



	def _o_setUISensitivity(self, bSensitive):
		self._handle.setSensitive( bSensitive )


	def _o_computeSensitivity(self):
		if self._var is not None:
			return self._var.isValid()  and  self._position.isValid()  and  self._spaceRotation.isValid()
		else:
			return False


	def _o_setHandlePosition(self):
		self._handle.position = self.position

	def _o_setHandleSpaceRotation(self):
		if self._var is not None  and  self._var.value is not None:
			self._handle.spaceRotation = self.spaceRotation * self._var.value


	def _p_onStart(self, handle, viewport):
		self._initialValue = self._var.value
		self._p_commandHistoryBreak()
		self.startSignal.emit( self, viewport )

	def _p_onRotate(self, handle, viewport, axis, angle):
		q = Quaternion.rotateAxis( axis, angle )
		self._var.value = self._initialValue * q
		self.manipulateSignal.emit( self, viewport )

	def _p_onFinish(self, handle, viewport):
		self._initialValue = None
		self._p_commandHistoryBreak()
		self.finishSignal.emit( self, viewport )


	def _p_onVar(self, value):
		self._spaceRotation.refresh()



	def _p_getSnapping(self):
		return self._handle.snapping

	def _p_setSnapping(self, snapping):
		self._handle.snapping = snapping


	snapping = property( _p_getSnapping, _p_setSnapping )

