##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Vector2, Vector3

from gSculpt.Util.SignalSlot import ClassSignal

from ScaleHandle3d import Scale2Handle3d, Scale3Handle3d

from gSculpt.Manipulator.Manipulator import TransformedManipulator3d




class Scale2Manipulator3d (TransformedManipulator3d):
	startSignal = ClassSignal()
	manipulateSignal = ClassSignal()
	finishSignal = ClassSignal()


	def __init__(self, snapping=None):
		super( Scale2Manipulator3d, self ).__init__()
		self._var = None
		self._handle = Scale2Handle3d()
		self._initialValue = None
		self.snapping = snapping

		self._o_setUISensitivity( False )


	def manipAttachToView(self, view):
		super( Scale2Manipulator3d, self ).manipAttachToView( view )
		if self._var is not None:
			self._handle.attachToView( self._view )

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		if self._var is not None:
			self._handle.detachFromView()
		super( Scale2Manipulator3d, self ).manipDetachFromView()


	def attachVariable(self, var):
		assert self._var is None, 'already attached to a variable'
		self._var = var

		self._o_watchVarValidity( self._var )
		self._o_refreshSensitivity()

		self._handle.startSignal.connect( self._p_onStart )
		self._handle.scaleSignal.connect( self._p_onScale )
		self._handle.finishSignal.connect( self._p_onFinish )

		if self._view is not None:
			self._handle.attachToView( self._view )

	def detachVariable(self):
		assert self._var is not None, 'not attached to a variable'

		self._o_stopWatchingVarValidity( self._var )

		self._handle.startSignal.disconnect( self._p_onStart )
		self._handle.scaleSignal.disconnect( self._p_onScale )
		self._handle.finishSignal.disconnect( self._p_onFinish )

		if self._view is not None:
			self._handle.detachFromView()
		self._var = None
		self._o_refreshSensitivity()



	def _o_setUISensitivity(self, bSensitive):
		self._handle.setSensitive( bSensitive )


	def _o_computeSensitivity(self):
		if self._var is not None:
			return self._var.isValid()  and  self._position.isValid()  and  self._spaceRotation.isValid()
		else:
			return False


	def _o_setHandlePosition(self):
		self._handle.position = self.position

	def _o_setHandleSpaceRotation(self):
		self._handle.spaceRotation = self.spaceRotation

	def _p_onStart(self, handle, viewport):
		self._initialValue = self._var.value
		self._p_commandHistoryBreak()
		self.startSignal.emit( self, viewport )

	def _p_onScale(self, handle, viewport, scale, constraints):
		if self.snapping is not None:
			scale = self.snapping.snapVector2( scale, constraints, viewport.pointerStatus.bControlKey, viewport.pointerStatus.bShiftKey )
		self._var.value = Vector2( self._initialValue.x * scale.x,  self._initialValue.y * scale.y )
		self.manipulateSignal.emit( self, viewport )

	def _p_onFinish(self, handle, viewport):
		self._initialValue = None
		self._p_commandHistoryBreak()
		self.finishSignal.emit( self, viewport )






class Scale3Manipulator3d (TransformedManipulator3d):
	startSignal = ClassSignal()
	manipulateSignal = ClassSignal()
	finishSignal = ClassSignal()


	def __init__(self, snapping=None):
		super( Scale3Manipulator3d, self ).__init__()
		self._var = None
		self._handle = Scale3Handle3d()
		self._initialValue = None
		self.snapping = snapping

		self._o_setUISensitivity( False )


	def manipAttachToView(self, view):
		super( Scale3Manipulator3d, self ).manipAttachToView( view )
		if self._var is not None:
			self._handle.attachToView( self._view )

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		if self._var is not None:
			self._handle.detachFromView()
		super( Scale3Manipulator3d, self ).manipDetachFromView()


	def attachVariable(self, var):
		assert self._var is None, 'already attached to a variable'
		self._var = var

		self._o_watchVarValidity( self._var )
		self._o_refreshSensitivity()

		self._handle.startSignal.connect( self._p_onStart )
		self._handle.scaleSignal.connect( self._p_onScale )
		self._handle.finishSignal.connect( self._p_onFinish )

		if self._view is not None:
			self._handle.attachToView( self._view )

	def detachVariable(self):
		assert self._var is not None, 'not attached to a variable'

		self._o_stopWatchingVarValidity( self._var )

		self._handle.startSignal.disconnect( self._p_onStart )
		self._handle.scaleSignal.disconnect( self._p_onScale )
		self._handle.finishSignal.disconnect( self._p_onFinish )

		if self._view is not None:
			self._handle.detachFromView()
		self._var = None
		self._o_refreshSensitivity()



	def _o_setUISensitivity(self, bSensitive):
		self._handle.setSensitive( bSensitive )


	def _o_computeSensitivity(self):
		if self._var is not None:
			return self._var.isValid()  and  self._position.isValid()  and  self._spaceRotation.isValid()
		else:
			return False


	def _o_setHandlePosition(self):
		self._handle.position = self.position

	def _o_setHandleSpaceRotation(self):
		self._handle.spaceRotation = self.spaceRotation


	def _p_onStart(self, handle, viewport):
		self._initialValue = self._var.value
		self._p_commandHistoryBreak()
		self.startSignal.emit( self, viewport )

	def _p_onScale(self, handle, viewport, scale, constraints):
		if self.snapping is not None:
			scale = self.snapping.snapVector3( scale, constraints, viewport.pointerStatus.bControlKey, viewport.pointerStatus.bShiftKey )
		self._var.value = Vector3( self._initialValue.x * scale.x,  self._initialValue.y * scale.y,  self._initialValue.z * scale.z )
		self.manipulateSignal.emit( self, viewport )

	def _p_onFinish(self, handle, viewport):
		self._initialValue = None
		self._p_commandHistoryBreak()
		self.finishSignal.emit( self, viewport )
