##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Util.SignalSlot import ClassSignal

from gSculpt.WorkArea.Viewport import Viewport, ViewportScrollListener
from gSculpt.WorkArea.Viewport3d import Viewport3d

from gSculpt.Manipulator.Manipulator import Manipulator





class ScrollManipulator (Manipulator, ViewportScrollListener):
	manipulateSignal = ClassSignal()

	def __init__(self, lower, upper, rate):
		super( ScrollManipulator, self ).__init__()
		self._view = None
		self._var = None
		self.lower = lower
		self.upper = upper
		self.rate = rate
		self._bScrollManipulatorSensitive = True

		self._o_setUISensitivity( False )




	def manipAttachToView(self, view):
		assert self._view is None, 'already attached to a view'
		self._view = view
		if self._var is not None:
			self._view.addScrollListener( self, Viewport )

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		if self._var is not None:
			self._view.removeScrollListener( self, Viewport )
		self._view = None


	def attachVariable(self, var):
		assert self._var is None,  'variable already attached'
		self._var = var

		self._o_watchVarValidity( self._var )
		self._o_refreshSensitivity()

		if self._view is not None:
			self._view.addScrollListener( self, Viewport )

	def detachVariable(self):
		assert self._var is not None,  'no variable attached'

		self._o_stopWatchingVarValidity( self._var )

		if self._view is not None:
			self._view.removeScrollListener( self, Viewport )
		self._var = None
		self._o_refreshSensitivity()



	def _o_setUISensitivity(self, bSensitive):
		self._bScrollManipulatorSensitive = bSensitive

	def _o_computeSensitivity(self):
		if self._var is not None:
			return self._var.isValid()
		else:
			return False


	def evScroll(self, viewport, x, y):
		if self._var is not None  and  self._bScrollManipulatorSensitive:
			value = self._var.value  +  y * self.rate
			value = min( value, self.upper )
			value = max( value, self.lower )
			self._var.value = value
			self.manipulateSignal.emit( self, viewport )

