##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from OpenGL.GL import *

from gSculpt.Graphics.Graphics import Region3d

from gSculpt.Model.Model import MarkPredicate

from gSculpt.View.View import PaintLayer

from gSculpt.Manipulator.Manipulator import Manipulator




class MarkRegionManipulator (Manipulator):
	def __init__(self):
		super( MarkRegionManipulator, self ).__init__()

		self._predicateVar = None
		self._regionVar = None
		self._backfaceCullVar = None
		self._bEnabledVar = None

		self._o_setUISensitivity( False )


	def attachPredicateVar(self, predicate):
		self._predicateVar = predicate

	def attachRegionVar(self, region):
		self._regionVar = region
		self._regionVar.changedSignal.connect( self._p_onRegionChanged )

	def attachBackfaceCullVar(self, backfaceCull):
		self._backfaceCullVar = backfaceCull

	def attachEnabledVar(self, enabled):
		self._bEnabledVar = enabled

	def detachVariables(self):
		if self._regionVar is not None:
			self._regionVar.changedSignal.disconnect( self._p_onRegionChanged )
		self._predicateVar = None
		self._regionVar = None
		self._backfaceCullVar = None
		self._bEnabledVar = None


	def _o_setUISensitivity(self, bSensitive):
		pass



	def _o_createNewRegion(self, viewport):
		# Region
		if self._regionVar is not None:
			self._regionVar.value = Region3d()

		# Backface culling
		if self._backfaceCullVar is not None:
			self._backfaceCullVar.value = viewport.camera.createBackfaceCull()

		# Enable
		if self._bEnabledVar is not None:
			self._bEnabledVar.value = True


	def _o_handleKeyStatus(self, viewport):
		if self._predicateVar is not None:
			pstat = viewport.pointerStatus

			bShift = pstat.bShiftKey
			bControl = pstat.bControlKey


			if not bShift  and  not bControl:
				pred = MarkPredicate.MARK
			elif bShift  and  not bControl:
				pred = MarkPredicate.UNMARK
			elif not bShift  and  bControl:
				pred = MarkPredicate.ADD
			elif bShift  and  bControl:
				pred = MarkPredicate.FILTER

			self._predicateVar.value = pred


	def _o_paintRegion(self, viewport, paintLayer):
		if paintLayer == PaintLayer.PAINTLAYER_OVERLAY_BLEND:
			if self._regionVar is not None:
				region = self._regionVar.value

				if region is not None:
					glDepthMask( False )
					glColor4d( 0.0, 0.0, 0.3, 0.7 )
					region.paint()
					glDepthMask( True )


	def _p_onRegionChanged(self):
		self._o_viewPostRedraw()
