##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.FileIO.IOXml import *

from gSculpt.Math.Math import Axis, Vector2, Vector3, Point2, Point3, Quaternion, Plane, Colour3f

from gSculpt.Kernel.DB import Field, registerFieldType




# XML IO for enumerated types

axisIOXmlEnum = IOXmlEnum( Axis, 'Axis', Axis.AXIS_Z )
axisIOXmlEnum.value( Axis.AXIS_X, 'x' )
axisIOXmlEnum.value( Axis.AXIS_Y, 'y' )
axisIOXmlEnum.value( Axis.AXIS_Z, 'z' )
axisIOXmlEnum.value( Axis.AXIS_NEGATIVE_X, 'neg_x' )
axisIOXmlEnum.value( Axis.AXIS_NEGATIVE_Y, 'neg_y' )
axisIOXmlEnum.value( Axis.AXIS_NEGATIVE_Z, 'neg_z' )

ioXmlReadAxisProp = axisIOXmlEnum.propReader
ioXmlReadAxisNode = axisIOXmlEnum.nodeReader
ioXmlWriteAxisProp = axisIOXmlEnum.propWriter
ioXmlWriteAxisNode = axisIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'Axis', Axis )


#//////////////////////////////////////////////////////////////////////////////
#			Field types
#//////////////////////////////////////////////////////////////////////////////

class AxisField (Field):
	valueClass = Axis
	defaultValue = Axis.AXIS_Z
	xmlNodeValueReader = ioXmlReadAxisNode
	xmlNodeValueWriter = ioXmlWriteAxisNode


class Vector2Field (Field):
	valueClass = Vector2
	defaultValue = Vector2()


class Vector3Field (Field):
	valueClass = Vector3
	defaultValue = Vector3()


class Point2Field (Field):
	valueClass = Point2
	defaultValue = Point2()


class Point3Field (Field):
	valueClass = Point3
	defaultValue = Point3()


class QuaternionField (Field):
	valueClass = Quaternion
	defaultValue = Quaternion()


class PlaneField (Field):
	valueClass = Plane
	defaultValue = Plane()


class Colour3fField (Field):
	valueClass = Colour3f
	defaultValue = Colour3f()


registerFieldType( Axis, AxisField )
registerFieldType( Vector2, Vector2Field )
registerFieldType( Vector3, Vector3Field )
registerFieldType( Point2, Point2Field )
registerFieldType( Point3, Point3Field )
registerFieldType( Quaternion, QuaternionField )
registerFieldType( Plane, PlaneField )
registerFieldType( Colour3f, Colour3fField )


