##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import RowEditor
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSSceneNodeViewSettings import GSSceneNodeViewSettings

from gSculpt.Mesh.Mesh import MeshPreserveNormalSharpness, MeshLiveSubdWireframeMode, MeshPainter
from gSculpt.Mesh import MeshPreserveNormalSharpnessVar, MeshLiveSubdWireframeModeVar
from gSculpt.Mesh.MeshPreserveNormalSharpnessColumnEditor import MeshPreserveNormalSharpnessColumnEditRadioGroup
from gSculpt.Mesh.MeshLiveSubdWireframeModeColumnEditor import MeshLiveSubdWireframeModeColumnEditRadioGroup


class GSSceneNodeMeshViewSettings (GSSceneNodeViewSettings):
	painterClass = MeshPainter


	bLiveSubdivisionEnabled = FieldColumn( bool, False )
	liveSubdivisionIterations = FieldColumn( int, 1 )
	liveSubdivisionMaxFaces = FieldColumn( int, 81920 )
	liveSubdivisionNormalSharpness = FieldColumn( MeshPreserveNormalSharpness, MeshPreserveNormalSharpness.IF_SHARP )
	liveSubdivisionWireframeMode = FieldColumn( MeshLiveSubdWireframeMode, MeshLiveSubdWireframeMode.CONTROL )


	def applyToViewSettings(self, viewSettings):
		viewSettings.bMeshLiveSubdivisionEnabled = self.bLiveSubdivisionEnabled
		viewSettings.meshLiveSubdivisionIterations = self.liveSubdivisionIterations
		viewSettings.meshLiveSubdivisionMaxFaces = self.liveSubdivisionMaxFaces
		viewSettings.meshLiveSubdivisionNormalSharpness = self.liveSubdivisionNormalSharpness
		viewSettings.meshLiveSubdivisionWireframeMode = self.liveSubdivisionWireframeMode


	def _o_applyToPainter(self, painter):
		painter.setBackgroundMeshViewSettings( self.bLiveSubdivisionEnabled, self.liveSubdivisionIterations, self.liveSubdivisionMaxFaces,
										self.liveSubdivisionNormalSharpness, self.liveSubdivisionWireframeMode )



class GSSceneNodeMeshViewSettingsEditor (RowEditor):
	row = RowRefColumn( GSSceneNodeMeshViewSettings )

	rowClass = GSSceneNodeMeshViewSettings

	bLiveSubdEnabledCheck = BoolColumnEditCheck( row.bLiveSubdivisionEnabled, _( 'Enabled' ) )
	liveSubdIterationsEntry = IntColumnEditEntry( row.liveSubdivisionIterations, 0, 16, 1, 3 )
	liveSubdMaxFacesEntry = IntColumnEditEntry( row.liveSubdivisionMaxFaces, 1, 1 << 30, 128, 4096 )
	liveSubdNormalSharpnessRadio = MeshPreserveNormalSharpnessColumnEditRadioGroup( row.liveSubdivisionNormalSharpness )
	liveSubdWireframeModeRadio = MeshLiveSubdWireframeModeColumnEditRadioGroup( row.liveSubdivisionWireframeMode )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Mesh view settings' )
	layout.row()
	layout.row()  <<  _( 'Live subdivision' )
	layout.row()  <<  bLiveSubdEnabledCheck.check
	layout.row()  <<  _( 'Iterations:' )  <<  liveSubdIterationsEntry.entry  <<  None  <<  None
	layout.row()  <<  _( 'Max. faces:' )  <<  liveSubdMaxFacesEntry.entry  <<  None  <<  None
	layout.row()  <<  _( 'Normal sharpness:' )  <<  liveSubdNormalSharpnessRadio.preserve  <<  liveSubdNormalSharpnessRadio.ifSharp  <<  liveSubdNormalSharpnessRadio.smooth
	layout.row()  <<  _( 'Wireframe mode:' )  <<  liveSubdWireframeModeRadio.none  <<  liveSubdWireframeModeRadio.control  <<  liveSubdWireframeModeRadio.full
	layout.finalise()
