##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.WorkArea.Viewport import ViewportEventHandler

from gSculpt.Mesh import GSProductMesh


class MeshManipulatorHelper (ViewportEventHandler):
	def __init__(self):
		super( MeshManipulatorHelper, self ).__init__()
		self._meshManipHelperView = None
		self._meshSource = None
		self._currentViewport = None
		self._bSensitive = True


	def attachView(self, view):
		assert self._meshManipHelperView is None, 'view already attached'
		self._meshManipHelperView = view

	def detachView(self):
		assert self._meshManipHelperView is not None, 'no view attached'
		self._meshManipHelperView = None


	def attachMeshSource(self, source):
		assert self._meshSource is None, 'mesh source already attached'
		self._meshSource = source
		self._meshSource.changedSignal.connect( self._p_onSourceValueChanged )
		self._bDirty = True

	def detachMeshSource(self):
		assert self._meshSource is not None, 'no mesh source attached'
		self._meshSource.changedSignal.disconnect( self._p_onSourceValueChanged )
		self._meshSource = None
		self._bDirty = False
		self._bPickValid = False


	def _o_setMeshManipulatorHelperSensitivity(self, bSensitive):
		bSensitivityChanged = bSensitive != self._bSensitive
		self._bSensitive = bSensitive
		if self._meshManipHelperView is not None  and  bSensitivityChanged:
			self._meshManipHelperView.postRedraw()



	def _o_getMesh(self):
		if self._meshSource is not None:
			return self._meshSource.getValueAsClass( GSProductMesh )
		else:
			return None

	def _o_reset(self):
		pass


	def _p_onSourceValueChanged(self):
		if self._meshManipHelperView is not None:
			self._o_reset()
