##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Colour3f

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.WorkArea.Viewport3d import Viewport3d

from gSculpt.Manipulator.Manipulator import Manipulator

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import GSProcedure

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model.TargetList import TargetList
from gSculpt.Model.TargetListColumnEditor import TargetListColumnEditor
from gSculpt.Model import ModelDraw

from gSculpt.Mesh.Mesh import GSProductMesh
from gSculpt.Mesh.MeshPickHelper import MeshPickHelper, MeshPickSeededEdgeListHelper





class MeshMarkPickManipulatorBase (Manipulator):
	def __init__(self):
		super( MeshMarkPickManipulatorBase, self ).__init__()
		self._var = None

		self._o_setUISensitivity( False )


	def manipAttachToView(self, view):
		super( MeshMarkPickManipulatorBase, self ).manipAttachToView( view )
		self.attachView( view )
		self._view.addEventHandler( self, Viewport3d )		# HACK: need to handle other viewport types
		self._view.addPainter( self, Viewport3d )
		self._view.postRedraw()

	def manipDetachFromView(self):
		self._view.removeEventHandler( self, Viewport3d )
		self._view.removePainter( self, Viewport3d )
		self._view.postRedraw()
		self.detachView()
		super( MeshMarkPickManipulatorBase, self ).manipDetachFromView()


	def attachVariable(self, var):
		assert self._var is None, 'already attached to a variable'
		self._var = var
		self._o_watchVarValidity( self._var )
		self._o_refreshSensitivity()

	def detachVariable(self):
		assert self._var is not None, 'not attached to a variable'
		self._o_stopWatchingVarValidity( self._var )
		self._var = None
		self._o_refreshSensitivity()


	def _o_computeSensitivity(self):
		if self._var is not None:
			return self._var.isValid()
		else:
			return False




class MeshMarkPickManipulator (MeshMarkPickManipulatorBase, MeshPickHelper):
	def _o_setUISensitivity(self, bSensitive):
		self._o_setMeshPickHelperSensitivity( bSensitive )


	def _p_commitPick(self, viewport, pick):
		"Protected - call to commit a pick"
		assert self._var is not None, 'no variable attached'
		self._var.value = self._var.value + [ pick ]
		self._p_commandHistoryBreak()




class MeshMarkPickEdgeListManipulator (MeshMarkPickManipulatorBase, MeshPickSeededEdgeListHelper):
	def __init__(self):
		super( MeshMarkPickEdgeListManipulator, self ).__init__()
		self._bMarked = False


	def _o_setUISensitivity(self, bSensitive):
		self._o_setMeshPickHelperSensitivity( bSensitive )


	def _p_seedChosen(self, mesh, seedEdgeIndex):
		self._bMarked = mesh.isEdgeMarked( seedEdgeIndex )

	def _p_paintValidHighlight(self, viewport, paintLayer, segmentList, seedSegment):
		for segment in segmentList:
			if self._bMarked:
				ModelDraw.drawEdgePickHighlight( segment, ModelDraw.MARKED_TARGET_HIGHLIGHT_COLOUR, paintLayer )
			else:
				ModelDraw.drawEdgePickHighlight( segment, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer )

	def _p_paintInvalidHighlight(self, viewport, paintLayer, seedSegment):
		ModelDraw.drawEdgePickHighlight( seedSegment, ModelDraw.INVALID_TARGET_HIGHLIGHT_COLOUR, paintLayer )


	def _p_commitPick(self, viewport, pick):
		"Protected - call to commit a pick"
		assert self._var is not None, 'no variable attached'
		self._var.value = self._var.value + [ pick ]
		self._p_commandHistoryBreak()




class ProcMeshMarkPick (GSProcedure):
	description = _( 'Abstract mesh pick' )

	targetList = FieldColumn( TargetList, TargetList() )


	inputClass = GSProductMesh



class MeshMarkPickAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshMarkPick

	def getResult(self):
		return self._inputObject



class ToolMeshMarkPick (GSProcedureTool):
	row = RowRefColumn( ProcMeshMarkPick )

	functionClass = ProcMeshMarkPick
	adjustableInvoker = MeshMarkPickAdjustableInvoker
	pageTitle = _( 'Pick' )
	titleText = _( 'Pick' )


	targetListEditor = TargetListColumnEditor( row.targetList )


	layout = FormLayoutFactory()
	layout.row()  <<  targetListEditor.numTargets
	layout.row()  <<  None  <<  ( targetListEditor.removeTarget, 2 )  <<  None
	layout.row()
	layout.finalise()





