##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Colour3f

from gSculpt.Kernel.DB import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model.Model import pyMPickList_to_MPickList, MPickList

from gSculpt.MeshTools.Mark.ToolMeshMarkPick import MeshMarkPickEdgeListManipulator, ProcMeshMarkPick, MeshMarkPickAdjustableInvoker, ToolMeshMarkPick



class MeshMarkPickEdgeBoundariesManipulator (MeshMarkPickEdgeListManipulator):
	def _p_discoverEdgeList(self, mesh, seedEdgeIndex):
		if mesh.isEdgeBoundary( seedEdgeIndex ):
			discoveredEdgeIndices = mesh.discoverEdgeBoundary( seedEdgeIndex )
			discoveredSegments = self._o_edgeIndexListToSegmentList( mesh, discoveredEdgeIndices )
			return True, discoveredEdgeIndices, discoveredSegments
		else:
			return False, None, None


	def _p_pickConfirmed(self, viewport, pick):
		self._p_commitPick( viewport, pick )






class MeshMarkPickEdgeBoundariesColumnManipulator (GSProcedureTool.GSProcedureToolColumnManipulator):
	def __init__(self, fieldColumn):
		super( MeshMarkPickEdgeBoundariesColumnManipulator, self ).__init__()
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshMarkPickEdgeBoundariesManipulator()

	def initialiseElement(self, element, rowEditContext):
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		element.detachVariable()
		element.commandHistory = None


	def installElementEventHandlers(self, element, rowEditContext):
		element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		element.manipDetachFromView()


	def attachOutputProductSource(self, element, source):
		element.attachMeshSource( source )

	def detachOutputProductSource(self, element, source):
		element.detachMeshSource()




class ProcMeshMarkPickEdgeBoundaries (ProcMeshMarkPick):
	description = _( 'Pick edge boundaries' )

	def procedureInvoke(self, inputObject, errorHandler):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self.targetList ], mpicks )
		inputObject.markEdgeBoundaries_pick( mpicks )
		return inputObject



class MeshMarkPickEdgeBoundariesAdjustableInvoker (MeshMarkPickAdjustableInvoker):
	functionClass = ProcMeshMarkPickEdgeBoundaries

	def save(self):
		self._inputObject.markEdges_save()

	def restore(self):
		self._inputObject.markEdges_restore()

	def adjust(self):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self._proc.targetList ], mpicks )
		self._inputObject.markEdgeBoundaries_pick( mpicks )



class ToolMeshMarkPickEdgeBoundaries (ToolMeshMarkPick):
	row = RowRefColumn( ProcMeshMarkPickEdgeBoundaries )

	functionClass = ProcMeshMarkPickEdgeBoundaries
	adjustableInvoker = MeshMarkPickEdgeBoundariesAdjustableInvoker
	pageTitle = _( 'Pk.Rng.' )
	titleText = _( 'Pick Edge Boundaries' )

	targetListManip = MeshMarkPickEdgeBoundariesColumnManipulator( row.targetList )
	targetListManip.enable()



