##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Colour3f

from gSculpt.Kernel.DB import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model import ModelDraw
from gSculpt.Model.Model import pyMPickList_to_MPickList, MPickList

from gSculpt.MeshTools.Mark.ToolMeshMarkPick import MeshMarkPickManipulator, ProcMeshMarkPick, MeshMarkPickAdjustableInvoker, ToolMeshMarkPick



class MeshMarkPickEdgesManipulator (MeshMarkPickManipulator):
	def __init__(self):
		super( MeshMarkPickEdgesManipulator, self ).__init__()
		self._bMarked = False
		self._edgeSegment = None


	def _p_refreshHit(self, viewport, mesh, pick):
		edgeIndex, closestPoint, thruFaceIndex = mesh.pickEdge( pick.toMPick() )

		if edgeIndex != -1:
			self._edgeSegment = mesh.getEdgeSegment( edgeIndex )
			self._bMarked = mesh.isEdgeMarked( edgeIndex )
			return True
		else:
			self._edgeSegment = None
			self._bMarked = False
			return False


	def _p_paintHighlight(self, viewport, paintLayer):
		if self._bMarked:
			ModelDraw.drawEdgePickHighlight( self._edgeSegment, ModelDraw.MARKED_TARGET_HIGHLIGHT_COLOUR, paintLayer );
		else:
			ModelDraw.drawEdgePickHighlight( self._edgeSegment, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer );


	def _p_pickConfirmed(self, viewport, pick):
		self._p_commitPick( viewport, pick )



class MeshMarkPickEdgesColumnManipulator (GSProcedureTool.GSProcedureToolColumnManipulator):
	def __init__(self, fieldColumn):
		super( MeshMarkPickEdgesColumnManipulator, self ).__init__()
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshMarkPickEdgesManipulator()

	def initialiseElement(self, element, rowEditContext):
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		element.detachVariable()
		element.commandHistory = None


	def installElementEventHandlers(self, element, rowEditContext):
		element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		element.manipDetachFromView()


	def attachOutputProductSource(self, element, source):
		element.attachMeshSource( source )

	def detachOutputProductSource(self, element, source):
		element.detachMeshSource()




class ProcMeshMarkPickEdges (ProcMeshMarkPick):
	description = _( 'Pick edges' )

	def procedureInvoke(self, inputObject, errorHandler):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self.targetList ], mpicks )
		inputObject.markEdges_pick( mpicks )
		return inputObject



class MeshMarkPickEdgesAdjustableInvoker (MeshMarkPickAdjustableInvoker):
	functionClass = ProcMeshMarkPickEdges

	def save(self):
		self._inputObject.markEdges_save()

	def restore(self):
		self._inputObject.markEdges_restore()

	def adjust(self):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self._proc.targetList ], mpicks )
		self._inputObject.markEdges_pick( mpicks )



class ToolMeshMarkPickEdges (ToolMeshMarkPick):
	row = RowRefColumn( ProcMeshMarkPickEdges )

	functionClass = ProcMeshMarkPickEdges
	adjustableInvoker = MeshMarkPickEdgesAdjustableInvoker
	pageTitle = _( 'Pick' )
	titleText = _( 'Pick Edges' )

	targetListManip = MeshMarkPickEdgesColumnManipulator( row.targetList )
	targetListManip.enable()



