##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Colour3f

from gSculpt.Kernel.DB import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model import ModelDraw
from gSculpt.Model.Model import pyMPickList_to_MPickList, MPickList

from gSculpt.MeshTools.Mark.ToolMeshMarkPick import MeshMarkPickManipulator, ProcMeshMarkPick, MeshMarkPickAdjustableInvoker, ToolMeshMarkPick



class MeshMarkPickFacesManipulator (MeshMarkPickManipulator):
	def __init__(self):
		super( MeshMarkPickFacesManipulator, self ).__init__()
		self._bMarked = False
		self._faceTriangles = []


	def _p_refreshHit(self, viewport, mesh, pick):
		faceIndex, intersectionPoint = mesh.pickFace( pick.toMPick() )

		if faceIndex != -1:
			self._faceTriangles = mesh.getFaceTriangles( faceIndex )
			self._bMarked = mesh.isFaceMarked( faceIndex )
			return True
		else:
			self._faceTriangles = []
			self._bMarked = False
			return False


	def _p_paintHighlight(self, viewport, paintLayer):
		if self._bMarked:
			ModelDraw.drawFacePickHighlight( self._faceTriangles, ModelDraw.MARKED_TARGET_HIGHLIGHT_COLOUR, paintLayer );
		else:
			ModelDraw.drawFacePickHighlight( self._faceTriangles, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer );


	def _p_pickConfirmed(self, viewport, pick):
		self._p_commitPick( viewport, pick )



class MeshMarkPickFacesColumnManipulator (GSProcedureTool.GSProcedureToolColumnManipulator):
	def __init__(self, fieldColumn):
		super( MeshMarkPickFacesColumnManipulator, self ).__init__()
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshMarkPickFacesManipulator()

	def initialiseElement(self, element, rowEditContext):
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		element.detachVariable()
		element.commandHistory = None


	def installElementEventHandlers(self, element, rowEditContext):
		element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		element.manipDetachFromView()


	def attachOutputProductSource(self, element, source):
		element.attachMeshSource( source )

	def detachOutputProductSource(self, element, source):
		element.detachMeshSource()




class ProcMeshMarkPickFaces (ProcMeshMarkPick):
	description = _( 'Pick faces' )

	def procedureInvoke(self, inputObject, errorHandler):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self.targetList ], mpicks )
		inputObject.markFaces_pick( mpicks )
		return inputObject



class MeshMarkPickFacesAdjustableInvoker (MeshMarkPickAdjustableInvoker):
	functionClass = ProcMeshMarkPickFaces

	def save(self):
		self._inputObject.markFaces_save()

	def restore(self):
		self._inputObject.markFaces_restore()

	def adjust(self):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self._proc.targetList ], mpicks )
		self._inputObject.markFaces_pick( mpicks )



class ToolMeshMarkPickFaces (ToolMeshMarkPick):
	row = RowRefColumn( ProcMeshMarkPickFaces )

	functionClass = ProcMeshMarkPickFaces
	adjustableInvoker = MeshMarkPickFacesAdjustableInvoker
	pageTitle = _( 'Pick' )
	titleText = _( 'Pick Faces' )

	targetListManip = MeshMarkPickFacesColumnManipulator( row.targetList )
	targetListManip.enable()



