##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Colour3f

from gSculpt.Kernel.DB import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model import ModelDraw
from gSculpt.Model.Model import pyMPickList_to_MPickList, MPickList

from gSculpt.MeshTools.Mark.ToolMeshMarkPick import MeshMarkPickManipulator, ProcMeshMarkPick, MeshMarkPickAdjustableInvoker, ToolMeshMarkPick



class MeshMarkPickVerticesManipulator (MeshMarkPickManipulator):
	def __init__(self):
		super( MeshMarkPickVerticesManipulator, self ).__init__()
		self._bMarked = False
		self._vertexPosition = None


	def _p_refreshHit(self, viewport, mesh, pick):
		vertexIndex, thruFaceIndex = mesh.pickVertex( pick.toMPick() )

		if vertexIndex != -1:
			self._vertexPosition = mesh.getVertexPosition( vertexIndex )
			self._bMarked = mesh.isVertexMarked( vertexIndex )
			return True
		else:
			self._vertexPosition = None
			self._bMarked = False
			return False


	def _p_paintHighlight(self, viewport, paintLayer):
		if self._bMarked:
			ModelDraw.drawVertexPickHighlight( self._vertexPosition, ModelDraw.MARKED_TARGET_HIGHLIGHT_COLOUR, paintLayer );
		else:
			ModelDraw.drawVertexPickHighlight( self._vertexPosition, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer );


	def _p_pickConfirmed(self, viewport, pick):
		self._p_commitPick( viewport, pick )



class MeshMarkPickVerticesColumnManipulator (GSProcedureTool.GSProcedureToolColumnManipulator):
	def __init__(self, fieldColumn):
		super( MeshMarkPickVerticesColumnManipulator, self ).__init__()
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshMarkPickVerticesManipulator()

	def initialiseElement(self, element, rowEditContext):
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		element.detachVariable()
		element.commandHistory = None


	def installElementEventHandlers(self, element, rowEditContext):
		element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		element.manipDetachFromView()


	def attachOutputProductSource(self, element, source):
		element.attachMeshSource( source )

	def detachOutputProductSource(self, element, source):
		element.detachMeshSource()




class ProcMeshMarkPickVertices (ProcMeshMarkPick):
	description = _( 'Pick vertices' )

	def procedureInvoke(self, inputObject, errorHandler):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self.targetList ], mpicks )
		inputObject.markVertices_pick( mpicks )
		return inputObject



class MeshMarkPickVerticesAdjustableInvoker (MeshMarkPickAdjustableInvoker):
	functionClass = ProcMeshMarkPickVertices

	def save(self):
		self._inputObject.markVertices_save()

	def restore(self):
		self._inputObject.markVertices_restore()

	def adjust(self):
		mpicks = MPickList()
		pyMPickList_to_MPickList( [ pick.toMPick()  for  pick in self._proc.targetList ], mpicks )
		self._inputObject.markVertices_pick( mpicks )



class ToolMeshMarkPickVertices (ToolMeshMarkPick):
	row = RowRefColumn( ProcMeshMarkPickVertices )

	functionClass = ProcMeshMarkPickVertices
	adjustableInvoker = MeshMarkPickVerticesAdjustableInvoker
	pageTitle = _( 'Pick' )
	titleText = _( 'Pick Vertices' )

	targetListManip = MeshMarkPickVerticesColumnManipulator( row.targetList )
	targetListManip.enable()



