##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Vector3

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *
from gSculpt.Kernel.Enum import Enum

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Mesh.Mesh import GSProductMesh, MeshPreserveNormalSharpness
from gSculpt.Mesh.MeshPreserveNormalSharpnessColumnEditor import MeshPreserveNormalSharpnessColumnEditRadioGroup






# MeshSubdivisionScheme enumeration
class MeshSubdivisionScheme (Enum):
	LINEAR = 0
	CATMULL_CLARK = 1




# XML IO for MeshSubdivisionScheme

subdivSchemeIOXmlEnum = IOXmlEnum( MeshSubdivisionScheme, 'MeshSubdivisionScheme', MeshSubdivisionScheme.CATMULL_CLARK )
subdivSchemeIOXmlEnum.value( MeshSubdivisionScheme.LINEAR, 'linear' )
subdivSchemeIOXmlEnum.value( MeshSubdivisionScheme.CATMULL_CLARK, 'catmull_clark' )

ioXmlReadMeshSubdivisionSchemeProp = subdivSchemeIOXmlEnum.propReader
ioXmlReadMeshSubdivisionSchemeNode = subdivSchemeIOXmlEnum.nodeReader
ioXmlWriteMeshSubdivisionSchemeProp = subdivSchemeIOXmlEnum.propWriter
ioXmlWriteMeshSubdivisionSchemeNode = subdivSchemeIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshSubdivisionScheme', MeshSubdivisionScheme )


# Field for MeshSubdivisionScheme

class MeshSubdivisionSchemeField (Field):
	valueClass = MeshSubdivisionScheme
	defaultValue = MeshSubdivisionScheme.CATMULL_CLARK
	xmlNodeValueReader = ioXmlReadMeshSubdivisionSchemeNode
	xmlNodeValueWriter = ioXmlWriteMeshSubdivisionSchemeNode

registerFieldType( MeshSubdivisionScheme, MeshSubdivisionSchemeField )


# Variable editor for MeshSubdivisionScheme

class MeshSubdivisionSchemeVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshSubdivisionSchemeVarEditRadioGroup, self ).__init__()
		self.linear = self.addChoice( MeshSubdivisionScheme.LINEAR, _( 'Linear' ) )
		self.catmullClark = self.addChoice( MeshSubdivisionScheme.CATMULL_CLARK, _( 'Catmull-Clark' ) )


# Column editor for MeshSubdivisionScheme

class MeshSubdivisionSchemeColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshSubdivisionSchemeColumnEditRadioGroup, self ).__init__( [ 'linear', 'catmullClark' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshSubdivisionSchemeVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshSubdivisionSchemeColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshSubdivisionSchemeColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()






class ProcMeshSubdivide (GSProcedure):
	description = _( 'Subdivide' )

	bMarkedFacesOnly = FieldColumn( bool, False )
	scheme = FieldColumn( MeshSubdivisionScheme, MeshSubdivisionScheme.CATMULL_CLARK )
	preserveNormalSharpness = FieldColumn( MeshPreserveNormalSharpness, MeshPreserveNormalSharpness.IF_SHARP )

	inputClass = GSProductMesh
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		bCatmullClark = self.scheme == MeshSubdivisionScheme.CATMULL_CLARK
		inputObject.subdivide( self.bMarkedFacesOnly, bCatmullClark, self.preserveNormalSharpness )
		return inputObject




class ToolMeshSubdivide (GSProcedureTool):
	row = RowRefColumn( ProcMeshSubdivide )

	functionClass = ProcMeshSubdivide
	pageTitle = _( 'Subd.' )
	titleText = _( 'Subdivide' )

	bMarkedFacesOnlyCheck = BoolColumnEditCheck( row.bMarkedFacesOnly, _( 'Marked faces only' ) )
	schemeRadio = MeshSubdivisionSchemeColumnEditRadioGroup( row.scheme )
	preserveSharpnessRadio = MeshPreserveNormalSharpnessColumnEditRadioGroup( row.preserveNormalSharpness )

	layout = FormLayoutFactory()
	layout.row()  <<  bMarkedFacesOnlyCheck.check
	layout.row()
	layout.row()  <<  _( 'Scheme:' )
	layout.row()  <<  schemeRadio.linear
	layout.row()  <<  schemeRadio.catmullClark
	layout.row()
	layout.row()  <<  _( 'Preserve normal sharpness:' )
	layout.row()  <<  preserveSharpnessRadio.preserve
	layout.row()  <<  preserveSharpnessRadio.ifSharp
	layout.row()  <<  preserveSharpnessRadio.smooth
	layout.row()
	layout.finalise()
