##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.PolyBlend.PolyBlendComponent import *

from gSculpt.Manipulator.ColumnManipulators import IntColumnScrollManipulator

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import GSProductMesh



class ProcMeshBridge (GSProcedure):
	description = _( 'Bridge' )

	blend = CompositeColumn( PolyBlendComponent )
	segments = FieldColumn( int, 1 )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		inputObject.bridge( self.blend.makePolyBlend(), self.segments )
		return inputObject



class ToolMeshBridge (GSProcedureTool):
	row = RowRefColumn( ProcMeshBridge )

	functionClass = ProcMeshBridge
	pageTitle = _( 'Bridge' )
	titleText = _( 'Bridge' )

	blendEdit = PolyBlendComponentColumnEdit( row.blend )
	numSegmentsEntry = IntColumnEditEntry( row.segments, 1, 1048576, 1, 10 )

	numSegmentsManip = IntColumnScrollManipulator( row.segments, 1, 1048576 )
	numSegmentsManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Use the scroll wheel to' )
	layout.row()  <<  _( 'change the number of segments' )
	layout.row()
	layout.block( 4 )  <<  blendEdit.rowEditor
	layout.row()
	layout.row()  <<  _( 'Segments:' )  <<  numSegmentsEntry.entry
	layout.row()
	layout.finalise()
