##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.Math.Math import Vector3

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshDissolveEdgesStyle, GSProductMesh

from gSculpt.Manipulator.RealManipulator import RealManipulator



# XML IO for MeshDissolveEdgesStyle

dissolveStyleIOXmlEnum = IOXmlEnum( MeshDissolveEdgesStyle, 'MeshDissolveEdgesStyle', MeshDissolveEdgesStyle.MIDPOINT_MAT_A )
dissolveStyleIOXmlEnum.value( MeshDissolveEdgesStyle.A, 'a' )
dissolveStyleIOXmlEnum.value( MeshDissolveEdgesStyle.B, 'b' )
dissolveStyleIOXmlEnum.value( MeshDissolveEdgesStyle.MIDPOINT_MAT_A, 'midpoint_mat_a' )
dissolveStyleIOXmlEnum.value( MeshDissolveEdgesStyle.MIDPOINT_MAT_B, 'midpoint_mat_b' )

ioXmlReadMeshDissolveEdgesStyleProp = dissolveStyleIOXmlEnum.propReader
ioXmlReadMeshDissolveEdgesStyleNode = dissolveStyleIOXmlEnum.nodeReader
ioXmlWriteMeshDissolveEdgesStyleProp = dissolveStyleIOXmlEnum.propWriter
ioXmlWriteMeshDissolveEdgesStyleNode = dissolveStyleIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshDissolveEdgesStyle', MeshDissolveEdgesStyle )


# Field for MeshDissolveEdgesStyle

class MeshDissolveEdgesStyleField (Field):
	valueClass = MeshDissolveEdgesStyle
	defaultValue = MeshDissolveEdgesStyle.MIDPOINT_MAT_A
	xmlNodeValueReader = ioXmlReadMeshDissolveEdgesStyleNode
	xmlNodeValueWriter = ioXmlWriteMeshDissolveEdgesStyleNode

registerFieldType( MeshDissolveEdgesStyle, MeshDissolveEdgesStyleField )


# Variable editor for MeshDissolveEdgesStyle

class MeshDissolveEdgesStyleVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshDissolveEdgesStyleVarEditRadioGroup, self ).__init__()
		self.a = self.addChoice( MeshDissolveEdgesStyle.A, 'A' )
		self.b = self.addChoice( MeshDissolveEdgesStyle.B, 'B' )
		self.midPointMatA = self.addChoice( MeshDissolveEdgesStyle.MIDPOINT_MAT_A, _( 'Mid-point, material A' ) )
		self.midPointMatB = self.addChoice( MeshDissolveEdgesStyle.MIDPOINT_MAT_B, _( 'Mid-point, material B' ) )


# Column editor MeshDissolveEdgesStyle

class MeshDissolveEdgesStyleColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshDissolveEdgesStyleColumnEditRadioGroup, self ).__init__( [ 'a', 'b', 'midPointMatA', 'midPointMatB' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshDissolveEdgesStyleVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshDissolveEdgesStyleColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshDissolveEdgesStyleColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()







class ProcMeshDissolveEdges (GSProcedure):
	description = _( 'Dissolve edges' )

	style = FieldColumn( MeshDissolveEdgesStyle, MeshDissolveEdgesStyle.MIDPOINT_MAT_A )
	bDissolveSimpleValenceTwoVertices = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		inputObject.dissolveMarkedEdges( self.style, self.bDissolveSimpleValenceTwoVertices )
		return inputObject



class ToolMeshDissolveEdges (GSProcedureTool):
	row = RowRefColumn( ProcMeshDissolveEdges )

	functionClass = ProcMeshDissolveEdges
	pageTitle = _( 'Dissolve' )
	titleText = _( 'Dissolve Edges' )

	styleRadio = MeshDissolveEdgesStyleColumnEditRadioGroup( row.style )
	bDissolveSimpleValenceTwoVerticesCheck = BoolColumnEditCheck( row.bDissolveSimpleValenceTwoVertices, _( 'Clean dissolve' ) )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Style:' )
	layout.row()  <<  styleRadio.a
	layout.row()  <<  styleRadio.b
	layout.row()  <<  styleRadio.midPointMatA
	layout.row()  <<  styleRadio.midPointMatB
	layout.row()
	layout.row()  <<  bDissolveSimpleValenceTwoVerticesCheck.check
	layout.row()
	layout.finalise()
