##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.RealManipulator import RealManipulator
from gSculpt.Manipulator.ColumnManipulators import RealColumnManipulator, Real2ColumnManipulator, ColumnManipulatorSnappingDistance

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshExpandEdgesAdjuster, GSProductMesh




class ProcMeshExpandEdges (GSProcedure):
	description = _( 'Expand edges' )

	expand = FieldColumn( float, 0.0 )
	extrude = FieldColumn( float, 0.0 )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )
	bUnmarkExtrudedEdges = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshExpandEdgesAdjuster()
		inputObject.expandMarkedEdges( adjuster, self.bMarkBoundaryEdges, self.bNSharpBoundaryEdges, self.bUnmarkExtrudedEdges )
		adjuster.setParameters( self.expand, self.extrude )
		return inputObject



class ExpandEdgesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshExpandEdges

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshExpandEdgesAdjuster()
		self.mesh.expandMarkedEdges( self.adjuster, self._proc.bMarkBoundaryEdges, self._proc.bNSharpBoundaryEdges, self._proc.bUnmarkExtrudedEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.expand, self._proc.extrude )

	def getResult(self):
		return self.mesh




class ToolMeshExpandEdges (GSProcedureTool):
	row = RowRefColumn( ProcMeshExpandEdges )

	functionClass = ProcMeshExpandEdges
	adjustableInvoker = ExpandEdgesAdjustableInvoker
	pageTitle = _( 'Expand' )
	titleText = _( 'Expand Edges' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedEdgesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	expandEntry = FloatColumnEditEntry( row.expand, 0.0, 1.0e8, 0.1, 10.0, 4 )
	extrudeEntry = FloatColumnEditEntry( row.extrude, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'N-sharp boundary edges' ) )
	bUnmarkExtrudedEdgesCheck = BoolColumnEditCheck( row.bUnmarkExtrudedEdges, _( 'Unmark extruded edges' ) )

	expandManip = Real2ColumnManipulator( row.expand, 0.0, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									row.extrude, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									xSnapping=ColumnManipulatorSnappingDistance(), ySnapping=ColumnManipulatorSnappingDistance() )
	expandManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Expand:' )  <<  expandEntry.entry
	layout.row()  <<  _( 'Extrude:' )  <<  extrudeEntry.entry
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()  <<  bUnmarkExtrudedEdgesCheck.check
	layout.row()
	layout.finalise()
