##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.Math.Math import Vector2, Vector3

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.RealManipulator import RealManipulator
from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshExtrudeEdgesAdjuster, GSProductMesh





class ProcMeshExtrudeEdges (GSProcedure):
	__version__ = '1'
	description = _( 'Extrude edges' )

	extrude = FieldColumn( Vector3, Vector3() )
	segments = FieldColumn( int, 1 )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshExtrudeEdgesAdjuster()
		inputObject.extrudeMarkedEdges( adjuster, self.segments, False, self.getMarkExtrudedEdges() )
		adjuster.setParameters( self.extrude, Vector2( 0.0, 1.0 ) )
		return inputObject


	def getMarkExtrudedEdges(self):
		try:
			v = self.__version__
		except AttributeError:
			v = None

		if v is None:
			return False
		elif v == '1':
			return True
		else:
			return True


class ExtrudeEdgesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshExtrudeEdges

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshExtrudeEdgesAdjuster()
		self.mesh.extrudeMarkedEdges( self.adjuster, self._proc.segments, False, self._proc.getMarkExtrudedEdges() )

	def adjust(self):
		self.adjuster.setParameters( self._proc.extrude, Vector2( 0.0, 1.0 ) )

	def getResult(self):
		return self.mesh




class ToolMeshExtrudeEdges (GSProcedureTool):
	row = RowRefColumn( ProcMeshExtrudeEdges )

	functionClass = ProcMeshExtrudeEdges
	adjustableInvoker = ExtrudeEdgesAdjustableInvoker
	pageTitle = _( 'Extrude' )
	titleText = _( 'Extrude Edges' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedEdgesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	extrudeEntry = Vector3ColumnEditEntry( row.extrude, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	segmentsEntry = IntColumnEditEntry( row.segments, 1, 1048576, 1, 10 )


	extrudeManip = Translation3ColumnManipulator3d( row.extrude, focalPoint, snapping=ColumnManipulatorSnappingDistance() )
	extrudeManip.enable()

	numSegmentsManip = IntColumnScrollManipulator( row.segments, 1, 1048576 )
	numSegmentsManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Use the scroll wheel to' )
	layout.row()  <<  _( 'change the number of segments' )
	layout.row()
	layout.row()  <<  _( 'Extrude:' )
	layout.row()  <<  'X:'  <<  extrudeEntry.x
	layout.row()  <<  'Y:'  <<  extrudeEntry.y
	layout.row()  <<  'Z:'  <<  extrudeEntry.z
	layout.row()
	layout.row()  <<  _( 'Segments:' )  <<  segmentsEntry.entry
	layout.row()

	layout.finalise()
