##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import GSProductMesh



class ProcMeshNSplitEdges (GSProcedure):
	description = _( 'N-Split edges' )

	numSplits = FieldColumn( int, 1 )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		inputObject.nSplitMarkedEdges( self.numSplits + 1 )
		return inputObject



class ToolMeshNSplitEdges (GSProcedureTool):
	row = RowRefColumn( ProcMeshNSplitEdges )

	functionClass = ProcMeshNSplitEdges
	pageTitle = _( 'N-Split' )
	titleText = _( 'N-Split Edges' )

	numSplitsEntry = IntColumnEditEntry( row.numSplits, 1, 1048576, 1, 10 )

	numSplitsManip = IntColumnScrollManipulator( row.numSplits, 1, 1048576 )
	numSplitsManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Use the scroll wheel to' )
	layout.row()  <<  _( 'change the number of vertices' )
	layout.row()  <<  _( 'inserted into the marked edges' )
	layout.row()
	layout.row()  <<  _( 'Splits:' )  <<  numSplitsEntry.entry
	layout.row()
	layout.finalise()
