##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.Math.Math import Vector3, Matrix4

from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import GSProductMesh, MeshTransformation

from gSculpt.MeshTools.MeshEdit.ToolMeshEditScale import *




class ProcMeshScaleEdges (ProcMeshEditScale):
	description = _( 'Scale edges' )

	def procedureInvoke(self, inputObject, errorHandler):
		transformation = MeshTransformation()
		inputObject.initEdgeTransformation( transformation, self.bPerGroup )
		transformation.centredTransform( Matrix4.scale( self.scale ) )
		return inputObject



class ScaleEdgesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshScaleEdges

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.transformation = MeshTransformation()
		self.mesh.initEdgeTransformation( self.transformation, self._proc.bPerGroup )

	def adjust(self):
		self.transformation.centredTransform( Matrix4.scale( self._proc.scale ) )

	def getResult(self):
		return self.mesh




class ToolMeshScaleEdges (ToolMeshEditScale):
	row = RowRefColumn( ProcMeshScaleEdges )

	functionClass = ProcMeshScaleEdges
	adjustableInvoker = ScaleEdgesAdjustableInvoker
	pageTitle = _( 'Scale' )
	titleText = _( 'Scale Edges' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedEdgesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	scaleManip = Scale3ColumnManipulator3d( row.scale, focalPoint, snapping=ColumnManipulatorSnappingFraction() )
	scaleManip.enable()
