##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Manipulator.RealManipulator import RealManipulator

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.RealManipulator import RealManipulator
from gSculpt.Manipulator.ColumnManipulators import Real1ColumnManipulator, ColumnManipulatorSnappingFraction

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshSlideEdgesAdjuster, GSProductMesh




class ProcMeshSlideEdges (GSProcedure):
	__version__ = '1'
	description = _( 'Slide edges' )

	slide = FieldColumn( float, 0.0 )

	inputClass = GSProductMesh


	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshSlideEdgesAdjuster()
		inputObject.slideMarkedEdges( adjuster, self.getSlideVersion() )
		adjuster.setParameters( self.slide )
		return inputObject


	def getSlideVersion(self):
		try:
			v = self.__version__
		except AttributeError:
			v = None

		if v is None:
			return 0
		elif v == '1':
			return 1
		else:
			return 1



class SlideEdgesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshSlideEdges

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshSlideEdgesAdjuster()
		self.mesh.slideMarkedEdges( self.adjuster, self._proc.getSlideVersion() )

	def adjust(self):
		self.adjuster.setParameters( self._proc.slide )

	def getResult(self):
		return self.mesh




class ToolMeshSlideEdges (GSProcedureTool):
	row = RowRefColumn( ProcMeshSlideEdges )

	functionClass = ProcMeshSlideEdges
	adjustableInvoker = SlideEdgesAdjustableInvoker
	pageTitle = _( 'Slide' )
	titleText = _( 'Slide Edges' )

	slideEntry = FloatColumnEditEntry( row.slide, -1.0, 1.0, 0.001, 0.1, 4 )

	slideManip = Real1ColumnManipulator( row.slide, -1.0, 1.0, RealManipulator.FixedScale( 0.0025 ), snapping=ColumnManipulatorSnappingFraction() )
	slideManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Slide:' )  <<  slideEntry.entry
	layout.row()
	layout.finalise()
