##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import Real2ColumnManipulator, RealColumnManipulator, ColumnManipulatorSnappingDistance
from gSculpt.Manipulator.RealManipulator import RealManipulator

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshBevelFacesAdjuster, GSProductMesh

from gSculpt.MeshTools.MeshEdit.MeshInsetFacesStyle import *





class ProcMeshBevelFaces (GSProcedure):
	description = _( 'Bevel faces' )

	inset = FieldColumn( float, 0.0 )
	extrude = FieldColumn( float, 0.0 )
	style = FieldColumn( MeshInsetFacesStyle, MeshInsetFacesStyle.FROMEDGES )
	bSingular = FieldColumn( bool, False )
	bFlowThruMarkedEdges = FieldColumn( bool, False )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bMarkInsetEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )
	bNSharpInsetEdges = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshBevelFacesAdjuster()
		if self.bSingular:
			inputObject.bevelMarkedFacesSingular( adjuster, self.style, self.bMarkBoundaryEdges, self.bMarkInsetEdges, self.bNSharpBoundaryEdges, self.bNSharpInsetEdges )
		else:
			inputObject.bevelMarkedFaces( adjuster, self.style, self.bFlowThruMarkedEdges, self.bMarkBoundaryEdges, self.bMarkInsetEdges, self.bNSharpBoundaryEdges, self.bNSharpInsetEdges )
		adjuster.setParameters( self.inset, self.extrude )
		return inputObject



class BevelFacesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshBevelFaces

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshBevelFacesAdjuster()
		if self._proc.bSingular:
			self.mesh.bevelMarkedFacesSingular( self.adjuster, self._proc.style, self._proc.bMarkBoundaryEdges, self._proc.bMarkInsetEdges, self._proc.bNSharpBoundaryEdges, self._proc.bNSharpInsetEdges )
		else:
			self.mesh.bevelMarkedFaces( self.adjuster, self._proc.style, self._proc.bFlowThruMarkedEdges, self._proc.bMarkBoundaryEdges, self._proc.bMarkInsetEdges, self._proc.bNSharpBoundaryEdges, self._proc.bNSharpInsetEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.inset, self._proc.extrude )

	def getResult(self):
		return self.mesh




class ToolMeshBevelFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshBevelFaces )

	functionClass = ProcMeshBevelFaces
	adjustableInvoker = BevelFacesAdjustableInvoker
	pageTitle = _( 'Bevel' )
	titleText = _( 'Bevel Faces' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedFacesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	insetEntry = FloatColumnEditEntry( row.inset, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	extrudeEntry = FloatColumnEditEntry( row.extrude, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	styleRadio = MeshInsetFacesStyleColumnEditRadioGroup( row.style )
	bSingularCheck = BoolColumnEditCheck( row.bSingular, _( 'Singular' ) )
	bFlowThruMarkedEdgesCheck = BoolColumnEditCheck( row.bFlowThruMarkedEdges, _( 'Flow thru marked edges' ) )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bMarkInsetEdgesCheck = BoolColumnEditCheck( row.bMarkInsetEdges, _( 'Mark inset edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'Normal sharpen boundary edges' ) )
	bNSharpInsetEdgesCheck = BoolColumnEditCheck( row.bNSharpInsetEdges, _( 'Normal sharpen inset edges' ) )

	bevelManip = Real2ColumnManipulator( row.inset, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
								row.extrude, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
								xSnapping=ColumnManipulatorSnappingDistance(), ySnapping=ColumnManipulatorSnappingDistance() )
	bevelManip.enable()

	bFlowThruMarkedEdgesCheck.inverseSensitivity = row.bSingular

	layout = FormLayoutFactory()
	layout.row()  <<  bSingularCheck.check
	layout.row()
	layout.row()  <<  _( 'Style:' )  <<  styleRadio.fromEdges
	layout.row()  <<  None  <<  styleRadio.toCentre
	layout.row()
	layout.row()  <<  _( 'Inset:' )  <<  insetEntry.entry
	layout.row()  <<  _( 'Extrude:' )  <<  extrudeEntry.entry
	layout.row()
	layout.row()  <<  bFlowThruMarkedEdgesCheck.check
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bMarkInsetEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpInsetEdgesCheck.check
	layout.row()
	layout.finalise()



	def __init__(self):
		super( ToolMeshBevelFaces, self ).__init__()
		self._worldSpaceScale = None
		self._fixedScale = None

	def initialise(self):
		super( ToolMeshBevelFaces, self ).initialise()
		self._worldSpaceScale = self.bevelManip.xScale
		self._fixedScale = RealManipulator.FixedScale( 0.0025 )
		self._func.fields.style.valueSignal.connect( self._p_onStyle )
		self._p_onStyle( self._func.style )

	def shutdown(self):
		super( ToolMeshBevelFaces, self ).shutdown()
		self._func.fields.style.valueSignal.disconnect( self._p_onStyle )
		self._worldSpaceScale = None
		self._fixedScale = None


	def _p_onStyle(self, value):
		if value == MeshInsetFacesStyle.FROMEDGES:
			self.bevelManip.snapping = self._editorSettings.snapSettings.distance
			self.bevelManip.xScale = self._worldSpaceScale
		elif value == MeshInsetFacesStyle.TOCENTRE:
			self.bevelManip.snapping = self._editorSettings.snapSettings.fraction
			self.bevelManip.xScale = self._fixedScale
