##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.Math.Math import Vector3

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshVectorExtrudeFacesAdjuster, GSProductMesh




class ProcMeshDuplicateFaces (GSProcedure):
	description = _( 'Duplicate faces' )

	offset = FieldColumn( Vector3, Vector3() )
	bMarkOriginalFaces = FieldColumn( bool, False )
	bMarkDuplicateFaces = FieldColumn( bool, True )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshVectorExtrudeFacesAdjuster()
		inputObject.duplicateMarkedFaces( adjuster, self.bMarkOriginalFaces, self.bMarkDuplicateFaces )
		adjuster.setParameters( self.offset )
		return inputObject



class DuplicateFacesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshDuplicateFaces

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshVectorExtrudeFacesAdjuster()
		self.mesh.duplicateMarkedFaces( self.adjuster, self._proc.bMarkOriginalFaces, self._proc.bMarkDuplicateFaces )

	def adjust(self):
		self.adjuster.setParameters( self._proc.offset )

	def getResult(self):
		return self.mesh




class ToolMeshDuplicateFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshDuplicateFaces )

	functionClass = ProcMeshDuplicateFaces
	adjustableInvoker = DuplicateFacesAdjustableInvoker
	pageTitle = _( 'DupF' )
	titleText = _( 'Duplicate Faces' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedFacesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	offsetEntry =Vector3ColumnEditEntry( row.offset, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	bMarkOriginalFacesCheck = BoolColumnEditCheck( row.bMarkOriginalFaces, _( 'Mark original faces' ) )
	bMarkDuplicateFacesCheck = BoolColumnEditCheck( row.bMarkDuplicateFaces, _( 'Mark duplicate faces' ) )

	offsetManip = Translation3ColumnManipulator3d( row.offset, focalPoint, snapping=ColumnManipulatorSnappingDistance() )
	offsetManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Offset:' )
	layout.row()  <<  'X:'  <<  offsetEntry.x
	layout.row()  <<  'Y:'  <<  offsetEntry.y
	layout.row()  <<  'Z:'  <<  offsetEntry.z
	layout.row()
	layout.row()  <<  bMarkOriginalFacesCheck.check
	layout.row()  <<  bMarkDuplicateFacesCheck.check
	layout.row()
	layout.finalise()
