##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import Real2ColumnManipulator, RealColumnManipulator, ColumnManipulatorSnappingDistance

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshExpandFacesAdjuster, GSProductMesh





class ProcMeshExpandFaces (GSProcedure):
	description = _( 'Expand faces' )

	expand = FieldColumn( float, 0.0 )
	extrude = FieldColumn( float, 0.0 )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bMarkExpandEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )
	bNSharpExpandEdges = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshExpandFacesAdjuster()
		inputObject.expandMarkedFaces( adjuster, self.bMarkBoundaryEdges, self.bMarkExpandEdges, self.bNSharpBoundaryEdges, self.bNSharpExpandEdges )
		adjuster.setParameters( self.expand, self.extrude )
		return inputObject



class ExpandFacesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshExpandFaces

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshExpandFacesAdjuster()
		self.mesh.expandMarkedFaces( self.adjuster, self._proc.bMarkBoundaryEdges, self._proc.bMarkExpandEdges, self._proc.bNSharpBoundaryEdges, self._proc.bNSharpExpandEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.expand, self._proc.extrude )

	def getResult(self):
		return self.mesh




class ToolMeshExpandFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshExpandFaces )

	functionClass = ProcMeshExpandFaces
	adjustableInvoker = ExpandFacesAdjustableInvoker
	pageTitle = _( 'Expand' )
	titleText = _( 'Expand Faces' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedFacesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	expandEntry = FloatColumnEditEntry( row.expand, 0.0, 1.0e8, 0.1, 10.0, 4 )
	extrudeEntry = FloatColumnEditEntry( row.extrude, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bMarkExpandEdgesCheck = BoolColumnEditCheck( row.bMarkExpandEdges, _( 'Mark expand edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'Normal sharpen boundary edges' ) )
	bNSharpExpandEdgesCheck = BoolColumnEditCheck( row.bNSharpExpandEdges, _( 'Normal sharpen expand edges' ) )

	expandManip = Real2ColumnManipulator( row.expand, 0.0, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									row.extrude, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									xSnapping=ColumnManipulatorSnappingDistance(), ySnapping=ColumnManipulatorSnappingDistance() )
	expandManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Expand:' )  <<  expandEntry.entry
	layout.row()  <<  _( 'Extrude:' )  <<  extrudeEntry.entry
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bMarkExpandEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpExpandEdgesCheck.check
	layout.row()
	layout.finalise()
