##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshExtrudeFacesAdjuster, GSProductMesh, MeshExtrudeFacesDirection

from gSculpt.Manipulator.RealManipulator import RealManipulator
from gSculpt.Manipulator.ColumnManipulators import *




# XML IO for MeshExtrudeFacesDirection

directionIOXmlEnum = IOXmlEnum( MeshExtrudeFacesDirection, 'MeshExtrudeFacesDirection', MeshExtrudeFacesDirection.EXPAND )
directionIOXmlEnum.value( MeshExtrudeFacesDirection.EXPAND, 'expand' )
directionIOXmlEnum.value( MeshExtrudeFacesDirection.PERGROUP, 'per_group' )

ioXmlReadMeshExtrudeFacesDirectionProp = directionIOXmlEnum.propReader
ioXmlReadMeshExtrudeFacesDirectionNode = directionIOXmlEnum.nodeReader
ioXmlWriteMeshExtrudeFacesDirectionProp = directionIOXmlEnum.propWriter
ioXmlWriteMeshExtrudeFacesDirectionNode = directionIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshExtrudeFacesDirection', MeshExtrudeFacesDirection )


# Field for MeshExtrudeFacesDirection

class MeshExtrudeFacesDirectionField (Field):
	valueClass = MeshExtrudeFacesDirection
	defaultValue = MeshExtrudeFacesDirection.EXPAND
	xmlNodeValueReader = ioXmlReadMeshExtrudeFacesDirectionNode
	xmlNodeValueWriter = ioXmlWriteMeshExtrudeFacesDirectionNode

registerFieldType( MeshExtrudeFacesDirection, MeshExtrudeFacesDirectionField )


# Variable editor for MeshExtrudeFacesDirection

class MeshExtrudeFacesDirectionVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshExtrudeFacesDirectionVarEditRadioGroup, self ).__init__()
		self.expand = self.addChoice( MeshExtrudeFacesDirection.EXPAND, _( 'Expand' ) )
		self.perGroup = self.addChoice( MeshExtrudeFacesDirection.PERGROUP, _( 'Per group' ) )



# Column editor for MeshExtrudeFacesDirection

class MeshExtrudeFacesDirectionColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshExtrudeFacesDirectionColumnEditRadioGroup, self ).__init__( [ 'expand', 'perGroup' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshExtrudeFacesDirectionVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshExtrudeFacesDirectionColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshExtrudeFacesDirectionColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()









class ProcMeshExtrudeFaces (GSProcedure):
	description = _( 'Extrude faces' )

	extrusion = FieldColumn( float, 0.0 )
	direction = FieldColumn( MeshExtrudeFacesDirection, MeshExtrudeFacesDirection.EXPAND )
	segments = FieldColumn( int, 1 )
	bSingular = FieldColumn( bool, False )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bMarkExtrudedEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )
	bNSharpExtrudedEdges = FieldColumn( bool, True )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshExtrudeFacesAdjuster()
		if self.bSingular:
			inputObject.extrudeMarkedFacesSingular( adjuster, self.segments, self.bMarkBoundaryEdges, self.bMarkExtrudedEdges, self.bNSharpBoundaryEdges, self.bNSharpExtrudedEdges )
		else:
			inputObject.extrudeMarkedFaces( adjuster, self.direction, self.segments, self.bMarkBoundaryEdges, self.bMarkExtrudedEdges, self.bNSharpBoundaryEdges, self.bNSharpExtrudedEdges )
		adjuster.setParameters( self.extrusion )
		return inputObject



class ExtrudeFacesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshExtrudeFaces

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshExtrudeFacesAdjuster()
		if self._proc.bSingular:
			self.mesh.extrudeMarkedFacesSingular( self.adjuster, self._proc.segments, self._proc.bMarkBoundaryEdges, self._proc.bMarkExtrudedEdges, self._proc.bNSharpBoundaryEdges, self._proc.bNSharpExtrudedEdges )
		else:
			self.mesh.extrudeMarkedFaces( self.adjuster, self._proc.direction, self._proc.segments, self._proc.bMarkBoundaryEdges, self._proc.bMarkExtrudedEdges, self._proc.bNSharpBoundaryEdges, self._proc.bNSharpExtrudedEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.extrusion )

	def getResult(self):
		return self.mesh




class ToolMeshExtrudeFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshExtrudeFaces )

	functionClass = ProcMeshExtrudeFaces
	adjustableInvoker = ExtrudeFacesAdjustableInvoker
	pageTitle = _( 'Extrude' )
	titleText = _( 'Extrude Faces' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedFacesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	extrusionEntry = FloatColumnEditEntry( row.extrusion, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	directionRadio = MeshExtrudeFacesDirectionColumnEditRadioGroup( row.direction )
	bSingularCheck = BoolColumnEditCheck( row.bSingular, _( 'Singular' ) )
	segmentsEntry = IntColumnEditEntry( row.segments, 1, 1048576, 1, 10 )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bMarkExtrudedEdgesCheck = BoolColumnEditCheck( row.bMarkExtrudedEdges, _( 'Mark extruded edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'N-sharp boundary edges' ) )
	bNSharpExtrudedEdgesCheck = BoolColumnEditCheck( row.bNSharpExtrudedEdges, _( 'N-sharp extruded edges' ) )

	extrudeManip = Real1ColumnManipulator( row.extrusion, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									snapping=ColumnManipulatorSnappingDistance() )
	extrudeManip.enable()

	numSegmentsManip = IntColumnScrollManipulator( row.segments, 1, 1048576 )
	numSegmentsManip.enable()

	directionRadio.inverseSensitivity = row.bSingular

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Use the scroll wheel to' )
	layout.row()  <<  _( 'change the number of segments' )
	layout.row()
	layout.row()  <<  bSingularCheck.check
	layout.row()
	layout.row()  <<  _( 'Direction:' )  <<  directionRadio.expand
	layout.row()  <<  None  <<  directionRadio.perGroup
	layout.row()
	layout.row()  <<  _( 'Extrusion:' )  <<  extrusionEntry.entry
	layout.row()  <<  _( 'Segments:' )  <<  segmentsEntry.entry
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bMarkExtrudedEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpExtrudedEdgesCheck.check
	layout.row()
	layout.finalise()
