##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Vector3

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *
from gSculpt.Kernel.Enum import Enum

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Mesh.Mesh import GSProductMesh




# MeshFlattenFacesMode enumeration
class MeshFlattenFacesMode (Enum):
	X = 0
	Y = 1
	Z = 2
	NORMAL = 3




# XML IO for MeshFlattenFacesMode

flattenModeIOXmlEnum = IOXmlEnum( MeshFlattenFacesMode, 'MeshFlattenFacesMode', MeshFlattenFacesMode.NORMAL )
flattenModeIOXmlEnum.value( MeshFlattenFacesMode.X, 'x' )
flattenModeIOXmlEnum.value( MeshFlattenFacesMode.Y, 'y' )
flattenModeIOXmlEnum.value( MeshFlattenFacesMode.Z, 'z' )
flattenModeIOXmlEnum.value( MeshFlattenFacesMode.NORMAL, 'normal' )

ioXmlReadMeshFlattenFacesModeProp = flattenModeIOXmlEnum.propReader
ioXmlReadMeshFlattenFacesModeNode = flattenModeIOXmlEnum.nodeReader
ioXmlWriteMeshFlattenFacesModeProp = flattenModeIOXmlEnum.propWriter
ioXmlWriteMeshFlattenFacesModeNode = flattenModeIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshFlattenFacesMode', MeshFlattenFacesMode )


# Field for MeshFlattenFacesMode

class MeshFlattenFacesModeField (Field):
	valueClass = MeshFlattenFacesMode
	defaultValue = MeshFlattenFacesMode.NORMAL
	xmlNodeValueReader = ioXmlReadMeshFlattenFacesModeNode
	xmlNodeValueWriter = ioXmlWriteMeshFlattenFacesModeNode

registerFieldType( MeshFlattenFacesMode, MeshFlattenFacesModeField )


# Variable editor for MeshFlattenFacesMode

class MeshFlattenFacesModeVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshFlattenFacesModeVarEditRadioGroup, self ).__init__()
		self.x = self.addChoice( MeshFlattenFacesMode.X, _( 'X-axis' ) )
		self.y = self.addChoice( MeshFlattenFacesMode.Y, _( 'Y-axis' ) )
		self.z = self.addChoice( MeshFlattenFacesMode.Z, _( 'Z-axis' ) )
		self.normal = self.addChoice( MeshFlattenFacesMode.NORMAL, _( 'Normal' ) )


# Column editor for MeshFlattenFacesMode

class MeshFlattenFacesModeColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshFlattenFacesModeColumnEditRadioGroup, self ).__init__( [ 'x', 'y', 'z', 'normal' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshFlattenFacesModeVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshFlattenFacesModeColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshFlattenFacesModeColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()





class ProcMeshFlattenFaces (GSProcedure):
	description = _( 'Flatten faces' )

	bPerRegion = FieldColumn( bool, True )
	mode = FieldColumn( MeshFlattenFacesMode, MeshFlattenFacesMode.NORMAL )


	inputClass = GSProductMesh
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		mode = self.mode
		if mode == MeshFlattenFacesMode.X:
			inputObject.flattenFaces( Vector3( 1.0, 0.0, 0.0 ), self.bPerRegion )
		elif mode == MeshFlattenFacesMode.Y:
			inputObject.flattenFaces( Vector3( 0.0, 1.0, 0.0 ), self.bPerRegion )
		elif mode == MeshFlattenFacesMode.Z:
			inputObject.flattenFaces( Vector3( 0.0, 0.0, 1.0 ), self.bPerRegion )
		elif mode == MeshFlattenFacesMode.NORMAL:
			inputObject.flattenFacesNormal( self.bPerRegion )
		return inputObject




class ToolMeshFlattenFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshFlattenFaces )

	functionClass = ProcMeshFlattenFaces
	pageTitle = _( 'Flatten' )
	titleText = _( 'Flatten faces' )

	bPerRegionCheck = BoolColumnEditCheck( row.bPerRegion, _( 'Per region' ) )
	modeRadio = MeshFlattenFacesModeColumnEditRadioGroup( row.mode )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Flatten mode:' )
	layout.row()  <<  modeRadio.x
	layout.row()  <<  modeRadio.y
	layout.row()  <<  modeRadio.z
	layout.row()  <<  modeRadio.normal
	layout.row()
	layout.row()  <<  bPerRegionCheck.check
	layout.row()
	layout.finalise()
