##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.Math.Math import Vector3

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshVectorExtrudeFacesAdjuster, GSProductMesh




class ProcMeshFreeExtrudeFaces (GSProcedure):
	description = _( 'Free extrude faces' )

	extrusion = FieldColumn( Vector3, Vector3() )
	segments = FieldColumn( int, 1 )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bMarkExtrudedEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )
	bNSharpExtrudedEdges = FieldColumn( bool, True )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshVectorExtrudeFacesAdjuster()
		inputObject.vectorExtrudeMarkedFaces( adjuster, self.segments, self.bMarkBoundaryEdges, self.bMarkExtrudedEdges, self.bNSharpBoundaryEdges, self.bNSharpExtrudedEdges )
		adjuster.setParameters( self.extrusion )
		return inputObject



class FreeExtrudeFacesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshFreeExtrudeFaces

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshVectorExtrudeFacesAdjuster()
		self.mesh.vectorExtrudeMarkedFaces( self.adjuster, self._proc.segments, self._proc.bMarkBoundaryEdges, self._proc.bMarkExtrudedEdges, self._proc.bNSharpBoundaryEdges, self._proc.bNSharpExtrudedEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.extrusion )

	def getResult(self):
		return self.mesh




class ToolMeshFreeExtrudeFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshFreeExtrudeFaces )

	functionClass = ProcMeshFreeExtrudeFaces
	adjustableInvoker = FreeExtrudeFacesAdjustableInvoker
	pageTitle = _( 'F.Extr.' )
	titleText = _( 'Free Extrude Faces' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedFacesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	extrusionEntry =Vector3ColumnEditEntry( row.extrusion, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	segmentsEntry = IntColumnEditEntry( row.segments, 1, 1048576, 1, 10 )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bMarkExtrudedEdgesCheck = BoolColumnEditCheck( row.bMarkExtrudedEdges, _( 'Mark extruded edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'N-sharp boundary edges' ) )
	bNSharpExtrudedEdgesCheck = BoolColumnEditCheck( row.bNSharpExtrudedEdges, _( 'N-sharp extruded edges' ) )

	extrudeManip = Translation3ColumnManipulator3d( row.extrusion, focalPoint, snapping=ColumnManipulatorSnappingDistance() )
	extrudeManip.enable()

	numSegmentsManip = IntColumnScrollManipulator( row.segments, 1, 1048576 )
	numSegmentsManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Use the scroll wheel to' )
	layout.row()  <<  _( 'change the number of segments' )
	layout.row()
	layout.row()  <<  _( 'Extrusion:' )
	layout.row()  <<  'X:'  <<  extrusionEntry.x
	layout.row()  <<  'Y:'  <<  extrusionEntry.y
	layout.row()  <<  'Z:'  <<  extrusionEntry.z
	layout.row()
	layout.row()  <<  _( 'Segments:' )  <<  segmentsEntry.entry
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bMarkExtrudedEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpExtrudedEdgesCheck.check
	layout.row()
	layout.finalise()
