##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import Real1ColumnManipulator, RealColumnManipulator, ColumnManipulatorSnappingDistance
from gSculpt.Manipulator.RealManipulator import RealManipulator

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshInsetFacesAdjuster, GSProductMesh

from gSculpt.MeshTools.MeshEdit.MeshInsetFacesStyle import *





class ProcMeshInsetFaces (GSProcedure):
	description = _( 'Inset faces' )

	inset = FieldColumn( float, 0.0 )
	style = FieldColumn( MeshInsetFacesStyle, MeshInsetFacesStyle.FROMEDGES )
	bSingular = FieldColumn( bool, False )
	bFlowThruMarkedEdges = FieldColumn( bool, False )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bMarkInsetEdges = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshInsetFacesAdjuster()
		if self.bSingular:
			inputObject.insetMarkedFacesSingular( adjuster, self.style, self.bMarkBoundaryEdges, self.bMarkInsetEdges )
		else:
			inputObject.insetMarkedFaces( adjuster, self.style, self.bFlowThruMarkedEdges, self.bMarkBoundaryEdges, self.bMarkInsetEdges )
		adjuster.setParameters( self.inset )
		return inputObject



class InsetFacesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshInsetFaces

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshInsetFacesAdjuster()
		if self._proc.bSingular:
			self.mesh.insetMarkedFacesSingular( self.adjuster, self._proc.style, self._proc.bMarkBoundaryEdges, self._proc.bMarkInsetEdges )
		else:
			self.mesh.insetMarkedFaces( self.adjuster, self._proc.style, self._proc.bFlowThruMarkedEdges, self._proc.bMarkBoundaryEdges, self._proc.bMarkInsetEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.inset )

	def getResult(self):
		return self.mesh




class ToolMeshInsetFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshInsetFaces )

	functionClass = ProcMeshInsetFaces
	adjustableInvoker = InsetFacesAdjustableInvoker
	pageTitle = _( 'Inset' )
	titleText = _( 'Inset Faces' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedFacesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	insetEntry = FloatColumnEditEntry( row.inset, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	styleRadio = MeshInsetFacesStyleColumnEditRadioGroup( row.style )
	bSingularCheck = BoolColumnEditCheck( row.bSingular, _( 'Singular' ) )
	bFlowThruMarkedEdgesCheck = BoolColumnEditCheck( row.bFlowThruMarkedEdges, _( 'Flow thru marked edges' ) )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bMarkInsetEdgesCheck = BoolColumnEditCheck( row.bMarkInsetEdges, _( 'Mark inset edges' ) )

	insetManip = Real1ColumnManipulator( row.inset, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									snapping=ColumnManipulatorSnappingDistance() )
	insetManip.enable()

	bFlowThruMarkedEdgesCheck.inverseSensitivity = row.bSingular

	layout = FormLayoutFactory()
	layout.row()  <<  bSingularCheck.check
	layout.row()
	layout.row()  <<  _( 'Style:' )  <<  styleRadio.fromEdges
	layout.row()  <<  None  <<  styleRadio.toCentre
	layout.row()
	layout.row()  <<  _( 'Inset:' )  <<  insetEntry.entry
	layout.row()
	layout.row()  <<  bFlowThruMarkedEdgesCheck.check
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bMarkInsetEdgesCheck.check
	layout.row()
	layout.finalise()



	def __init__(self):
		super( ToolMeshInsetFaces, self ).__init__()
		self._worldSpaceScale = None
		self._fixedScale = None

	def initialise(self):
		super( ToolMeshInsetFaces, self ).initialise()
		self._worldSpaceScale = self.insetManip.scale
		self._fixedScale = RealManipulator.FixedScale( 0.0025 )
		self._func.fields.style.valueSignal.connect( self._p_onStyle )
		self._p_onStyle( self._func.style )

	def shutdown(self):
		super( ToolMeshInsetFaces, self ).shutdown()
		self._func.fields.style.valueSignal.disconnect( self._p_onStyle )
		self._worldSpaceScale = None
		self._fixedScale = None


	def _p_onStyle(self, value):
		if value == MeshInsetFacesStyle.FROMEDGES:
			self.insetManip.snapping = self._editorSettings.snapSettings.distance
			self.insetManip.scale = self._worldSpaceScale
		elif value == MeshInsetFacesStyle.TOCENTRE:
			self.insetManip.snapping = self._editorSettings.snapSettings.fraction
			self.insetManip.scale = self._fixedScale





