##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *
from gSculpt.Kernel.Enum import Enum

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Model.Model import MarkPredicate
from gSculpt.Model.MarkPredicateColumnEditor import MarkPredicateColumnEditRadioGroup

from gSculpt.Mesh.Mesh import GSProductMesh



# MeshMarkFacesTarget enumeration
class MeshMarkFacesTarget (Enum):
	ALL = 0
	VERTEXMARKED = 1
	WHOLLYVERTEXMARKED = 2
	EDGEMARKED = 3
	WHOLLYEDGEMARKED = 4




# XML IO for MeshMarkFacesTarget

markTargetIOXmlEnum = IOXmlEnum( MeshMarkFacesTarget, 'MeshMarkFacesTarget', MeshMarkFacesTarget.ALL )
markTargetIOXmlEnum.value( MeshMarkFacesTarget.ALL, 'all' )
markTargetIOXmlEnum.value( MeshMarkFacesTarget.VERTEXMARKED, 'vertex_marked' )
markTargetIOXmlEnum.value( MeshMarkFacesTarget.WHOLLYVERTEXMARKED, 'wholly_vertex_marked' )
markTargetIOXmlEnum.value( MeshMarkFacesTarget.EDGEMARKED, 'edge_marked' )
markTargetIOXmlEnum.value( MeshMarkFacesTarget.WHOLLYEDGEMARKED, 'wholly_edge_marked' )

ioXmlReadMeshMarkFacesTargetProp = markTargetIOXmlEnum.propReader
ioXmlReadMeshMarkFacesTargetNode = markTargetIOXmlEnum.nodeReader
ioXmlWriteMeshMarkFacesTargetProp = markTargetIOXmlEnum.propWriter
ioXmlWriteMeshMarkFacesTargetNode = markTargetIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshMarkFacesTarget', MeshMarkFacesTarget )


# Field for MeshMarkFacesTarget

class MeshMarkFacesTargetField (Field):
	valueClass = MeshMarkFacesTarget
	defaultValue = MeshMarkFacesTarget.ALL
	xmlNodeValueReader = ioXmlReadMeshMarkFacesTargetNode
	xmlNodeValueWriter = ioXmlWriteMeshMarkFacesTargetNode

registerFieldType( MeshMarkFacesTarget, MeshMarkFacesTargetField )


# Variable editor for MeshMarkFacesTarget

class MeshMarkFacesTargetVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshMarkFacesTargetVarEditRadioGroup, self ).__init__()
		self.all = self.addChoice( MeshMarkFacesTarget.ALL, _( 'All' ) )
		self.vertexMarked = self.addChoice( MeshMarkFacesTarget.VERTEXMARKED, _( 'With selected vertices' ) )
		self.whollyVertexMarked = self.addChoice( MeshMarkFacesTarget.WHOLLYVERTEXMARKED, _( 'With all vertices selected' ) )
		self.edgeMarked = self.addChoice( MeshMarkFacesTarget.EDGEMARKED, _( 'With selected edges' ) )
		self.whollyEdgeMarked = self.addChoice( MeshMarkFacesTarget.WHOLLYEDGEMARKED, _( 'With all edges selected' ) )


# Column editor for MeshMarkFacesTarget

class MeshMarkFacesTargetColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshMarkFacesTargetColumnEditRadioGroup, self ).__init__( [ 'all', 'vertexMarked', 'whollyVertexMarked', 'edgeMarked', 'whollyEdgeMarked' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshMarkFacesTargetVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshMarkFacesTargetColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshMarkFacesTargetColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()





class ProcMeshMarkFaces (GSProcedure):
	predicate = FieldColumn( MarkPredicate, MarkPredicate.MARK )
	target = FieldColumn( MeshMarkFacesTarget, MeshMarkFacesTarget.ALL )


	def getDescription(self):
		if self.predicate == MarkPredicate.MARK:
			return _( 'Select faces' )
		elif self.predicate == MarkPredicate.UNMARK:
			return _( 'Unselect faces' )
		elif self.predicate == MarkPredicate.ADD:
			return _( 'Select faces' )
		elif self.predicate == MarkPredicate.FILTER:
			return _( 'Filter face selection' )
		elif self.predicate == MarkPredicate.INVERT:
			return _( 'Invert face selection' )


	inputClass = GSProductMesh
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		target = self.target
		if target == MeshMarkFacesTarget.ALL:
			inputObject.markFaces_all( self.predicate )
		elif target == MeshMarkFacesTarget.VERTEXMARKED:
			inputObject.markFaces_vertexMarked( self.predicate )
		elif target == MeshMarkFacesTarget.WHOLLYVERTEXMARKED:
			inputObject.markFaces_whollyVertexMarked( self.predicate )
		elif target == MeshMarkFacesTarget.EDGEMARKED:
			inputObject.markFaces_edgeMarked( self.predicate )
		elif target == MeshMarkFacesTarget.WHOLLYEDGEMARKED:
			inputObject.markFaces_whollyEdgeMarked( self.predicate )
		return inputObject



class ToolMeshMarkFaces (GSProcedureTool):
	row = RowRefColumn( ProcMeshMarkFaces )

	functionClass = ProcMeshMarkFaces
	pageTitle = _( 'Select' )
	titleText = _( 'Select faces' )

	predicateRadio = MarkPredicateColumnEditRadioGroup( row.predicate )
	targetRadio = MeshMarkFacesTargetColumnEditRadioGroup( row.target )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Select mode:' )
	layout.row()  <<  predicateRadio.mark
	layout.row()  <<  predicateRadio.unmark
	layout.row()  <<  predicateRadio.add
	layout.row()  <<  predicateRadio.filter
	layout.row()  <<  predicateRadio.invert
	layout.row()
	layout.row()  <<  _( 'Target:' )
	layout.row()  <<  targetRadio.all
	layout.row()  <<  targetRadio.vertexMarked
	layout.row()  <<  targetRadio.whollyVertexMarked
	layout.row()  <<  targetRadio.edgeMarked
	layout.row()  <<  targetRadio.whollyEdgeMarked
	layout.row()
	layout.finalise()

