##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.Kernel.DB import Field, registerFieldType
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup


from gSculpt.Mesh.Mesh import MeshInsetFacesStyle



# XML IO for MeshInsetFacesStyle

styleIOXmlEnum = IOXmlEnum( MeshInsetFacesStyle, 'MeshInsetFacesStyle', MeshInsetFacesStyle.FROMEDGES )
styleIOXmlEnum.value( MeshInsetFacesStyle.FROMEDGES, 'from_edges' )
styleIOXmlEnum.value( MeshInsetFacesStyle.TOCENTRE, 'to_centre' )

ioXmlReadMeshInsetFacesStyleProp = styleIOXmlEnum.propReader
ioXmlReadMeshInsetFacesStyleNode = styleIOXmlEnum.nodeReader
ioXmlWriteMeshInsetFacesStyleProp = styleIOXmlEnum.propWriter
ioXmlWriteMeshInsetFacesStyleNode = styleIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshInsetFacesStyle', MeshInsetFacesStyle )


# Field for MeshInsetFacesStyle

class MeshInsetFacesStyleField (Field):
	valueClass = MeshInsetFacesStyle
	defaultValue = MeshInsetFacesStyle.FROMEDGES
	xmlNodeValueReader = ioXmlReadMeshInsetFacesStyleNode
	xmlNodeValueWriter = ioXmlWriteMeshInsetFacesStyleNode

registerFieldType( MeshInsetFacesStyle, MeshInsetFacesStyleField )


# Variable editor for MeshInsetFacesStyle

class MeshInsetFacesStyleVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshInsetFacesStyleVarEditRadioGroup, self ).__init__()
		self.fromEdges = self.addChoice( MeshInsetFacesStyle.FROMEDGES, _( 'From edges' ) )
		self.toCentre = self.addChoice( MeshInsetFacesStyle.TOCENTRE, _( 'To centre' ) )



# Column editor for MeshInsetFacesStyle

class MeshInsetFacesStyleColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshInsetFacesStyleColumnEditRadioGroup, self ).__init__( [ 'fromEdges', 'toCentre' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshInsetFacesStyleVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshInsetFacesStyleColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshInsetFacesStyleColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()

