##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.Kernel.DB import Field, registerFieldType
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup


from gSculpt.Mesh.Mesh import MeshWeldStyle



# XML IO for MeshWeldStyle

directionIOXmlEnum = IOXmlEnum( MeshWeldStyle, 'MeshWeldStyle', MeshWeldStyle.MIDPOINT )
directionIOXmlEnum.value( MeshWeldStyle.MIDPOINT, 'mid_point' )
directionIOXmlEnum.value( MeshWeldStyle.A, 'a' )
directionIOXmlEnum.value( MeshWeldStyle.B, 'b' )

ioXmlReadMeshWeldStyleProp = directionIOXmlEnum.propReader
ioXmlReadMeshWeldStyleNode = directionIOXmlEnum.nodeReader
ioXmlWriteMeshWeldStyleProp = directionIOXmlEnum.propWriter
ioXmlWriteMeshWeldStyleNode = directionIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshWeldStyle', MeshWeldStyle )


# Field for MeshWeldStyle

class MeshWeldStyleField (Field):
	valueClass = MeshWeldStyle
	defaultValue = MeshWeldStyle.MIDPOINT
	xmlNodeValueReader = ioXmlReadMeshWeldStyleNode
	xmlNodeValueWriter = ioXmlWriteMeshWeldStyleNode

registerFieldType( MeshWeldStyle, MeshWeldStyleField )


# Variable editor for MeshWeldStyle

class MeshWeldStyleVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshWeldStyleVarEditRadioGroup, self ).__init__()
		self.midPoint = self.addChoice( MeshWeldStyle.MIDPOINT, _( 'Mid-point' ) )
		self.a = self.addChoice( MeshWeldStyle.A, 'A' )
		self.b = self.addChoice( MeshWeldStyle.B, 'B' )



# Column editor for MeshWeldStyle

class MeshWeldStyleColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshWeldStyleColumnEditRadioGroup, self ).__init__( [ 'midPoint', 'a', 'b' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshWeldStyleVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshWeldStyleColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshWeldStyleColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()

