##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

import pygtk
pygtk.require( '2.0' )
import gtk

from gSculpt.Math.Math import Colour3f, Point3, Segment3

from gSculpt.Util.CoEventHandler import CoEventHandler

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive, ioObjectFactoryRegister

from gSculpt.View.View import PaintLayer

from gSculpt.WorkArea.Viewport import ViewportContextButtonListener
from gSculpt.WorkArea.Viewport3d import Viewport3d, Viewport3dPainter

from gSculpt.UI.FormLayout import FormLayout

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import BoolColumnEditCheck

from gSculpt.ProceduralCore.GSFunction import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model import ModelDraw
from gSculpt.Mesh.Mesh import GSProductMesh, pyMeshDrawQuadsPointList_to_MeshDrawQuadsPointList, CppMeshDrawQuadsPoint, CppMeshDrawQuadsPointList, MeshDrawQuadsPointTarget, MeshDrawQuadsDirection, MeshDrawQuadsState
from gSculpt.Mesh.MeshManipulatorHelper import MeshManipulatorHelper

from gSculpt.Model.Pick3d import Pick3d
from gSculpt.Model.TargetList import TargetList
from gSculpt.Model.TargetListColumnEditor import TargetListColumnEditor
from gSculpt.Model.Model import GSProductModelBlank

from gSculpt.Manipulator.Manipulator import Manipulator






drawQuadsTargetIOXmlEnum = IOXmlEnum( MeshDrawQuadsPointTarget, 'MeshDrawQuadsPointTarget', MeshDrawQuadsPointTarget.NEWPOINT )
drawQuadsTargetIOXmlEnum.value( MeshDrawQuadsPointTarget.NEWPOINT, 'new_point' )
drawQuadsTargetIOXmlEnum.value( MeshDrawQuadsPointTarget.EXISTINGVERTEX, 'existing_vertex' )

ioXmlReadMeshDrawQuadsPointTargetProp = drawQuadsTargetIOXmlEnum.propReader
ioXmlReadMeshDrawQuadsPointTargetNode = drawQuadsTargetIOXmlEnum.nodeReader
ioXmlWriteMeshDrawQuadsPointTargetProp = drawQuadsTargetIOXmlEnum.propWriter
ioXmlWriteMeshDrawQuadsPointTargetNode = drawQuadsTargetIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshDrawQuadsPointTarget', MeshDrawQuadsPointTarget )





drawQuadsDirectionIOXmlEnum = IOXmlEnum( MeshDrawQuadsDirection, 'MeshDrawQuadsDirection', MeshDrawQuadsDirection.FORWARD )
drawQuadsDirectionIOXmlEnum.value( MeshDrawQuadsDirection.FORWARD, 'forward' )
drawQuadsDirectionIOXmlEnum.value( MeshDrawQuadsDirection.PINGPONG, 'pingpong' )

ioXmlReadMeshDrawQuadsPointDirectionProp = drawQuadsDirectionIOXmlEnum.propReader
ioXmlReadMeshDrawQuadsPointDirectionNode = drawQuadsDirectionIOXmlEnum.nodeReader
ioXmlWriteMeshDrawQuadsPointDirectionProp = drawQuadsDirectionIOXmlEnum.propWriter
ioXmlWriteMeshDrawQuadsPointDirectionNode = drawQuadsDirectionIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshDrawQuadsDirection', MeshDrawQuadsDirection )




class MeshDrawQuadsPoint (object):
	def __init__(self, pick=None, position=None, target=MeshDrawQuadsPointTarget.NEWPOINT, direction=MeshDrawQuadsDirection.FORWARD):
		if pick is None:
			pick = Pick3d()
		if position is None:
			position = Point3()
		self.pick = pick
		self.position = position
		self.target = target
		self.direction = direction


	def toCppMeshDrawQuadsPoint(self):
		return CppMeshDrawQuadsPoint( self.pick.toMPick(), self.position, self.target, self.direction )


	def __readxml__(self, xmlNode):
		xmlNode.getChild( 'pick' )  >>  self.pick
		xmlNode.getChild( 'position' )  >>  self.position
		self.target = ioXmlReadMeshDrawQuadsPointTargetNode( xmlNode.getChild( 'target' ), self.target )
		self.direction = ioXmlReadMeshDrawQuadsPointDirectionNode( xmlNode.getChild( 'direction' ), self.direction )

	def __writexml__(self, xmlNode):
		xmlNode.addChild( 'pick' )  <<  self.pick
		xmlNode.addChild( 'position' )  <<  self.position
		ioXmlWriteMeshDrawQuadsPointTargetNode( xmlNode.addChild( 'target' ), self.target )
		ioXmlWriteMeshDrawQuadsPointDirectionNode( xmlNode.addChild( 'direction' ), self.direction )


ioObjectFactoryRegister( 'MeshDrawQuadsPoint', MeshDrawQuadsPoint )







class ProcMeshDrawQuads (GSProcedure):
	pointList = FieldColumn( TargetList, TargetList() )
	bCloseRows = FieldColumn( bool, False )
	bCloseColumns = FieldColumn( bool, False )
	numColumns = FieldColumn( int, 1 )
	bFlip = FieldColumn( bool, False )
	bNumColumnsSet = FieldColumn( bool, False )

	description = _( 'Draw quad mesh' )
	inputClass = [ GSProductModelBlank, GSProductMesh ]
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		if isinstance( inputObject, GSProductMesh ):
			mesh = inputObject
		else:
			mesh = GSProductMesh()

		cppPoints = CppMeshDrawQuadsPointList()
		pyMeshDrawQuadsPointList_to_MeshDrawQuadsPointList( [ point.toCppMeshDrawQuadsPoint()  for  point in self.pointList ], cppPoints )
		mesh.drawQuads( cppPoints, self.bCloseRows, self.bCloseColumns, self.numColumns, self.bFlip  )
		return mesh






class MeshDrawQuadsManipulator (Manipulator, MeshManipulatorHelper, ViewportContextButtonListener, Viewport3dPainter):
	EVENT_BUTTONDOWN = 0
	EVENT_BUTTONUP = 1
	EVENT_MOTION = 2
	EVENT_LEAVE = 3
	EVENT_CONTEXT = 4
	EVENT_RESET = 5


	class Event (object):
		def __init__(self, eventType, **kwargs):
			self.eventType = eventType
			self.__dict__.update( kwargs )

	class ResetException:
		pass

	class TerminateException:
		pass




	def __init__(self):
		super( MeshDrawQuadsManipulator, self ).__init__()
		self._pointList = None
		self._bCloseRows = None
		self._bCloseColumns = None
		self._numColumns = None
		self._bNumColumnsSet = None

		self._coEventHandler = None
		self._storedEvents = []

		self._vertexHighlightPosition = None
		self._bTargetValid = True
		self._pointHighlightPosition = None

		self._bDisplayRequiresRefresh = True
		self._displayRow0Points = []
		self._displayRow1Points = []
		self._displayRowSegs = []
		self._displayRow0ClosingSeg = None
		self._displayRow1ClosingSeg = None
		self._displayColumnSegs = []

		self._bComplete = False

		self.state = None

		self._o_setUISensitivity( False )







	def manipAttachToView(self, view):
		super( MeshDrawQuadsManipulator, self ).manipAttachToView( view )
		self._coEventHandler = CoEventHandler( self._p_manipulatorLoop )
		self._coEventHandler.start()
		self.attachView( view )
		self._view.addEventHandler( self, Viewport3d )		# HACK: need to handle other viewport types
		self._view.addPainter( self, Viewport3d )
		self._view.postRedraw()

	def manipDetachFromView(self):
		self._view.removeEventHandler( self, Viewport3d )
		# Very important that we remove the context button listener here, as if the manipulator is removed before completing
		# the editing cycle (by undo/redo for example), then the context listener can remain, blocking other listeners with a lower priority
		self._view.removeContextButtonListener( self, Viewport3d )
		self._view.removePainter( self, Viewport3d )
		self._view.postRedraw()
		self.detachView()
		self._coEventHandler.terminate()
		self._coEventHandler = None
		super( MeshDrawQuadsManipulator, self ).manipDetachFromView()


	def attachVariables(self, pointList, bCloseRows, bCloseColumns, numColumns, bNumColumnsSet):
		assert self._pointList is None, 'already attached to a variable'
		self._pointList = pointList
		self._bCloseRows = bCloseRows
		self._bCloseColumns = bCloseColumns
		self._numColumns = numColumns
		self._bNumColumnsSet = bNumColumnsSet
		self._pointList.changedSignal.connect( self._p_onPointListChanged )
		self._bCloseRows.changedSignal.connect( self._p_onVarChanged )
		self._bCloseColumns.changedSignal.connect( self._p_onVarChanged )
		self._numColumns.changedSignal.connect( self._p_onVarChanged )
		self._bNumColumnsSet.changedSignal.connect( self._p_onVarChanged )
		self._o_watchVarValidity( self._pointList )
		self._o_watchVarValidity( self._bCloseRows )
		self._o_watchVarValidity( self._bCloseColumns )
		self._o_watchVarValidity( self._numColumns )
		self._o_watchVarValidity( self._bNumColumnsSet )
		self._o_refreshSensitivity()

	def detachVariables(self):
		assert self._pointList is not None, 'not attached to a variable'
		self._pointList.changedSignal.disconnect( self._p_onPointListChanged )
		self._bCloseRows.changedSignal.disconnect( self._p_onVarChanged )
		self._bCloseColumns.changedSignal.disconnect( self._p_onVarChanged )
		self._numColumns.changedSignal.disconnect( self._p_onVarChanged )
		self._bNumColumnsSet.changedSignal.disconnect( self._p_onVarChanged )
		self._o_stopWatchingVarValidity( self._pointList )
		self._o_stopWatchingVarValidity( self._bCloseRows )
		self._o_stopWatchingVarValidity( self._bCloseColumns )
		self._o_stopWatchingVarValidity( self._numColumns )
		self._o_stopWatchingVarValidity( self._bNumColumnsSet )
		self._pointList = None
		self._bCloseRows = None
		self._bCloseColumns = None
		self._numColumns = None
		self._bNumColumnsSet = None
		self._o_refreshSensitivity()



	def _o_setUISensitivity(self, bSensitive):
		self._o_setMeshManipulatorHelperSensitivity( bSensitive )

	def _o_computeSensitivity(self):
		if self._pointList is not None  and  self._bCloseRows is not None  and  self._bCloseColumns is not None  and  self._numColumns is not None:
			return self._pointList.isValid()  and  self._bCloseRows.isValid()  and  self._bCloseColumns.isValid()  and  self._numColumns.isValid()
		else:
			return False



	def _p_getPoint(self, event, existingVertexIndex, target):
		mesh = self._o_getMesh()
		pick = event.viewport.createPick()
		position = event.viewport.computeDrawPoint()
		self._pointHighlightPosition = position

		self._bTargetValid = True

		if event.viewport.pointerStatus.bShiftKey:
			if mesh is not None:
				vertexIndex, thruFaceIndex = mesh.pickVertex( pick.toMPick() )

				if vertexIndex != existingVertexIndex  or  target != MeshDrawQuadsPointTarget.EXISTINGVERTEX:
					target = MeshDrawQuadsPointTarget.EXISTINGVERTEX
					if vertexIndex != -1:
						self._vertexHighlightPosition = mesh.getVertexPosition( vertexIndex )
						self._pointHighlightPosition = self._vertexHighlightPosition
						self._bTargetValid = self.state.checkVertex( vertexIndex )
					else:
						self._vertexHighlightPosition = None
					self._view.postRedraw()
					existingVertexIndex = vertexIndex
				elif vertexIndex == -1:
					self._view.postRedraw()
			else:
				existingVertexIndex = None
				self._vertexHighlightPosition = None
				self._view.postRedraw()
		else:
			target = MeshDrawQuadsPointTarget.NEWPOINT
			existingVertexIndex = None
			self._vertexHighlightPosition = None
			self._view.postRedraw()

		return pick, position, existingVertexIndex, target




	def _p_getPointFromUser(self):
		self._vertexHighlightPosition = None
		self._bTargetValid = True
		self._pointHighlightPosition = None

		pick = None
		position = None
		existingVertexIndex = None
		target = None



		event = self._coEventHandler.waitForNextEvent()

		bNumColumnsSet = self._bNumColumnsSet.value




		# Get the point
		while event.eventType != MeshDrawQuadsManipulator.EVENT_BUTTONDOWN  or  not self._bTargetValid:
			if event.eventType == MeshDrawQuadsManipulator.EVENT_RESET:
				self._view.postRedraw()
				raise MeshDrawQuadsManipulator.ResetException

			if event.eventType == MeshDrawQuadsManipulator.EVENT_CONTEXT:
				if not bNumColumnsSet:
					self._numColumns.value = len( self._pointList.value )
					self._bNumColumnsSet.value = True
					bNumColumnsSet = True
					self._p_commandHistoryBreak()
					self._view.postRedraw()
				else:
					self._view.postRedraw()
					self._view.removeContextButtonListener( self, Viewport3d )
					raise MeshDrawQuadsManipulator.TerminateException


			if event.eventType != MeshDrawQuadsManipulator.EVENT_LEAVE:
				pick, position, existingVertexIndex, target = self._p_getPoint( event, existingVertexIndex, target )
			else:
				self._pointHighlightPosition = None
				self._view.postRedraw()

			event = self._coEventHandler.waitForNextEvent()


		if self._bCloseRows.value:
			direction = MeshDrawQuadsDirection.FORWARD
		else:
			direction = MeshDrawQuadsDirection.PINGPONG



		drawPoint = MeshDrawQuadsPoint( deepcopy( pick ), deepcopy( position ), target, direction )
		self._pointList.value = self._pointList.value + [ drawPoint ]
		if not bNumColumnsSet:
			self._numColumns.value = len( self._pointList.value )  +  1
		self._p_commandHistoryBreak()




	def _p_drawLoop(self):
		try:
			# The context button can be pressed by the user to finish the first row, or finish drawing
			self._view.addContextButtonListener( self, Viewport3d )

			while True:
				try:
					self._p_getPointFromUser()
				except MeshDrawQuadsManipulator.ResetException:
					self._view.postRedraw()
		except MeshDrawQuadsManipulator.TerminateException:
			self._bComplete = True



	def _p_manipulatorLoop(self):
		while True:
			self._p_drawLoop()

			event = self._coEventHandler.waitForNextEvent()
			while event.eventType != MeshDrawQuadsManipulator.EVENT_BUTTONDOWN:
				event = self._coEventHandler.waitForNextEvent()
			self._bComplete = False




	def evPaint3d(self, viewport, paintLayer):
		super( MeshDrawQuadsManipulator, self ).evPaint3d( viewport, paintLayer )

		if not self._bComplete:
			if self._vertexHighlightPosition is not None:
				if self._bTargetValid:
					ModelDraw.drawVertexPickHighlight( self._vertexHighlightPosition, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer )
				else:
					ModelDraw.drawVertexPickHighlight( self._vertexHighlightPosition, ModelDraw.INVALID_TARGET_HIGHLIGHT_COLOUR, paintLayer )

			if self._pointHighlightPosition is not None   and   self._vertexHighlightPosition is None:
				if paintLayer == PaintLayer.PAINTLAYER_WIREFRAME  or  paintLayer == PaintLayer.PAINTLAYER_TRANSPARENCY:
					ModelDraw.drawVertexReticle( self._pointHighlightPosition, paintLayer == PaintLayer.PAINTLAYER_TRANSPARENCY )


			self._p_updateDisplay()

			for point in self._displayRow0Points:
				if point is not None:
					ModelDraw.drawCircleHighlight( point, ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )

			for point in self._displayRow1Points:
				if point is not None:
					ModelDraw.drawCircleHighlight( point, ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )

			ModelDraw.drawSegments( self._displayRowSegs, ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )
			ModelDraw.drawSegments( self._displayColumnSegs, ModelDraw.MODIFICATION2_HIGHLIGHT_COLOUR, paintLayer )

			if self._displayRow0ClosingSeg is not None:
				ModelDraw.drawSegment( self._displayRow0ClosingSeg, ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )

			if self._pointHighlightPosition is not None:
				column = self.state.getColumn()
				bDirectionForward = self.state.isRowDirectionForward()


				prevInRow = None
				nextInRow = None

				if len( self._displayRow1Points ) > 0:
					if bDirectionForward:
						if column > 0:
							prevInRow = self._displayRow1Points[column-1]
							if self.state.areRowsClosed()   and   column  ==  self.state.getNumColumns() - 1:
								nextInRow = self._displayRow1Points[0]
					else:
						if column < ( self.state.getNumColumns() - 1 ):
							prevInRow = self._displayRow1Points[column+1]
							if self.state.areRowsClosed():
								nextInRow = self._displayRow1Points[-1]
				if prevInRow is not None:
					ModelDraw.drawSegment( Segment3( self._pointHighlightPosition, prevInRow ), ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )
				if nextInRow is not None:
					ModelDraw.drawSegment( Segment3( self._pointHighlightPosition, nextInRow ), ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )

				try:
					pointInPrevRow = self._displayRow0Points[column]
				except IndexError:
					pass
				else:
					ModelDraw.drawSegment( Segment3( self._pointHighlightPosition, pointInPrevRow ), ModelDraw.MODIFICATION2_HIGHLIGHT_COLOUR, paintLayer )

				if self.state.areColumnsClosed()  and  self.state.getNumColumns() > 0  and  self.state.getNumRows() > 1:
					pointInFirstRow = self.state.getQuadMeshVertexPosition( column, 0 )
					if pointInFirstRow is not None:
						ModelDraw.drawSegment( Segment3( self._pointHighlightPosition, pointInFirstRow ), ModelDraw.MODIFICATION2_HIGHLIGHT_COLOUR, paintLayer )
			else:
				if self._displayRow1ClosingSeg is not None:
					ModelDraw.drawSegment( self._displayRow1ClosingSeg, ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR, paintLayer )



	def _p_rowSegments(self, pointList):
		if len( pointList ) < 2:
			return []
		else:
			return [ Segment3( a, b )    for a, b in zip( pointList[:-1], pointList[1:] )   if a is not None and b is not None ]

	def _p_rowClosingSegment(self, pointList):
		if len( pointList ) >= 3:
			a, b = pointList[-1], pointList[0]
			if a is not None  and  b is not None:
				return Segment3( a, b )
		return None

	def _p_columnSegments(self, row0, row1):
		return [ Segment3( a, b )    for a, b in zip( row0, row1 )   if a is not None and b is not None ]


	def _p_updateDisplay(self):
		if self._bDisplayRequiresRefresh:
			numColumns = self.state.getNumColumns()
			if not self._bNumColumnsSet.value:
				numColumns -= 1

			numRows = self.state.getNumRows()
			bCloseRows = self.state.areRowsClosed()
			bCloseColumns = self.state.areColumnsClosed()
			row = self.state.getRow()
			if not self._bNumColumnsSet.value:
				row = 0

			rowPrev = row - 1
			if rowPrev >= 0  and  rowPrev < numRows:
				self._displayRow0Points = [ self.state.getQuadMeshVertexPosition( x, rowPrev )   for x in xrange( 0, numColumns ) ]
			else:
				self._displayRow0Points = []

			if row >= 0  and  row < numRows:
				self._displayRow1Points = [ self.state.getQuadMeshVertexPosition( x, row )   for x in xrange( 0, numColumns ) ]
			else:
				self._displayRow1Points = []

			self._displayRowSegs = self._p_rowSegments( self._displayRow0Points )
			self._displayRowSegs += self._p_rowSegments( self._displayRow1Points )
			self._displayColumnSegs = self._p_columnSegments( self._displayRow0Points, self._displayRow1Points )

			if bCloseRows:
				self._displayRow0ClosingSeg = self._p_rowClosingSegment( self._displayRow0Points )
				self._displayRow1ClosingSeg = self._p_rowClosingSegment( self._displayRow1Points )
			else:
				self._displayRow0ClosingSeg = None
				self._displayRow1ClosingSeg = None

			if bCloseColumns  and  numRows > 2:
				firstRow = [ self.state.getQuadMeshVertexPosition( x, 0 )   for x in xrange( 0, numColumns ) ]
				self._displayColumnSegs += self._p_columnSegments( self._displayRow1Points, firstRow )

			self._bDisplayRequiresRefresh = False




	def evPrimaryButtonDown(self, viewport):
		super( MeshDrawQuadsManipulator, self ).evPrimaryButtonDown( viewport )
		if self._bSensitive:
			self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_BUTTONDOWN, viewport=viewport ) )

	def evPrimaryButtonUp(self, viewport):
		super( MeshDrawQuadsManipulator, self ).evPrimaryButtonDown( viewport )
		if self._bSensitive:
			self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_BUTTONUP, viewport=viewport ) )


	def evMotion(self, viewport, point):
		super( MeshDrawQuadsManipulator, self ).evMotion( viewport, point )
		if self._bSensitive:
			self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_MOTION, viewport=viewport, point=point ) )

	def evEnter(self, viewport, point):
		super( MeshDrawQuadsManipulator, self ).evEnter( viewport, point )
		if self._bSensitive:
			self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_MOTION, viewport=viewport, point=point ) )

	def evLeave(self, viewport, point):
		super( MeshDrawQuadsManipulator, self ).evLeave( viewport, point )
		if self._bSensitive:
			self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_LEAVE, viewport=viewport, point=point ) )

	def evContextButton(self, viewport):
		if self._bSensitive:
			self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_CONTEXT, viewport=viewport ) )


	def _p_onVarChanged(self):
		self._bDisplayRequiresRefresh = True
		self._view.postRedraw()

	def _p_onPointListChanged(self):
		self._o_reset()

	def _o_reset(self):
		self._bDisplayRequiresRefresh = True
		self._coEventHandler.sendEvent( MeshDrawQuadsManipulator.Event( MeshDrawQuadsManipulator.EVENT_RESET ) )





class MeshDrawQuadsColumnManipulator (GSProcedureTool.GSProcedureToolColumnManipulator):
	def __init__(self, pointListColumn, bCloseRowsColumn, bCloseColumnsColumn, numColumnsColumn, bNumColumnsSetColumn):
		super( MeshDrawQuadsColumnManipulator, self ).__init__()
		self._pointListColumn = pointListColumn
		self._bCloseRowsColumn = bCloseRowsColumn
		self._bCloseColumnsColumn = bCloseColumnsColumn
		self._numColumnsColumn = numColumnsColumn
		self._bNumColumnsSetColumn = bNumColumnsSetColumn

	def createElement(self, rowEditContext):
		return MeshDrawQuadsManipulator()

	def initialiseElement(self, element, rowEditContext):
		pointList = self._p_getVar( self._pointListColumn, rowEditContext )
		bCloseRows = self._p_getVar( self._bCloseRowsColumn, rowEditContext )
		bCloseColumns = self._p_getVar( self._bCloseColumnsColumn, rowEditContext )
		numColumns = self._p_getVar( self._numColumnsColumn, rowEditContext )
		bNumColumnsSet = self._p_getVar( self._bNumColumnsSetColumn, rowEditContext )
		element.attachVariables( pointList, bCloseRows, bCloseColumns, numColumns, bNumColumnsSet )
		element.state = rowEditContext.rowEditor._adjustableInvoker.state
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory

	def shutdownElement(self, element, rowEditContext):
		element.detachVariables()
		element.commandHistory = None
		element.state = None


	def installElementEventHandlers(self, element, rowEditContext):
		element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		element.manipDetachFromView()


	def attachInputProductSource(self, element, source):
		element.attachMeshSource( source )

	def detachInputProductSource(self, element, source):
		element.detachMeshSource()





class DrawQuadsAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshDrawQuads

	def __init__(self, *args, **kwargs):
		super( DrawQuadsAdjustableInvoker, self ).__init__( *args, **kwargs )
		self.state = MeshDrawQuadsState()


	def main(self):
		if isinstance( self._inputObject, GSProductMesh ):
			self.mesh = deepcopy( self._inputObject )
		else:
			self.mesh = GSProductMesh()
		cppPoints = CppMeshDrawQuadsPointList()
		pyMeshDrawQuadsPointList_to_MeshDrawQuadsPointList( [ point.toCppMeshDrawQuadsPoint()  for  point in self._proc.pointList ], cppPoints )
		self.mesh.drawQuadsWithState( cppPoints, self._proc.bCloseRows, self._proc.bCloseColumns, self._proc.numColumns, self._proc.bFlip, self.state  )

	def getResult(self):
		return self.mesh




class ToolMeshDrawQuads (GSProcedureTool.GSProcedureTool):
	row = RowRefColumn( ProcMeshDrawQuads )

	functionClass = ProcMeshDrawQuads
	adjustableInvoker = DrawQuadsAdjustableInvoker
	pageTitle = _( 'DrwQMsh' )
	titleText = _( 'Draw Quad Mesh' )


	targetListEditor = TargetListColumnEditor( row.pointList )
	bCloseRowsCheck = BoolColumnEditCheck( row.bCloseRows, _( 'Close rows' ) )
	bCloseColumnsCheck = BoolColumnEditCheck( row.bCloseColumns, _( 'Close columns' ) )
	bFlipCheck = BoolColumnEditCheck( row.bFlip, _( 'Flip' ) )

	pointListManip = MeshDrawQuadsColumnManipulator( row.pointList, row.bCloseRows, row.bCloseColumns, row.numColumns, row.bNumColumnsSet )
	pointListManip.enable()



	def __init__(self):
		super( ToolMeshDrawQuads, self ).__init__()
		self._applyTemplatesButton = None


	def createUI(self):
		super( ToolMeshDrawQuads, self ).createUI()
		self._applyTemplatesButton = gtk.Button( _( 'Apply drawing templates' ) )
		self._applyTemplatesButton.show()
		self._applyTemplatesButton.connect( 'clicked', self._p_onApplyDrawingTemplates )

	def destroyUI(self):
		super( ToolMeshDrawQuads, self ).destroyUI()
		self._applyTemplatesButton = None


	def layout(self):
		layout = FormLayout()
		layout.row()  <<  _( 'Click to place vertices. After' )
		layout.row()  <<  _( 'placing the first row of vertices,' )
		layout.row()  <<  _( 'right-click to mark the end.' )
		layout.row()  <<  _( 'Further clicks will place vertices' )
		layout.row()  <<  _( 'in subsequent rows.' )
		layout.row()  <<  _( 'Right-click a second time to' )
		layout.row()  <<  _( 'finish.' )
		layout.row()  <<  _( 'After this, you can continue' )
		layout.row()  <<  _( 'by clicking once.' )
		layout.row()
		layout.row()  <<  _( '<shift>-click to weld to an' )
		layout.row()  <<  _( 'existing vertex in the mesh.' )
		layout.row()  <<  _( 'Note: this only works on' )
		layout.row()  <<  _( 'boundaries (first and last rows' )
		layout.row()  <<  _( 'and columns)' )
		layout.row()
		layout.row()  <<  self.targetListEditor.numTargets
		layout.row()  <<  None  <<  ( self.targetListEditor.removeTarget, 2 )  <<  None
		layout.row()
		layout.row()  <<  self.bCloseRowsCheck.check
		layout.row()  <<  self.bCloseColumnsCheck.check
		layout.row()
		layout.row()  <<  self.bFlipCheck.check
		layout.row()
		layout.row()  <<  self._applyTemplatesButton
		layout.row()  <<  _( '(Use if you forgot to enable' )
		layout.row()  <<  _( 'drawing templates before drawing.)' )
		layout.row()
		layout.finalise()
		return layout.getWidget()



	def _p_onApplyDrawingTemplates(self, button):
		viewport = self.view.currentViewport
		if viewport is not None:
			if isinstance( viewport, Viewport3d ):
				self._commandHistory.finishCommand()
				points = deepcopy( self.procedure.pointList )
				for point in points:
					ray = point.pick.computeRay()
					pos = viewport.raytraceDrawingTemplates( ray )
					if pos is not None:
						point.position = pos
				self.procedure.pointList = points
				self._commandHistory.finishCommand()

