##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import copy, deepcopy

from OpenGL.GL import *
from OpenGL.GLU import *

from gSculpt.Math.Math import Colour3f, Vector3, Matrix4, Plane

from gSculpt.Graphics.Graphics import glColor4Colour3fA, glVertex3Point3

from gSculpt.FileIO.IOXml import IOXmlEnum, ioXmlReadBoolNode, ioXmlWriteBoolNode, ioXmlReadFloatNode, ioXmlWriteFloatNode, ioObjectFactoryRegister, ioObjectFactoryRegisterPrimitive

from gSculpt.View.View import PaintLayer

from gSculpt.WorkArea.Viewport import ViewportEventHandler, ViewportScrollListener
from gSculpt.WorkArea.Viewport3d import Viewport3d, Viewport3dPainter

from gSculpt.gSculptConfig import gSculptConfig

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Util.CoEventHandler import CoEventHandler

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.Enum import Enum
from gSculpt.Kernel.Variable import VarInterface, Variable
from gSculpt.Kernel.ColumnEditors import BasicColumnWidgetEditor

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.ProceduralCore.GSFunction import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Model import ModelDraw
from gSculpt.Model.TargetList import TargetList
from gSculpt.Model.TargetListColumnEditor import TargetListColumnEditor
from gSculpt.Model.Pick3d import Pick3d

from gSculpt.Mesh.Mesh import GSProductMesh, CppMeshSurfaceTweakComponent, CppMeshSurfaceTweakComponentList, pyMeshSurfaceTweakComponentList_to_CppMeshSurfaceTweakComponentList, MeshSurfaceTweakComponentTarget, MeshSurfaceTweakComponentMode, MeshSurfaceTweakAdjuster, MeshSurfaceTweakNormalAdjuster

from gSculpt.Manipulator.Manipulator import Manipulator

from gSculpt.MeshTools.MeshEditComponent import MeshEditComponent







# XML IO for MeshSurfaceTweakComponentTarget

targetIOXmlEnum = IOXmlEnum( MeshSurfaceTweakComponentTarget, 'MeshSurfaceTweakComponentTarget', MeshSurfaceTweakComponentTarget.VERTICES )
targetIOXmlEnum.value( MeshSurfaceTweakComponentTarget.VERTICES, 'vertices' )
targetIOXmlEnum.value( MeshSurfaceTweakComponentTarget.EDGES, 'edges' )
targetIOXmlEnum.value( MeshSurfaceTweakComponentTarget.EDGERINGS, 'edge_rings' )
targetIOXmlEnum.value( MeshSurfaceTweakComponentTarget.EDGELOOPS, 'edge_loops' )
targetIOXmlEnum.value( MeshSurfaceTweakComponentTarget.FACES, 'faces' )

ioXmlReadMeshSurfaceTweakComponentTargetProp = targetIOXmlEnum.propReader
ioXmlReadMeshSurfaceTweakComponentTargetNode = targetIOXmlEnum.nodeReader
ioXmlWriteMeshSurfaceTweakComponentTargetProp = targetIOXmlEnum.propWriter
ioXmlWriteMeshSurfaceTweakComponentTargetNode = targetIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshSurfaceTweakComponentTarget', MeshSurfaceTweakComponentTarget )


# Variable for MeshSurfaceTweakComponentTarget

class MeshSurfaceTweakComponentTargetVarInterface (VarInterface):
	valueClass = MeshSurfaceTweakComponentTarget

class MeshSurfaceTweakComponentTargetVariable (Variable):
	valueClass = MeshSurfaceTweakComponentTarget

class MeshSurfaceTweakComponentTargetField (Field):
	valueClass = MeshSurfaceTweakComponentTarget
	defaultValue = MeshSurfaceTweakComponentTarget.VERTICES
	xmlNodeValueReader = ioXmlReadMeshSurfaceTweakComponentTargetNode
	xmlNodeValueWriter = ioXmlWriteMeshSurfaceTweakComponentTargetNode

registerFieldType( MeshSurfaceTweakComponentTarget, MeshSurfaceTweakComponentTargetField )


# Variable editor for MeshSurfaceTweakComponentTarget

class MeshSurfaceTweakComponentTargetVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshSurfaceTweakComponentTargetVarEditRadioGroup, self ).__init__()
		self.vertices = self.addChoice( MeshSurfaceTweakComponentTarget.VERTICES, _( 'Vertices' ) )
		self.edges = self.addChoice( MeshSurfaceTweakComponentTarget.EDGES, _( 'Edges' ) )
		self.edgeRings = self.addChoice( MeshSurfaceTweakComponentTarget.EDGERINGS, _( 'Edge rings' ) )
		self.edgeLoops = self.addChoice( MeshSurfaceTweakComponentTarget.EDGELOOPS, _( 'Edge loops' ) )
		self.faces = self.addChoice( MeshSurfaceTweakComponentTarget.FACES, _( 'Faces' ) )


# Column editor for MeshSurfaceTweakComponentTarget

class MeshSurfaceTweakComponentTargetColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshSurfaceTweakComponentTargetColumnEditRadioGroup, self ).__init__( [ 'vertices', 'edges', 'edgeRings', 'edgeLoops', 'faces' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshSurfaceTweakComponentTargetVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshSurfaceTweakComponentTargetColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshSurfaceTweakComponentTargetColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()









# XML IO for MeshSurfaceTweakComponentMode

modeIOXmlEnum = IOXmlEnum( MeshSurfaceTweakComponentMode, 'MeshSurfaceTweakComponentMode', MeshSurfaceTweakComponentMode.SURFACE )
modeIOXmlEnum.value( MeshSurfaceTweakComponentMode.SURFACE, 'surface' )
modeIOXmlEnum.value( MeshSurfaceTweakComponentMode.NORMAL, 'normal' )

ioXmlReadMeshSurfaceTweakComponentModeProp = modeIOXmlEnum.propReader
ioXmlReadMeshSurfaceTweakComponentModeNode = modeIOXmlEnum.nodeReader
ioXmlWriteMeshSurfaceTweakComponentModeProp = modeIOXmlEnum.propWriter
ioXmlWriteMeshSurfaceTweakComponentModeNode = modeIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshSurfaceTweakComponentMode', MeshSurfaceTweakComponentMode )


# Variable for MeshSurfaceTweakComponentMode

class MeshSurfaceTweakComponentModeVarInterface (VarInterface):
	valueClass = MeshSurfaceTweakComponentMode

class MeshSurfaceTweakComponentModeVariable (Variable):
	valueClass = MeshSurfaceTweakComponentMode






#
# MeshSurfaceTweakComponent class
#

class MeshSurfaceTweakComponent (object):
	def __init__(self, pick=None, target=MeshSurfaceTweakComponentTarget.VERTICES, mode=MeshSurfaceTweakComponentMode.SURFACE, bClearSelection=True,
				viewVector=None, displacementAlongSurface=None, displacementAlongNormal=None):
		super( MeshSurfaceTweakComponent, self ).__init__()
		if pick is None:
			pick = Pick3d()
		self.pick = pick
		self.target = target
		self.mode = mode
		self.bClearSelection = bClearSelection
		if viewVector is None:
			viewVector = Vector3()
		self.viewVector = copy( viewVector )
		if displacementAlongSurface is None:
			displacementAlongSurface = Vector3()
		self.displacementAlongSurface = copy( displacementAlongSurface )
		if displacementAlongNormal is None:
			displacementAlongNormal = 0.0
		self.displacementAlongNormal = displacementAlongNormal


	def toCppMeshSurfaceTweakComponent(self):
		return CppMeshSurfaceTweakComponent( self.pick.toMPick(), self.target, self.mode, self.bClearSelection, self.viewVector, self.displacementAlongSurface, self.displacementAlongNormal )


	def __readxml__(self, xmlNode):
		xmlNode.getChild( 'pick' )  >>  self.pick
		self.target = ioXmlReadMeshSurfaceTweakComponentTargetNode( xmlNode.getChild( 'target' ), self.target )
		self.mode = ioXmlReadMeshSurfaceTweakComponentModeNode( xmlNode.getChild( 'mode' ), self.mode )
		self.bClearSelection = ioXmlReadBoolNode( xmlNode.getChild( 'b_clear_selection' ), self.bClearSelection )
		xmlNode.getChild( 'view_vector' )  >>  self.viewVector
		xmlNode.getChild( 'displacement_along_surface' )  >>  self.displacementAlongSurface
		self.displacementAlongNormal = ioXmlReadFloatNode( xmlNode.getChild( 'displacement_along_normal' ), self.displacementAlongNormal )

	def __writexml__(self, xmlNode):
		xmlNode.addChild( 'pick' )  <<  self.pick
		ioXmlWriteMeshSurfaceTweakComponentTargetNode( xmlNode.addChild( 'target' ), self.target )
		ioXmlWriteMeshSurfaceTweakComponentModeNode( xmlNode.addChild( 'mode' ), self.mode )
		ioXmlWriteBoolNode( xmlNode.addChild( 'b_clear_selection' ), self.bClearSelection )
		xmlNode.addChild( 'view_vector' )  <<  self.viewVector
		xmlNode.addChild( 'displacement_along_surface' )  <<  self.displacementAlongSurface
		ioXmlWriteFloatNode( xmlNode.addChild( 'displacement_along_normal' ), self.displacementAlongNormal )


	def __copy__(self):
		return MeshSurfaceTweakComponent( copy( self.pick ), self.target, self.mode, self.bClearSelection,						\
										copy( self.viewVector ), copy( self.displacementAlongSurface ), self.displacementAlongNormal )

ioObjectFactoryRegister( 'MeshSurfaceTweakComponent', MeshSurfaceTweakComponent )










class ProcMeshSurfaceTweakComponents (GSProcedure):
	description = _( 'Surface tweak components' )

	tweaks = FieldColumn( TargetList, TargetList() )

	inputClass = GSProductMesh



	def procedureInvoke(self, inputObject, errorHandler):
		cppTweaks = CppMeshSurfaceTweakComponentList()
		pyMeshSurfaceTweakComponentList_to_CppMeshSurfaceTweakComponentList( [ tweak.toCppMeshSurfaceTweakComponent()   for tweak in self.tweaks ], cppTweaks )
		inputObject.surfaceTweakComponents( cppTweaks, True )
		return inputObject






class MeshSurfaceTweakComponentsManipulator (Manipulator, ViewportEventHandler, ViewportScrollListener, Viewport3dPainter):
	EV_BUTTON_DOWN = 0
	EV_BUTTON_UP = 1
	EV_MOTION = 2
	EV_LEAVE = 3
	EV_TARGET = 4
	EV_RESET = 5
	EV_OUTPUTCHANGED = 6


	class Event (object):
		def __init__(self, eventType, **kwargs):
			self.eventType = eventType
			self.__dict__.update( kwargs )


	class ResetException:
		pass



	def __init__(self):
		super( MeshSurfaceTweakComponentsManipulator, self ).__init__()

		self._tweaksVar = None
		self._targetVar = None

		self._tweakTarget = None
		self._mode = MeshSurfaceTweakComponentMode.SURFACE

		self._coEventHandler = None

		self._highlightVertexPosition = None
		self._highlightEdgeSegment = None
		self._highlightEdgeRingOrLoopSegments = []
		self._highlightFaceTriangles = None
		self._initialTargetCentre = None

		self._f_onAdjustTweak = None
		self._f_onDebugDraw = None

		self._meshSource = None

		self._o_setUISensitivity( False )




	def manipAttachToView(self, view):
		assert self._view is None, 'already attached to a view'
		super( MeshSurfaceTweakComponentsManipulator, self ).manipAttachToView( view )
		self._coEventHandler = CoEventHandler( self._p_tweakLoop )
		self._coEventHandler.start()
		self._view.addEventHandler( self, Viewport3d )
		self._view.addScrollListener( self, Viewport3d )
		self._view.addPainter( self, Viewport3d )
		self._view.postRedraw()

	def manipDetachFromView(self):
		assert self._view is not None, 'not attached to a view'
		self._view.removeEventHandler( self, Viewport3d )
		self._view.removeScrollListener( self, Viewport3d )
		self._view.removePainter( self, Viewport3d )
		self._view.postRedraw()
		self._coEventHandler.terminate()
		self._coEventHandler = None
		super( MeshSurfaceTweakComponentsManipulator, self ).manipDetachFromView()



	def attachVariables(self, tweaks, target):
		self._tweaksVar = tweaks
		self._targetVar = target
		self._targetVar.changedSignal.connect( self._p_onTarget )

	def detachVariables(self):
		self._targetVar.changedSignal.connect( self._p_onTarget )
		self._tweaksVar = None
		self._targetVar = None


	def _o_setUISensitivity(self, bSensitive):
		pass



	def attachMeshSource(self, source):
		assert self._meshSource is None, 'mesh source already attached'
		self._meshSource = source
		self._meshSource.changedSignal.connect( self._p_onOutputMeshSourceValueChanged )

	def detachMeshSource(self):
		assert self._meshSource is not None, 'no mesh source attached'
		self._meshSource.changedSignal.disconnect( self._p_onOutputMeshSourceValueChanged )
		self._meshSource = None


	def attachInputMeshSource(self, source):
		source.changedSignal.connect( self._p_onMeshSourceValueChanged )

	def detachInputMeshSource(self, source):
		source.changedSignal.disconnect( self._p_onMeshSourceValueChanged )



	def _p_getTarget(self, viewport, targetComponentIndex, ringSeedToRing, loopSeedToLoop, ringSeedToRingSegs, loopSeedToLoopSegs):
		mesh = self._p_getMesh()
		targetPick = viewport.createPick()

		target = self._targetVar.value
		if target == MeshSurfaceTweakComponentTarget.VERTICES:
			# Target vertices
			vertexIndex, thruFaceIndex = mesh.pickVertex( targetPick.toMPick() )

			if vertexIndex != targetComponentIndex:
				if vertexIndex != -1:
					# Highlight the vertex
					self._highlightVertexPosition = mesh.getVertexPosition( vertexIndex )
					self._initialTargetCentre = self._highlightVertexPosition
				else:
					self._highlightVertexPosition = None
					self._initialTargetCentre = None
				self._view.postRedraw()

			targetComponentIndex = vertexIndex
		elif target == MeshSurfaceTweakComponentTarget.EDGES:
			edgeIndex, closestPoint, thruFaceIndex = mesh.pickEdge( targetPick.toMPick() )

			if edgeIndex != targetComponentIndex:
				if edgeIndex != -1:
					# Highlight the edge
					self._highlightEdgeSegment = mesh.getEdgeSegment( edgeIndex )
					self._initialTargetCentre = self._highlightEdgeSegment.getMidPoint()
				else:
					self._highlightEdgeSegment = None
					self._initialTargetCentre = None
				self._view.postRedraw()

			targetComponentIndex = edgeIndex
		elif target == MeshSurfaceTweakComponentTarget.EDGERINGS  or  target == MeshSurfaceTweakComponentTarget.EDGELOOPS:
			seedIndex, closestPoint, thruFaceIndex = mesh.pickEdge( targetPick.toMPick() )


			if seedIndex != targetComponentIndex:
				if seedIndex != -1:
					def getHighlight(seedToList, seedToSegs, discoverFunction):
						segs = []
						try:
							segs = seedToSegs[seedIndex]
						except KeyError:
							try:
								edgeIndices = seedToList[seedIndex]
							except KeyError:
								edgeIndices = discoverFunction( seedIndex )
								seedToList[seedIndex] = edgeIndices
							segs = [ mesh.getEdgeSegment( edgeIndex )   for edgeIndex in edgeIndices ]
							seedToSegs[seedIndex] = segs
						return segs

					# Highlight the edges
					if target == MeshSurfaceTweakComponentTarget.EDGERINGS:
						self._highlightEdgeRingOrLoopSegments = getHighlight( ringSeedToRing, ringSeedToRingSegs, mesh.discoverEdgeRing )
					elif target == MeshSurfaceTweakComponentTarget.EDGELOOPS:
						self._highlightEdgeRingOrLoopSegments = getHighlight( loopSeedToLoop, loopSeedToLoopSegs, mesh.discoverEdgeLoop )
					self._initialTargetCentre = mesh.getEdgeSegment( seedIndex ).getMidPoint()
				else:
					self._highlightEdgeRingOrLoopSegments = []
					self._initialTargetCentre = None
				self._view.postRedraw()

			targetComponentIndex = seedIndex
		elif target == MeshSurfaceTweakComponentTarget.FACES:
			faceIndex, intersectionPoint = mesh.pickFace( targetPick.toMPick() )

			if faceIndex != targetComponentIndex:
				if faceIndex != -1:
					# Highlight the face
					self._highlightFaceTriangles = mesh.getFaceTriangles( faceIndex )
					self._initialTargetCentre = mesh.getFaceCentroid( faceIndex )
				else:
					self._highlightFaceTriangles = None
					self._initialTargetCentre = None
				self._view.postRedraw()

			targetComponentIndex = faceIndex
		else:
			assert False, 'invalid target'

		return targetPick, targetComponentIndex



	def _p_getTweakFromUser(self, ringSeedToRing, loopSeedToLoop):
		self._highlightVertexPosition = None
		self._highlightEdgeSegment = None
		self._highlightEdgeRingOrLoopSegments = []
		self._highlightFaceTriangles = None
		self._initialTargetCentre = None
		self._tweakTarget = None
		self._mode = MeshSurfaceTweakComponentMode.SURFACE

		targetComponentIndex = -1
		targetPick = None
		displacementAlongSurface = Vector3()
		displacementAlongNormal = 0.0
		targetDistance = None
		ringSeedToRingSegs = {}
		loopSeedToLoopSegs = {}
		viewport = None
		viewVector = None

		event = self._coEventHandler.waitForNextEvent()

		# Get the target component
		while not ( event.eventType == MeshSurfaceTweakComponentsManipulator.EV_BUTTON_DOWN  and  targetComponentIndex != -1 ):
			if event.eventType == MeshSurfaceTweakComponentsManipulator.EV_RESET:
				self._view.postRedraw()
				raise MeshSurfaceTweakComponentsManipulator.ResetException

			if event.eventType == MeshSurfaceTweakComponentsManipulator.EV_TARGET  or  event.eventType == MeshSurfaceTweakComponentsManipulator.EV_OUTPUTCHANGED:
				self._highlightVertexPosition = None
				self._highlightEdgeSegment = None
				self._highlightEdgeRingOrLoopSegments = []
				self._highlightFaceTriangles = None
				targetComponentIndex = -1
				if event.eventType == MeshSurfaceTweakComponentsManipulator.EV_OUTPUTCHANGED:
					ringSeedToRingSegs = {}
					loopSeedToLoopSegs = {}
			else:
				viewport = event.viewport

				if event.eventType != MeshSurfaceTweakComponentsManipulator.EV_LEAVE  and  viewport is not None:
					targetPick, targetComponentIndex = self._p_getTarget( viewport, targetComponentIndex, ringSeedToRing, loopSeedToLoop, ringSeedToRingSegs, loopSeedToLoopSegs )
					viewVector = viewport.camera.cameraPosition.forwardAxis

			event = self._coEventHandler.waitForNextEvent()


		if viewport.pointerStatus.bShiftKey:
			self._mode = MeshSurfaceTweakComponentMode.NORMAL
		else:
			self._mode = MeshSurfaceTweakComponentMode.SURFACE
		self._tweakTarget = self._targetVar.value


		# We have a target component
		# Remove the highlight
		self._highlightVertexPosition = None
		self._highlightEdgeSegment = None
		self._highlightEdgeRingOrLoopSegments = []
		self._highlightFaceTriangles = None

		# If the selection is valid
		if self._initialTargetCentre is not None:
			targetDistance = event.viewport.camera.transformWorldToCameraZ( self._initialTargetCentre )
			startPoint = event.viewport.pointerStatus.positionF
			startPointWorld = event.viewport.camera.transformScreenToWorldCameraZ( startPoint, targetDistance )
			bScaledMotion = event.bScaledMotion

			# Add the tweak object to the tweak list
			self._tweaksVar.value = self._tweaksVar.value + [ MeshSurfaceTweakComponent( deepcopy( targetPick ), self._targetVar.value, self._mode, not event.viewport.pointerStatus.bControlKey, viewVector, Vector3(), 0.0 ) ]
			self._view.forceRedraw()
		else:
			return


		event = self._coEventHandler.waitForNextEvent()

		# Now, track the displacement
		while not( event.eventType == MeshSurfaceTweakComponentsManipulator.EV_BUTTON_UP  and  event.bScaledMotion == bScaledMotion ):
			if event.eventType == MeshSurfaceTweakComponentsManipulator.EV_RESET:
				self._view.postRedraw()
				raise MeshSurfaceTweakComponentsManipulator.ResetException

			if not( event.eventType == MeshSurfaceTweakComponentsManipulator.EV_TARGET  or  event.eventType == MeshSurfaceTweakComponentsManipulator.EV_OUTPUTCHANGED ):
				endPoint = event.viewport.pointerStatus.positionF
				endPointWorld = event.viewport.camera.transformScreenToWorldCameraZ( endPoint, targetDistance )

				displacementAlongSurface = endPointWorld - startPointWorld
				displacementAlongNormal = event.viewport.camera.transformDistanceAcrossScreenToDistanceThruWorld( endPoint.x - startPoint.x, targetDistance )

				if bScaledMotion:
					displacementAlongSurface *= gSculptConfig.manipulatorFineTuneMotionScale
					displacementAlongNormal *= gSculptConfig.manipulatorFineTuneMotionScale

				if self._f_onAdjustTweak is not None:
					self._f_onAdjustTweak( self._tweakTarget, self._mode, viewVector, displacementAlongSurface, displacementAlongNormal )
				self._view.forceRedraw()

			event = self._coEventHandler.waitForNextEvent()


		assert len( self._tweaksVar.value ) > 0, 'no tweaks'
		topTweak = self._tweaksVar.value[-1]
		newTopTweak = MeshSurfaceTweakComponent( topTweak.pick, topTweak.target, topTweak.mode, topTweak.bClearSelection,					\
											topTweak.viewVector, displacementAlongSurface, displacementAlongNormal )
		tweaks = self._tweaksVar.value
		tweaks[-1] = newTopTweak
		self._tweaksVar.value = tweaks
		self._p_commandHistoryBreak()



	def _p_tweakLoop(self):
		ringSeedToRing = {}
		loopSeedToLoop = {}
		while True:
			try:
				self._p_getTweakFromUser( ringSeedToRing, loopSeedToLoop )
			except MeshSurfaceTweakComponentsManipulator.ResetException:
				print 'MeshSurfaceTweakComponentsManipulator._p_tweakLoop: reset'
				ringSeedToRing = {}
				loopSeedToLoop = {}
				self._view.postRedraw()



	def evPrimaryButtonDown(self, viewport):
		super( MeshSurfaceTweakComponentsManipulator, self ).evPrimaryButtonDown( viewport )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_BUTTON_DOWN, viewport=viewport, bScaledMotion=False ) )

	def evPrimaryButtonUp(self, viewport):
		super( MeshSurfaceTweakComponentsManipulator, self ).evPrimaryButtonUp( viewport )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_BUTTON_UP, viewport=viewport, bScaledMotion=False ) )


	def evSecondaryButtonDown(self, viewport):
		super( MeshSurfaceTweakComponentsManipulator, self ).evPrimaryButtonDown( viewport )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_BUTTON_DOWN, viewport=viewport, bScaledMotion=True ) )

	def evSecondaryButtonUp(self, viewport):
		super( MeshSurfaceTweakComponentsManipulator, self ).evPrimaryButtonUp( viewport )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_BUTTON_UP, viewport=viewport, bScaledMotion=True ) )


	def evMotion(self, viewport, point):
		super( MeshSurfaceTweakComponentsManipulator, self ).evMotion( viewport, point )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_MOTION, viewport=viewport ) )

	def evEnter(self, viewport, point):
		super( MeshSurfaceTweakComponentsManipulator, self ).evMotion( viewport, point )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_MOTION, viewport=viewport ) )

	def evLeave(self, viewport, point):
		super( MeshSurfaceTweakComponentsManipulator, self ).evMotion( viewport, point )
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_LEAVE, viewport=viewport ) )


	def evScroll(self, viewport, x, y):
		v = self._targetVar.value
		v -= y
		while v < MeshSurfaceTweakComponentTarget.VERTICES:
			v += ( MeshSurfaceTweakComponentTarget.FACES + 1 )
		while v > MeshSurfaceTweakComponentTarget.FACES:
			v -= ( MeshSurfaceTweakComponentTarget.FACES + 1 )
		self._targetVar.value = MeshSurfaceTweakComponentTarget( v )




	def evPaint3d(self, viewport, paintLayer):
		# Paint the target highlight
		if self._highlightVertexPosition is not None:
			ModelDraw.drawVertexPickHighlight( self._highlightVertexPosition, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer )
		if self._highlightEdgeSegment is not None:
			ModelDraw.drawEdgePickHighlight( self._highlightEdgeSegment, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer )
		for seg in self._highlightEdgeRingOrLoopSegments:
			ModelDraw.drawEdgePickHighlight( seg, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer )
		if self._highlightFaceTriangles is not None:
			ModelDraw.drawFacePickHighlight( self._highlightFaceTriangles, ModelDraw.TARGET_HIGHLIGHT_COLOUR, paintLayer )
		if self._f_onDebugDraw is not None:
			self._f_onDebugDraw( paintLayer )



	def _p_onTarget(self):
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_TARGET ) )

	def _p_onConstraints(self):
		self._view.postRedraw()


	def _p_onMeshSourceValueChanged(self):
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_RESET ) )

	def _p_onOutputMeshSourceValueChanged(self):
		self._coEventHandler.sendEvent( MeshSurfaceTweakComponentsManipulator.Event( MeshSurfaceTweakComponentsManipulator.EV_OUTPUTCHANGED ) )

	def _p_getMesh(self):
		if self._meshSource is not None:
			return self._meshSource.getValueAsClass( GSProductMesh )
		else:
			return None





class MeshSurfaceTweakComponentsColumnManipulator (GSProcedureTool.GSProcedureToolColumnManipulator):
	def __init__(self, tweaksColumn, targetColumn):
		super( MeshSurfaceTweakComponentsColumnManipulator, self ).__init__()
		self._tweaksColumn = tweaksColumn
		self._targetColumn = targetColumn


	def createElement(self, rowEditContext):
		return MeshSurfaceTweakComponentsManipulator()

	def destroyElement(self, element, rowEditContext):
		del rowEditContext.rowEditor.tweakManip

	def initialiseElement(self, element, rowEditContext):
		tweaks = self._p_getVar( self._tweaksColumn, rowEditContext )
		target = self._p_getVar( self._targetColumn, rowEditContext )
		element.attachVariables( tweaks, target )
		if rowEditContext.commandHistory is not None:
			element.commandHistory = rowEditContext.commandHistory


	def shutdownElement(self, element, rowEditContext):
		element.detachVariables()
		element.commandHistory = None


	def installElementEventHandlers(self, element, rowEditContext):
		element.manipAttachToView( rowEditContext.view )

	def uninstallElementEventHandlers(self, element, rowEditContext):
		element.manipDetachFromView()


	def attachInputProductSource(self, element, source):
		element.attachInputMeshSource( source )

	def detachInputProductSource(self, element, source):
		element.detachInputMeshSource( source )

	def attachOutputProductSource(self, element, source):
		element.attachMeshSource( source )

	def detachOutputProductSource(self, element, source):
		element.detachMeshSource()




class TweakComponentsAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshSurfaceTweakComponents

	def init(self):
		# Initialise with full pre-compute. This will remove the cost of the full pre-compute which would normally occur on each modification of the tweak list
		self._inputObject.preComputeFull()

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		cppTweaks = CppMeshSurfaceTweakComponentList()
		pyMeshSurfaceTweakComponentList_to_CppMeshSurfaceTweakComponentList( [ tweak.toCppMeshSurfaceTweakComponent()   for tweak in self._proc.tweaks ], cppTweaks )
		self.mesh.surfaceTweakComponents( cppTweaks, False )
		self.surfaceAdjusters = None
		self.normalAdjusters = None


	def adjustTweak(self, tweakTarget, mode, viewVector, displacementAlongSurface, displacementAlongNormal):
		if mode == MeshSurfaceTweakComponentMode.SURFACE:
			if self.surfaceAdjusters is None:
				self.surfaceAdjusters = MeshSurfaceTweakAdjuster()
				if tweakTarget == MeshSurfaceTweakComponentTarget.VERTICES:
					self.mesh.surfaceTweakAdjustMarkedVertices( self.surfaceAdjusters, viewVector )
				elif tweakTarget == MeshSurfaceTweakComponentTarget.EDGES  or								\
						tweakTarget == MeshSurfaceTweakComponentTarget.EDGERINGS  or						\
						tweakTarget == MeshSurfaceTweakComponentTarget.EDGELOOPS:
					self.mesh.surfaceTweakAdjustMarkedEdges( self.surfaceAdjusters, viewVector )
				if tweakTarget == MeshSurfaceTweakComponentTarget.FACES:
					self.mesh.surfaceTweakAdjustMarkedFaces( self.surfaceAdjusters, viewVector )
			self.surfaceAdjusters.tweak( displacementAlongSurface )
		elif mode == MeshSurfaceTweakComponentMode.NORMAL:
			if self.normalAdjusters is None:
				self.normalAdjusters = MeshSurfaceTweakNormalAdjuster()
				if tweakTarget == MeshSurfaceTweakComponentTarget.VERTICES:
					self.mesh.surfaceTweakNormalAdjustMarkedVertices( self.normalAdjusters )
				elif tweakTarget == MeshSurfaceTweakComponentTarget.EDGES  or								\
						tweakTarget == MeshSurfaceTweakComponentTarget.EDGERINGS  or						\
						tweakTarget == MeshSurfaceTweakComponentTarget.EDGELOOPS:
					self.mesh.surfaceTweakNormalAdjustMarkedEdges( self.normalAdjusters )
				if tweakTarget == MeshSurfaceTweakComponentTarget.FACES:
					self.mesh.surfaceTweakNormalAdjustMarkedFaces( self.normalAdjusters )
			self.normalAdjusters.tweak( displacementAlongNormal )

	def debugDraw(self, paintLayer):
		if self.surfaceAdjusters is not None:
			if paintLayer == PaintLayer.PAINTLAYER_TRANSPARENCY:
				self.surfaceAdjusters.debugDraw()


	def getResult(self):
		return self.mesh






class ToolMeshSurfaceTweakComponents (GSProcedureTool.GSProcedureTool):
	row = RowRefColumn( ProcMeshSurfaceTweakComponents )

	functionClass = ProcMeshSurfaceTweakComponents
	adjustableInvoker = TweakComponentsAdjustableInvoker
	pageTitle = _( 'SurfTwk' )
	titleText = _( 'Surface Tweak Components' )


	tweakTarget = FieldColumn( MeshSurfaceTweakComponentTarget, MeshSurfaceTweakComponentTarget.VERTICES )


	tweakListEditor = TargetListColumnEditor( row.tweaks,
									targetLabelText=_( '%d tweak' ), pluralTargetLabelText=_( '%d tweaks' ), removeTargetButtonText=_( 'Remove tweak' ) )

	tweakManip = MeshSurfaceTweakComponentsColumnManipulator( row.tweaks, tweakTarget )
	tweakManip.enable()

	targetRadio = MeshSurfaceTweakComponentTargetColumnEditRadioGroup( tweakTarget )



	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Click and drag to move along' )
	layout.row()  <<  _( 'surface. Hold <shift> to move' )
	layout.row()  <<  _( 'along surface normal.' )
	layout.row()  <<  _( 'Selection is cleared on each click,' )
	layout.row()  <<  _( 'unless <control> is pressed.' )
	layout.row()  <<  _( 'Use the scroll wheel to switch' )
	layout.row()  <<  _( 'between component types.' )
	layout.row()
	layout.row()  <<  _( 'Component type:' )
	layout.row()  <<  targetRadio.vertices
	layout.row()  <<  targetRadio.edges
	layout.row()  <<  targetRadio.edgeRings
	layout.row()  <<  targetRadio.edgeLoops
	layout.row()  <<  targetRadio.faces
	layout.row()
	layout.row()  <<  tweakListEditor.numTargets
	layout.row()  <<  None  <<  ( tweakListEditor.removeTarget, 2 )  <<  None
	layout.row()
	layout.finalise()



	def initialise(self):
		super( ToolMeshSurfaceTweakComponents, self ).initialise()
		self.tweakManip._f_onAdjustTweak = self._onAdjustTweak
		#self.tweakManip._f_onDebugDraw = self._onDebugDraw


	def shutdown(self):
		self.tweakManip._f_onAdjustTweak = None
		#self.tweakManip._f_onDebugDraw = None
		super( ToolMeshSurfaceTweakComponents, self ).shutdown()



	def _onAdjustTweak(self, tweakTarget, mode, viewVector, displacementAlongSurface, displacementAlongNormal):
		self._adjustableInvoker.adjustTweak( tweakTarget, mode, viewVector, displacementAlongSurface, displacementAlongNormal )

	def _onDebugDraw(self, paintLayer):
		self._adjustableInvoker.debugDraw( paintLayer )




	def setTarget(self, target):
		self.tweakTarget = target


