##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Manipulator.ColumnManipulators import *
from gSculpt.Manipulator.RealManipulator import RealManipulator

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshBevelVerticesAdjuster, GSProductMesh




class ProcMeshBevelVertices (GSProcedure):
	description = _( 'Bevel vertices' )

	bevel = FieldColumn( float, 0.0 )
	bUseExistingTexturing = FieldColumn( bool, True )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshBevelVerticesAdjuster()
		inputObject.bevelMarkedVertices( adjuster, self.bUseExistingTexturing, self.bMarkBoundaryEdges, self.bNSharpBoundaryEdges )
		adjuster.setParameters( self.bevel )
		return inputObject



class BevelVerticesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshBevelVertices

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshBevelVerticesAdjuster()
		self.mesh.bevelMarkedVertices( self.adjuster, self._proc.bUseExistingTexturing, self._proc.bMarkBoundaryEdges, self._proc.bNSharpBoundaryEdges )

	def adjust(self):
		self.adjuster.setParameters( self._proc.bevel )

	def getResult(self):
		return self.mesh




class ToolMeshBevelVertices (GSProcedureTool):
	row = RowRefColumn( ProcMeshBevelVertices )

	functionClass = ProcMeshBevelVertices
	adjustableInvoker = BevelVerticesAdjustableInvoker
	pageTitle = _( 'Bevel' )
	titleText = _( 'Bevel Vertices' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedVerticesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	bevelEntry = FloatColumnEditEntry( row.bevel, 0.0, 1.0e8, 0.1, 10.0, 4 )
	bUseExistingTexturingCheck = BoolColumnEditCheck( row.bUseExistingTexturing, _( 'Use existing texturing' ) )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'N-sharp boundary edges' ) )

	bevelManip = Real1ColumnManipulator( row.bevel, 0.0, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									snapping=ColumnManipulatorSnappingDistance() )
	bevelManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Bevel:' )  <<  bevelEntry.entry
	layout.row()
	layout.row()  <<  bUseExistingTexturingCheck.check
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()
	layout.finalise()
