##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import GSProductMesh



class ProcMeshDissolveVertices (GSProcedure):
	description = _( 'Dissolve vertices' )

	bMarkCreatedFaces = FieldColumn( bool, False )
	bTwoEdgesOnly = FieldColumn( bool, False )
	bTwoFacesOnly = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		inputObject.dissolveMarkedVertices( self.bTwoEdgesOnly, self.bTwoFacesOnly, self.bMarkCreatedFaces )
		return inputObject



class ToolMeshDissolveVertices (GSProcedureTool):
	row = RowRefColumn( ProcMeshDissolveVertices )

	functionClass = ProcMeshDissolveVertices
	pageTitle = _( 'Dissolve' )
	titleText = _( 'Dissolve Vertices' )

	bMarkCreatedFacesCheck = BoolColumnEditCheck( row.bMarkCreatedFaces, _( 'Mark created faces' ) )
	bTwoEdgesOnlyCheck = BoolColumnEditCheck( row.bTwoEdgesOnly, _( 'Two edges only' ) )
	bTwoFacesOnlyCheck = BoolColumnEditCheck( row.bTwoFacesOnly, _( 'Two faces only' ) )

	layout = FormLayoutFactory()
	layout.row()  <<  bMarkCreatedFacesCheck.check
	layout.row()  <<  bTwoEdgesOnlyCheck.check
	layout.row()  <<  bTwoFacesOnlyCheck.check
	layout.row()
	layout.finalise()
