##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Manipulator.ColumnManipulators import *
from gSculpt.Manipulator.RealManipulator import RealManipulator

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshExpandVerticesAdjuster, MeshExpandVerticesInMarkedFacesAdjuster, GSProductMesh



class ProcMeshExpandVertices (GSProcedure):
	description = _( 'Expand vertices' )

	bMarkedFacesOnly = FieldColumn( bool, False )
	expand = FieldColumn( float, 0.0 )
	extrude = FieldColumn( float, 0.0 )
	bMarkBoundaryEdges = FieldColumn( bool, False )
	bNSharpBoundaryEdges = FieldColumn( bool, True )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		if self.bMarkedFacesOnly:
			adjuster = MeshExpandVerticesInMarkedFacesAdjuster()
			inputObject.expandMarkedVerticesInMarkedFaces( adjuster, self.bMarkBoundaryEdges )
			adjuster.setParameters( self.expand )
		else:
			adjuster = MeshExpandVerticesAdjuster()
			inputObject.expandMarkedVertices( adjuster, self.bMarkBoundaryEdges, self.bNSharpBoundaryEdges )
			adjuster.setParameters( self.expand, self.extrude )
		return inputObject



class ExpandVerticesAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshExpandVertices

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		if self._proc.bMarkedFacesOnly:
			self.adjuster = MeshExpandVerticesInMarkedFacesAdjuster()
			self.mesh.expandMarkedVerticesInMarkedFaces( self.adjuster, self._proc.bMarkBoundaryEdges )
		else:
			self.adjuster = MeshExpandVerticesAdjuster()
			self.mesh.expandMarkedVertices( self.adjuster, self._proc.bMarkBoundaryEdges, self._proc.bNSharpBoundaryEdges )

	def adjust(self):
		if self._proc.bMarkedFacesOnly:
			self.adjuster.setParameters( self._proc.expand )
		else:
			self.adjuster.setParameters( self._proc.expand, self._proc.extrude )

	def getResult(self):
		return self.mesh




class ToolMeshExpandVertices (GSProcedureTool):
	row = RowRefColumn( ProcMeshExpandVertices )

	functionClass = ProcMeshExpandVertices
	adjustableInvoker = ExpandVerticesAdjustableInvoker
	pageTitle = _( 'Expand' )
	titleText = _( 'Expand Vertices' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedVerticesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	bMarkedFacesCheck = BoolColumnEditCheck( row.bMarkedFacesOnly, _( 'Marked faces only' ) )
	expandEntry = FloatColumnEditEntry( row.expand, 0.0, 1.0e8, 0.1, 10.0, 4 )
	extrudeEntry = FloatColumnEditEntry( row.extrude, -1.0e8, 1.0e8, 0.1, 10.0, 4 )
	bMarkBoundaryEdgesCheck = BoolColumnEditCheck( row.bMarkBoundaryEdges, _( 'Mark boundary edges' ) )
	bNSharpBoundaryEdgesCheck = BoolColumnEditCheck( row.bNSharpBoundaryEdges, _( 'N-sharp boundary edges' ) )

	expandManip = Real2ColumnManipulator( row.expand, 0.0, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									row.extrude, -1.0e8, 1.0e8, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									xSnapping=ColumnManipulatorSnappingDistance(), ySnapping=ColumnManipulatorSnappingDistance() )
	expandManip.enable()

	extrudeEntry.inverseSensitivity = row.bMarkedFacesOnly
	bNSharpBoundaryEdgesCheck.inverseSensitivity = row.bMarkedFacesOnly

	layout = FormLayoutFactory()
	layout.row()  <<  bMarkedFacesCheck.check
	layout.row()  <<  _( 'Expand:' )  <<  expandEntry.entry
	layout.row()  <<  _( 'Extrude:' )  <<  extrudeEntry.entry
	layout.row()
	layout.row()  <<  bMarkBoundaryEdgesCheck.check
	layout.row()  <<  bNSharpBoundaryEdgesCheck.check
	layout.row()
	layout.finalise()
