##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Vector3

from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *
from gSculpt.Kernel.Enum import Enum

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Mesh.Mesh import GSProductMesh




# MeshFlattenVerticesMode enumeration
class MeshFlattenVerticesMode (Enum):
	X = 0
	Y = 1
	Z = 2
	NORMAL = 3




# XML IO for MeshFlattenVerticesMode

flattenModeIOXmlEnum = IOXmlEnum( MeshFlattenVerticesMode, 'MeshFlattenVerticesMode', MeshFlattenVerticesMode.NORMAL )
flattenModeIOXmlEnum.value( MeshFlattenVerticesMode.X, 'x' )
flattenModeIOXmlEnum.value( MeshFlattenVerticesMode.Y, 'y' )
flattenModeIOXmlEnum.value( MeshFlattenVerticesMode.Z, 'z' )
flattenModeIOXmlEnum.value( MeshFlattenVerticesMode.NORMAL, 'normal' )

ioXmlReadMeshFlattenVerticesModeProp = flattenModeIOXmlEnum.propReader
ioXmlReadMeshFlattenVerticesModeNode = flattenModeIOXmlEnum.nodeReader
ioXmlWriteMeshFlattenVerticesModeProp = flattenModeIOXmlEnum.propWriter
ioXmlWriteMeshFlattenVerticesModeNode = flattenModeIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshFlattenVerticesMode', MeshFlattenVerticesMode )


# Field for MeshFlattenVerticesMode

class MeshFlattenVerticesModeField (Field):
	valueClass = MeshFlattenVerticesMode
	defaultValue = MeshFlattenVerticesMode.NORMAL
	xmlNodeValueReader = ioXmlReadMeshFlattenVerticesModeNode
	xmlNodeValueWriter = ioXmlWriteMeshFlattenVerticesModeNode

registerFieldType( MeshFlattenVerticesMode, MeshFlattenVerticesModeField )


# Variable editor for MeshFlattenVerticesMode

class MeshFlattenVerticesModeVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshFlattenVerticesModeVarEditRadioGroup, self ).__init__()
		self.x = self.addChoice( MeshFlattenVerticesMode.X, _( 'X-axis' ) )
		self.y = self.addChoice( MeshFlattenVerticesMode.Y, _( 'Y-axis' ) )
		self.z = self.addChoice( MeshFlattenVerticesMode.Z, _( 'Z-axis' ) )
		self.normal = self.addChoice( MeshFlattenVerticesMode.NORMAL, _( 'Normal' ) )


# Column editor for MeshFlattenVerticesMode

class MeshFlattenVerticesModeColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshFlattenVerticesModeColumnEditRadioGroup, self ).__init__( [ 'x', 'y', 'z', 'normal' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshFlattenVerticesModeVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshFlattenVerticesModeColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshFlattenVerticesModeColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()





class ProcMeshFlattenVertices (GSProcedure):
	description = _( 'Flatten vertices' )

	bPerRegion = FieldColumn( bool, True )
	mode = FieldColumn( MeshFlattenVerticesMode, MeshFlattenVerticesMode.NORMAL )


	inputClass = GSProductMesh
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		mode = self.mode
		if mode == MeshFlattenVerticesMode.X:
			inputObject.flattenVertices( Vector3( 1.0, 0.0, 0.0 ), self.bPerRegion )
		elif mode == MeshFlattenVerticesMode.Y:
			inputObject.flattenVertices( Vector3( 0.0, 1.0, 0.0 ), self.bPerRegion )
		elif mode == MeshFlattenVerticesMode.Z:
			inputObject.flattenVertices( Vector3( 0.0, 0.0, 1.0 ), self.bPerRegion )
		elif mode == MeshFlattenVerticesMode.NORMAL:
			inputObject.flattenVerticesNormal( self.bPerRegion )
		return inputObject




class ToolMeshFlattenVertices (GSProcedureTool):
	row = RowRefColumn( ProcMeshFlattenVertices )

	functionClass = ProcMeshFlattenVertices
	pageTitle = _( 'Flatten' )
	titleText = _( 'Flatten vertices' )

	bPerRegionCheck = BoolColumnEditCheck( row.bPerRegion, _( 'Per region' ) )
	modeRadio = MeshFlattenVerticesModeColumnEditRadioGroup( row.mode )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Flatten mode:' )
	layout.row()  <<  modeRadio.x
	layout.row()  <<  modeRadio.y
	layout.row()  <<  modeRadio.z
	layout.row()  <<  modeRadio.normal
	layout.row()
	layout.row()  <<  bPerRegionCheck.check
	layout.row()
	layout.finalise()
