##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.FileIO.IOXml import IOXmlEnum, ioObjectFactoryRegisterPrimitive

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.VarEdit.VarEdit import ChoiceVarEditRadioGroup

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import BasicColumnWidgetEditor
from gSculpt.Kernel.ColumnEditors import *
from gSculpt.Kernel.Enum import Enum

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSFunctionTool import *
from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool

from gSculpt.Model.Model import MarkPredicate
from gSculpt.Model.MarkPredicateColumnEditor import MarkPredicateColumnEditRadioGroup

from gSculpt.Mesh.Mesh import GSProductMesh




# MeshMarkVerticesTarget enumeration
class MeshMarkVerticesTarget (Enum):
	ALL = 0
	EDGEMARKED = 1
	FACEMARKED = 2
	FACEMARKBOUNDARY = 3
	FACEMARKINTERNAL = 4
	BOUNDARY = 5
	INTERNAL = 6




# XML IO for MeshMarkVerticesTarget

markTargetIOXmlEnum = IOXmlEnum( MeshMarkVerticesTarget, 'MeshMarkVerticesTarget', MeshMarkVerticesTarget.ALL )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.ALL, 'all' )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.EDGEMARKED, 'edge_marked' )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.FACEMARKED, 'face_marked' )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.FACEMARKBOUNDARY, 'face_mark_boundary' )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.FACEMARKINTERNAL, 'face_mark_internal' )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.BOUNDARY, 'boundary' )
markTargetIOXmlEnum.value( MeshMarkVerticesTarget.INTERNAL, 'internal' )

ioXmlReadMeshMarkVerticesTargetProp = markTargetIOXmlEnum.propReader
ioXmlReadMeshMarkVerticesTargetNode = markTargetIOXmlEnum.nodeReader
ioXmlWriteMeshMarkVerticesTargetProp = markTargetIOXmlEnum.propWriter
ioXmlWriteMeshMarkVerticesTargetNode = markTargetIOXmlEnum.nodeWriter

ioObjectFactoryRegisterPrimitive( 'MeshMarkVerticesTarget', MeshMarkVerticesTarget )


# Field for MeshMarkVerticesTarget

class MeshMarkVerticesTargetField (Field):
	valueClass = MeshMarkVerticesTarget
	defaultValue = MeshMarkVerticesTarget.ALL
	xmlNodeValueReader = ioXmlReadMeshMarkVerticesTargetNode
	xmlNodeValueWriter = ioXmlWriteMeshMarkVerticesTargetNode

registerFieldType( MeshMarkVerticesTarget, MeshMarkVerticesTargetField )


# Variable editor for MeshMarkVerticesTarget

class MeshMarkVerticesTargetVarEditRadioGroup (ChoiceVarEditRadioGroup):
	def __init__(self):
		super( MeshMarkVerticesTargetVarEditRadioGroup, self ).__init__()
		self.all = self.addChoice( MeshMarkVerticesTarget.ALL, _( 'All' ) )
		self.edgeMarked = self.addChoice( MeshMarkVerticesTarget.EDGEMARKED, _( 'With selected edges' ) )
		self.faceMarked = self.addChoice( MeshMarkVerticesTarget.FACEMARKED, _( 'With selected faces' ) )
		self.faceMarkBoundary = self.addChoice( MeshMarkVerticesTarget.FACEMARKBOUNDARY, _( 'On face selection boundary' ) )
		self.faceMarkInternal = self.addChoice( MeshMarkVerticesTarget.FACEMARKINTERNAL, _( 'All faces selected' ) )
		self.boundary = self.addChoice( MeshMarkVerticesTarget.BOUNDARY, _( 'Boundary' ) )
		self.internal = self.addChoice( MeshMarkVerticesTarget.INTERNAL, _( 'Internal' ) )


# Column editor for MeshMarkVerticesTarget

class MeshMarkVerticesTargetColumnEditRadioGroup (BasicColumnWidgetEditor):
	def __init__(self, fieldColumn):
		super( MeshMarkVerticesTargetColumnEditRadioGroup, self ).__init__( [ 'all', 'edgeMarked', 'faceMarked', 'faceMarkBoundary', 'faceMarkInternal', 'boundary', 'internal' ] )
		self._fieldColumn = fieldColumn


	def createElement(self, rowEditContext):
		return MeshMarkVerticesTargetVarEditRadioGroup()

	def initialiseElement(self, element, rowEditContext):
		super( MeshMarkVerticesTargetColumnEditRadioGroup, self ).initialiseElement( element, rowEditContext )
		field = self._p_getVar( self._fieldColumn, rowEditContext )
		element.attachVariable( field )

	def shutdownElement(self, element, rowEditContext):
		super( MeshMarkVerticesTargetColumnEditRadioGroup, self ).shutdownElement( element, rowEditContext )
		element.detachVariable()





class ProcMeshMarkVertices (GSProcedure):
	predicate = FieldColumn( MarkPredicate, MarkPredicate.MARK )
	target = FieldColumn( MeshMarkVerticesTarget, MeshMarkVerticesTarget.ALL )


	def getDescription(self):
		if self.predicate == MarkPredicate.MARK:
			return _( 'Select vertices' )
		elif self.predicate == MarkPredicate.UNMARK:
			return _( 'Unselect vertices' )
		elif self.predicate == MarkPredicate.ADD:
			return _( 'Select vertices' )
		elif self.predicate == MarkPredicate.FILTER:
			return _( 'Filter vertex selection' )
		elif self.predicate == MarkPredicate.INVERT:
			return _( 'Invert vertex selection' )


	inputClass = GSProductMesh
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		target = self.target
		if target == MeshMarkVerticesTarget.ALL:
			inputObject.markVertices_all( self.predicate )
		elif target == MeshMarkVerticesTarget.EDGEMARKED:
			inputObject.markVertices_edgeMarked( self.predicate )
		elif target == MeshMarkVerticesTarget.FACEMARKED:
			inputObject.markVertices_faceMarked( self.predicate )
		elif target == MeshMarkVerticesTarget.FACEMARKBOUNDARY:
			inputObject.markVertices_faceMarkBoundary( self.predicate )
		elif target == MeshMarkVerticesTarget.FACEMARKINTERNAL:
			inputObject.markVertices_faceMarkInternal( self.predicate )
		elif target == MeshMarkVerticesTarget.BOUNDARY:
			inputObject.markVertices_boundary( self.predicate )
		elif target == MeshMarkVerticesTarget.INTERNAL:
			inputObject.markVertices_internal( self.predicate )
		return inputObject



class ToolMeshMarkVertices (GSProcedureTool):
	row = RowRefColumn( ProcMeshMarkVertices )

	functionClass = ProcMeshMarkVertices
	pageTitle = _( 'Select' )
	titleText = _( 'Select vertices' )

	predicateRadio = MarkPredicateColumnEditRadioGroup( row.predicate )
	targetRadio = MeshMarkVerticesTargetColumnEditRadioGroup( row.target )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Select mode:' )
	layout.row()  <<  predicateRadio.mark
	layout.row()  <<  predicateRadio.unmark
	layout.row()  <<  predicateRadio.add
	layout.row()  <<  predicateRadio.filter
	layout.row()  <<  predicateRadio.invert
	layout.row()
	layout.row()  <<  _( 'Target:' )
	layout.row()  <<  targetRadio.all
	layout.row()  <<  targetRadio.edgeMarked
	layout.row()  <<  targetRadio.faceMarked
	layout.row()  <<  targetRadio.faceMarkBoundary
	layout.row()  <<  targetRadio.faceMarkInternal
	layout.row()  <<  targetRadio.boundary
	layout.row()  <<  targetRadio.internal
	layout.row()
	layout.finalise()

