##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Manipulator.RealManipulator import RealManipulator

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.RealManipulator import RealManipulator
from gSculpt.Manipulator.ColumnManipulators import RealColumnManipulator, Real1ColumnManipulator, ColumnManipulatorSnappingDistance

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import MeshMoveNormalAdjuster, GSProductMesh




class ProcMeshMoveVerticesNormal (GSProcedure):
	description = _( 'Move vertices normal' )

	distance = FieldColumn( float, 0.0 )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		adjuster = MeshMoveNormalAdjuster()
		inputObject.moveVerticesNormal( adjuster )
		adjuster.setParameters( self.distance )
		return inputObject



class MoveVerticesNormalAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcMeshMoveVerticesNormal

	def main(self):
		self.mesh = deepcopy( self._inputObject )
		self.adjuster = MeshMoveNormalAdjuster()
		self.mesh.moveVerticesNormal( self.adjuster )

	def adjust(self):
		self.adjuster.setParameters( self._proc.distance )

	def getResult(self):
		return self.mesh




class ToolMeshMoveVerticesNormal (GSProcedureTool):
	row = RowRefColumn( ProcMeshMoveVerticesNormal )

	functionClass = ProcMeshMoveVerticesNormal
	adjustableInvoker = MoveVerticesNormalAdjustableInvoker
	pageTitle = _( 'Mv.Nml.' )
	titleText = _( 'Move Normal' )


	def _p_focalPoint(self):
		return self.inputProduct.getMarkedVerticesCentre()

	focalPoint = FunctionColumn( _p_focalPoint )

	distanceEntry = FloatColumnEditEntry( row.distance, -1.0e18, 1.0e18, 0.001, 0.1, 4 )

	distanceManip = Real1ColumnManipulator( row.distance, -1.0e18, 1.0e18, RealColumnManipulator.WorldSpaceScaleFactory( focalPoint ),
									snapping=ColumnManipulatorSnappingDistance() )
	distanceManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Distance:' )  <<  distanceEntry.entry
	layout.row()
	layout.finalise()
