##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import GSProductMesh



class ProcMeshWeldVertices (GSProcedure):
	description = _( 'Weld vertices' )

	threshold = FieldColumn( float, 0.0 )
	bWeldLinkedVertices = FieldColumn( bool, True )
	bMarkMergedEdges = FieldColumn( bool, False )

	inputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		inputObject.weldMarkedVertices( self.threshold, self.bWeldLinkedVertices, self.bMarkMergedEdges )
		return inputObject



class ToolMeshWeldVertices (GSProcedureTool):
	row = RowRefColumn( ProcMeshWeldVertices )

	functionClass = ProcMeshWeldVertices
	pageTitle = _( 'Weld' )
	titleText = _( 'Weld Vertices' )

	thresholdEntry = FloatColumnEditEntry( row.threshold, 0.0, 1.0e8, 0.01, 1.0, 4 )
	bWeldLinkedVerticesCheck = BoolColumnEditCheck( row.bWeldLinkedVertices, _( 'Weld linked vertices' ) )
	bMarkMergedEdgesCheck = BoolColumnEditCheck( row.bMarkMergedEdges, _( 'Mark merged edges' ) )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Threshold:' )  <<  thresholdEntry.entry
	layout.row()  <<  bWeldLinkedVerticesCheck.check
	layout.row()  <<  bMarkMergedEdgesCheck.check
	layout.row()
	layout.finalise()
