##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
import pygtk
pygtk.require( '2.0' )
import gtk

from copy import copy

from TargetList import TargetList

from gSculpt.VarEdit.VarEdit import WidgetSingleVarEdit



class TargetListVarEditor (WidgetSingleVarEdit):
	def __init__(self, targetLabelText=_( '%d target' ), pluralTargetLabelText=_( '%d targets' ), removeTargetButtonText=_( 'Remove target' )):
		super( TargetListVarEditor, self ).__init__()
		self.commandHistory = None

		self.numTargets = gtk.Label( pluralTargetLabelText )
		self._targetLabelText = targetLabelText
		self._pluralTargetLabelText = pluralTargetLabelText

		self.removeTarget = gtk.Button( removeTargetButtonText )
		self.removeTarget.connect( 'clicked', self._p_onRemoveTarget )

		self._o_setUISensitivity( False )


	def attachVariable(self, var):
		super( TargetListVarEditor, self ).attachVariable( var )
		if self._var.isValid():
			self._p_onVarValue( self._var.value )
		self._var.valueSignal.connect( self._p_onVarValue )

	def detachVariable(self):
		assert self._var is not None, 'no variable attached'
		self._var.valueSignal.disconnect( self._p_onVarValue )
		super( TargetListVarEditor, self ).detachVariable()


	def show(self):
		self.numTargets.show()
		self.removeTarget.show()

	def hide(self):
		self.numTargets.hide()
		self.removeTarget.hide()

	def _o_setUISensitivity(self, bSensitive):
		self.numTargets.set_sensitive( bSensitive )
		self.removeTarget.set_sensitive( bSensitive )


	def _p_onRemoveTarget(self, widget):
		assert self._var is not None
		if len( self._var.value )  >  0:
			x = copy( self._var.value )
			x[:] = x[:-1]
			self._var.value = x
			if self.commandHistory is not None:
				self.commandHistory.finishCommand()


	def _p_onVarValue(self, value):
		if value is not None:
			if len( value ) != 1:
				text = self._pluralTargetLabelText  %  ( len( value ), )
			else:
				text = self._targetLabelText  %  ( len( value ), )
			self.numTargets.set_text( text )

