##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
import math

from gSculpt.Math.Math import Quaternion

from gSculpt.UI.UIAction import UIAction

from gSculpt.ProceduralTool.ProcToolGroup import ProcToolGroup, ToolGroupActionInstanceProc, ToolGroupActionInstanceSubmenu

from gSculpt.ModelTools.ToolVirtualMirror import ProcVirtualMirror



_virtualMirrorAction = UIAction( 'Tools.Model.VirtualMirror', ( _( 'Tools' ), _( 'Model' ), _( 'Virtual mirror' ) ),
			_( 'Virtual mirror' ), _( 'Virtual mirror' ), None, _( 'Change virtual mirror settings' ) )



class ModelVirtualMirrorToolGroup (ProcToolGroup):
	def __init__(self, nodeEditor):
		super( ModelVirtualMirrorToolGroup, self ).__init__( nodeEditor )

		self._virtualMirrorAction = _virtualMirrorAction.instance( ToolGroupActionInstanceProc, self, self._p_onVirtualMirror )

		self._modelSource = None


	def layoutContents(self, layout):
		layout.row()  <<  _( 'Virtual mirror' )
		layout.row()  <<  self._virtualMirrorAction.button()  <<  None  <<  None
		layout.row()


	def attachModelSource(self, source):
		self._modelSource = source

	def detachModelSource(self):
		self._modelSource = None


	def initContextMenu(self, contextMenu):
		contextMenu.layout.row()  <<  self._virtualMirrorAction.contextMenuItem( contextMenu )


	def _p_onVirtualMirror(self):
		proc = ProcVirtualMirror()
		model = self._modelSource.getValue()

		if model is not None  and  model.bVirtualMirrorEnabled:
			proc.position = model.virtualMirrorPlane.position
			proc.rotation = model.virtualMirrorPlane.rotation
			proc.size = model.virtualMirrorPlane.size
			proc.bVisible = model.virtualMirrorPlane.bVisible
		else:
			proc.rotation = Quaternion.rotateY( math.radians( -90.0 ) )
		proc.bEnabled = True

		return proc

