##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

from gSculpt.Math.Math import Point3, Quaternion

from gSculpt.UI.FormLayout import FormLayoutFactory
from gSculpt.UI.UIAction import UIAction

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Model.Model import GSProductModel




class ProcVirtualMirror (GSProcedure):
	description = _( 'Set mirror plane' )

	position = FieldColumn( Point3, Point3() )
	rotation = FieldColumn( Quaternion, Quaternion() )
	size = FieldColumn( float, 100.0 )
	bVisible = FieldColumn( bool, True )
	bEnabled = FieldColumn( bool, True )

	inputClass = GSProductModel


	def procedureInvoke(self, inputObject, errorHandler):
		inputObject.bVirtualMirrorEnabled = self.bEnabled
		inputObject.virtualMirrorPlane.position = self.position
		inputObject.virtualMirrorPlane.rotation = self.rotation
		inputObject.virtualMirrorPlane.size = self.size
		inputObject.virtualMirrorPlane.bVisible = self.bVisible
		return inputObject



_positionAction = UIAction( 'Tools.Model.VirtualMirrorMode.Position', ( _( 'Tools' ), _( 'Model' ), _( 'Virtual mirror mode' ), _( 'Position mode' ) ),
			_( 'Position' ), _( 'Position' ), 'w', _( 'Position mode' ) )
_rotationAction = UIAction( 'Tools.Model.VirtualMirrorMode.Rotation', ( _( 'Tools' ), _( 'Model' ), _( 'Virtual mirror mode' ), _( 'Rotation mode' ) ),
			_( 'Rotation' ), _( 'Rotation' ), 'r', _( 'Rotation mode' ) )
_sizeAction = UIAction( 'Tools.Model.VirtualMirrorMode.Size', ( _( 'Tools' ), _( 'Model' ), _( 'Virtual mirror mode' ), _( 'Size mode' ) ),
			_( 'Size' ), _( 'Size' ), 'e', _( 'Size mode' ) )



class ToolVirtualMirror (GSProcedureTool):
	row = RowRefColumn( ProcVirtualMirror )

	functionClass = ProcVirtualMirror
	pageTitle = _( 'V.Mirr.' )
	titleText = _( 'Virtual Mirror' )

	positionManip = Point3ColumnManipulator( row.position, snapping=ColumnManipulatorSnappingDistance() )
	rotationManip = RotationColumnManipulator( row.rotation, row.position, snapping=ColumnManipulatorSnappingAngle() )
	sizeManip = Real1ColumnManipulator( row.size, 0.0, 1.0e18, RealColumnManipulator.WorldSpaceScaleFactory( row.position ),
									snapping=ColumnManipulatorSnappingDistance() )

	manipulatorSwitch = ColumnManipulatorSwitch( [ positionManip, rotationManip, sizeManip ] )
	manipulatorSwitch.enable()

	manipulatorSelector = ColumnManipulatorSwitchRadioGroup( manipulatorSwitch )
	positionButton = manipulatorSelector.addChoice( positionManip, _positionAction )
	rotationButton = manipulatorSelector.addChoice( rotationManip, _rotationAction )
	sizeButton = manipulatorSelector.addChoice( sizeManip, _sizeAction )

	positionEntry = Point3ColumnEditEntry( row.position, -1.0e18, 1.0e18, 0.1, 10.0, 4 )
	rotationPopup = QuaternionColumnEditDirectionPopup( row.rotation, _( 'Set normal' ) )
	rotationEntry = QuaternionColumnEditEntry( row.rotation )
	sizeEntry = FloatColumnEditEntry( row.size, 0.0, 1.0e18, 0.1, 10.0, 4 )
	bVisibleCheck = BoolColumnEditCheck( row.bVisible, _( 'Show plane' ) )
	bEnabledCheck = BoolColumnEditCheck( row.bEnabled, _( 'Enabled' ) )



	layout = FormLayoutFactory()
	layout.row()  <<  bEnabledCheck.check
	layout.row()  <<  bVisibleCheck.check
	layout.row()
	layout.row()  <<  _( 'Mouse function:' )
	layout.row()  <<  positionButton
	layout.row()  <<  rotationButton
	layout.row()  <<  sizeButton
	layout.row()
	layout.row()  <<  _( 'Position:' )
	layout.row()  <<  'X:'  <<  positionEntry.x
	layout.row()  <<  'Y:'  <<  positionEntry.y
	layout.row()  <<  'Z:'  <<  positionEntry.z
	layout.row()
	layout.row()  <<  _( 'Rotation:' )
	layout.row()  <<  rotationPopup.popup
	layout.row()  <<  'X:'  <<  rotationEntry.x
	layout.row()  <<  'Y:'  <<  rotationEntry.y
	layout.row()  <<  'Z:'  <<  rotationEntry.z
	layout.row()
	layout.row()  <<  _( 'Size:' )  <<  sizeEntry.entry
	layout.row()
	layout.finalise()


