##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.PolyBlend.PolyBlend import PolyBlend

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import RowEditor, CompositeColumnEditor
from gSculpt.Kernel.ColumnEditors import *



class PolyBlendComponent (Row):
	collapsePenalty = FieldColumn( float, 1.0 )
	twistPenalty = FieldColumn( float, 1.0 )
	inversionPenalty = FieldColumn( float, 2.0 )
	areaWeighting = FieldColumn( float, 0.5 )


	def makePolyBlend(self):
		return PolyBlend( self.collapsePenalty, self.twistPenalty, self.inversionPenalty, self.areaWeighting )



class PolyBlendComponentEditor (RowEditor):
	row = RowRefColumn( PolyBlendComponent )

	rowClass = PolyBlendComponent

	collapsePenaltyEntry = FloatColumnEditEntry( row.collapsePenalty, -1024576.0, 1024576.0, 0.01, 0.1, 4 )
	twistPenaltyEntry = FloatColumnEditEntry( row.twistPenalty, -1024576.0, 1024576.0, 0.01, 0.1, 4 )
	inversionPenaltyEntry = FloatColumnEditEntry( row.inversionPenalty, -1024576.0, 1024576.0, 0.01, 0.1, 4 )
	areaWeightingEntry = FloatColumnEditEntry( row.areaWeighting, -1024576.0, 1024576.0, 0.01, 0.1, 4 )

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Collapse penalty:' )  <<  collapsePenaltyEntry.entry
	layout.row()  <<  _( 'Twist penalty:' )  <<  twistPenaltyEntry.entry
	layout.row()  <<  _( 'Inversion penalty:' )  <<  inversionPenaltyEntry.entry
	layout.row()  <<  _( 'Area weighting:' )  <<  areaWeightingEntry.entry
	layout.finalise()



class PolyBlendComponentColumnEdit (CompositeColumnEditor):
	compositeRowEditorClass = PolyBlendComponentEditor
