##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Point3, Matrix4
from gSculpt.Math.MathField import Point3Field

from gSculpt.Model.Model import GSProductModelBlank

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.DBEdit import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import BoxPrimitive, MeshVertexList, GSProductMesh



class ProcBoxPrimitive (GSProcedure):
	description = _( 'Box' )

	xSegments = FieldColumn( int, 1 )
	ySegments = FieldColumn( int, 1 )
	zSegments = FieldColumn( int, 1 )
	width = FieldColumn( float, 100.0 )
	height = FieldColumn( float, 100.0 )
	depth = FieldColumn( float, 100.0 )
	bFront = FieldColumn( bool, True )
	bBack = FieldColumn( bool, True )
	bLeft = FieldColumn( bool, True )
	bRight = FieldColumn( bool, True )
	bBottom = FieldColumn( bool, True )
	bTop = FieldColumn( bool, True )
	position = FieldColumn( Point3, Point3() )


	inputClass = GSProductModelBlank
	outputClass = GSProductMesh


	def procedureInvoke(self, inputObject, errorHandler):
		prim = BoxPrimitive( self.xSegments, self.ySegments, self.zSegments, self.width, self.height, self.depth,
						self.bFront, self.bBack, self.bLeft, self.bRight, self.bBottom, self.bTop )
		mesh = GSProductMesh()
		verts = MeshVertexList()
		prim.createBox( mesh, verts )
		verts.transform( Matrix4.translate( self.position.toVector3() ), mesh )
		return mesh



class BoxAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcBoxPrimitive

	def initInvoke(self):
		self.verts = MeshVertexList()

	def main(self):
		prim = BoxPrimitive( self._proc.xSegments, self._proc.ySegments, self._proc.zSegments, self._proc.width, self._proc.height, self._proc.depth, self._proc.bFront, self._proc.bBack, self._proc.bLeft, self._proc.bRight, self._proc.bBottom, self._proc.bTop )
		self.verts.clear()
		self.mesh = GSProductMesh()
		prim.createBox( self.mesh, self.verts )
		return self.mesh

	def save(self):
		self.verts.savePositions( self.mesh )

	def restore(self):
		self.verts.restorePositions( self.mesh )

	def adjust(self):
		self.verts.transform( Matrix4.translate( self._proc.position.toVector3() ), self.mesh )

	def getResult(self):
		return self.mesh




class ToolBoxPrimitive (GSProcedureTool):
	row = RowRefColumn( ProcBoxPrimitive )

	functionClass = ProcBoxPrimitive
	adjustableInvoker = BoxAdjustableInvoker
	pageTitle = _( 'Box' )
	titleText = _( 'Create Box' )

	xSegsEntry = IntColumnEditEntry( row.xSegments, 1, 1048576, 1, 10 )
	ySegsEntry = IntColumnEditEntry( row.ySegments, 1, 1048576, 1, 10 )
	zSegsEntry = IntColumnEditEntry( row.zSegments, 1, 1048576, 1, 10 )
	widthEntry = FloatColumnEditEntry( row.width, 0.0, 1.0e8, 0.1, 10.0, 4 )
	heightEntry = FloatColumnEditEntry( row.height, 0.0, 1.0e8, 1.0, 10.0, 4 )
	depthEntry = FloatColumnEditEntry( row.depth, 0.0, 1.0e8, 1.0, 10.0, 4 )
	positionEntry = Point3ColumnEditEntry( row.position, -1.0e8, 1.0e8, 1.0, 10.0, 4 )
	bFrontCheck = BoolColumnEditCheck( row.bFront, _( 'Front' ) )
	bBackCheck = BoolColumnEditCheck( row.bBack, _( 'Back' ) )
	bLeftCheck = BoolColumnEditCheck( row.bLeft, _( 'Left' ) )
	bRightCheck = BoolColumnEditCheck( row.bRight, _( 'Right' ) )
	bBottomCheck = BoolColumnEditCheck( row.bBottom, _( 'Bottom' ) )
	bTopCheck = BoolColumnEditCheck( row.bTop, _( 'Top' ) )

	positionManip = Point3ColumnManipulator( row.position, snapping=ColumnManipulatorSnappingDistance() )
	positionManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'X-Segs:' )  <<  xSegsEntry.entry
	layout.row()  <<  _( 'Y-Segs:' )  <<  ySegsEntry.entry
	layout.row()  <<  _( 'Z-Segs:' )  <<  zSegsEntry.entry
	layout.row()
	layout.row()  <<  _( 'Width:' )  <<  widthEntry.entry
	layout.row()  <<  _( 'Height:' )  <<  heightEntry.entry
	layout.row()  <<  _( 'Depth:' )  <<  depthEntry.entry
	layout.row()
	layout.row()  <<  _( 'Position' )
	layout.row()  <<  'X:'  <<  positionEntry.x
	layout.row()  <<  'Y:'  <<  positionEntry.y
	layout.row()  <<  'Z:'  <<  positionEntry.z
	layout.row()
	layout.row()  <<  bFrontCheck.check  <<  bBackCheck.check
	layout.row()  <<  bLeftCheck.check  <<  bRightCheck.check
	layout.row()  <<  bBottomCheck.check  <<  bTopCheck.check
	layout.row()
	layout.finalise()
