##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Point3, Matrix4, Axis
from gSculpt.Math.MathField import Point3Field, AxisField

from gSculpt.Model.Model import GSProductModelBlank

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import PlanePrimitive, MeshVertexList, GSProductMesh



class ProcPlanePrimitive (GSProcedure):
	description = _( 'Plane' )

	axis = FieldColumn( Axis, Axis.AXIS_Y )
	uSegments = FieldColumn( int, 1 )
	vSegments = FieldColumn( int, 1 )
	width = FieldColumn( float, 100.0 )
	height = FieldColumn( float, 100.0 )
	position = FieldColumn( Point3, Point3() )


	inputClass = GSProductModelBlank
	outputClass = GSProductMesh


	def procedureInvoke(self, inputObject, errorHandler):
		prim = PlanePrimitive( self.axis, self.uSegments, self.vSegments, self.width, self.height )
		mesh = GSProductMesh()
		verts = MeshVertexList()
		prim.createPlane( mesh, verts )
		verts.transform( Matrix4.translate( self.position.toVector3() ), mesh )
		return mesh



class PlaneAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcPlanePrimitive

	def initInvoke(self):
		self.verts = MeshVertexList()

	def main(self):
		prim = PlanePrimitive( self._proc.axis, self._proc.uSegments, self._proc.vSegments, self._proc.width, self._proc.height )
		self.verts.clear()
		self.mesh = GSProductMesh()
		prim.createPlane( self.mesh, self.verts )
		return self.mesh

	def save(self):
		self.verts.savePositions( self.mesh )

	def restore(self):
		self.verts.restorePositions( self.mesh )

	def adjust(self):
		self.verts.transform( Matrix4.translate( self._proc.position.toVector3() ), self.mesh )

	def getResult(self):
		return self.mesh




class ToolPlanePrimitive (GSProcedureTool):
	row = RowRefColumn( ProcPlanePrimitive )

	functionClass = ProcPlanePrimitive
	adjustableInvoker = PlaneAdjustableInvoker
	pageTitle = _( 'Plane' )
	titleText = _( 'Create Plane' )

	axisRadio = AxisColumnEditRadioGroup( row.axis )
	uSegsEntry = IntColumnEditEntry( row.uSegments, 1, 1048576, 1, 10 )
	vSegsEntry = IntColumnEditEntry( row.vSegments, 1, 1048576, 1, 10 )
	widthEntry = FloatColumnEditEntry( row.width, 0.0, 1.0e8, 0.1, 10.0, 4 )
	heightEntry = FloatColumnEditEntry( row.height, 0.0, 1.0e8, 1.0, 10.0, 4 )
	positionEntry = Point3ColumnEditEntry( row.position, -1.0e8, 1.0e8, 1.0, 10.0, 4 )

	positionManip = Point3ColumnManipulator( row.position, snapping=ColumnManipulatorSnappingDistance() )
	positionManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Plane axis:' )
	layout.row()  <<  axisRadio.x  <<  axisRadio.y  <<  axisRadio.z
	layout.row()
	layout.row()  <<  _( 'U-Segs.:' )  <<  uSegsEntry.entry
	layout.row()  <<  _( 'V-Segs.:' )  <<  vSegsEntry.entry
	layout.row()
	layout.row()  <<  _( 'Width:' )  <<  widthEntry.entry
	layout.row()  <<  _( 'Height:' )  <<  heightEntry.entry
	layout.row()
	layout.row()  <<  _( 'Position' )
	layout.row()  <<  'X:'  <<  positionEntry.x
	layout.row()  <<  'Y:'  <<  positionEntry.y
	layout.row()  <<  'Z:'  <<  positionEntry.z
	layout.row()
	layout.finalise()
