##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Point3, Matrix4, Axis
from gSculpt.Math.MathField import Point3Field, AxisField


from gSculpt.Model.Model import GSProductModelBlank

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import SpherePrimitive, MeshVertexList, GSProductMesh



class ProcSpherePrimitive (GSProcedure):
	description = _( 'Sphere' )

	axis = FieldColumn( Axis, Axis.AXIS_Y )
	stacks = FieldColumn( int, 12 )
	slices = FieldColumn( int, 24 )
	radius = FieldColumn( float, 50.0 )
	position = FieldColumn( Point3, Point3() )


	inputClass = GSProductModelBlank
	outputClass = GSProductMesh


	def procedureInvoke(self, inputObject, errorHandler):
		prim = SpherePrimitive( self.axis, self.stacks, self.slices, self.radius )
		mesh = GSProductMesh()
		verts = MeshVertexList()
		prim.createSphere( mesh, verts )
		verts.transform( Matrix4.translate( self.position.toVector3() ), mesh )
		return mesh



class SphereAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcSpherePrimitive

	def initInvoke(self):
		self.verts = MeshVertexList()

	def main(self):
		prim = SpherePrimitive( self._proc.axis, self._proc.stacks, self._proc.slices, self._proc.radius )
		self.verts.clear()
		self.mesh = GSProductMesh()
		prim.createSphere( self.mesh, self.verts )
		return self.mesh

	def save(self):
		self.verts.savePositions( self.mesh )

	def restore(self):
		self.verts.restorePositions( self.mesh )

	def adjust(self):
		self.verts.transform( Matrix4.translate( self._proc.position.toVector3() ), self.mesh )

	def getResult(self):
		return self.mesh




class ToolSpherePrimitive (GSProcedureTool):
	row = RowRefColumn( ProcSpherePrimitive )

	functionClass = ProcSpherePrimitive
	adjustableInvoker = SphereAdjustableInvoker
	pageTitle = _( 'Sphere' )
	titleText = _( 'Create Sphere' )

	axisRadio = AxisColumnEditRadioGroup( row.axis )
	stacksEntry = IntColumnEditEntry( row.stacks, 3, 1048576, 1, 10 )
	slicesEntry = IntColumnEditEntry( row.slices, 3, 1048576, 1, 10 )
	radiusEntry = FloatColumnEditEntry( row.radius, 0.0, 1.0e8, 0.1, 10.0, 4 )
	positionEntry = Point3ColumnEditEntry( row.position, -1.0e8, 1.0e8, 1.0, 10.0, 4 )

	positionManip = Point3ColumnManipulator( row.position, snapping=ColumnManipulatorSnappingDistance() )
	positionManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Sphere axis:' )
	layout.row()  <<  axisRadio.x  <<  axisRadio.y  <<  axisRadio.z
	layout.row()
	layout.row()  <<  _( 'Stacks:' )  <<  stacksEntry.entry
	layout.row()  <<  _( 'Slices:' )  <<  slicesEntry.entry
	layout.row()
	layout.row()  <<  _( 'Radius:' )  <<  radiusEntry.entry
	layout.row()
	layout.row()  <<  _( 'Position' )
	layout.row()  <<  'X:'  <<  positionEntry.x
	layout.row()  <<  'Y:'  <<  positionEntry.y
	layout.row()  <<  'Z:'  <<  positionEntry.z
	layout.row()
	layout.finalise()
