##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Math.Math import Point3, Matrix4, Axis
from gSculpt.Math.MathField import Point3Field, AxisField


from gSculpt.Model.Model import GSProductModelBlank

from gSculpt.UI.FormLayout import FormLayoutFactory

from gSculpt.Kernel.DB import *
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.ProceduralTool.GSProcedureAdjustableInvoker import *

from gSculpt.Manipulator.ColumnManipulators import *

from gSculpt.Mesh import Mesh
from gSculpt.Mesh.Mesh import TorusPrimitive, MeshVertexList, GSProductMesh



class ProcTorusPrimitive (GSProcedure):
	description = _( 'Torus' )

	plane = FieldColumn( Axis, Axis.AXIS_Y )
	majorSegments = FieldColumn( int, 20 )
	minorSegments = FieldColumn( int, 20 )
	majorRadius = FieldColumn( float, 100.0 )
	minorRadius = FieldColumn( float, 40.0 )
	position = FieldColumn( Point3, Point3() )


	inputClass = GSProductModelBlank
	outputClass = GSProductMesh

	def procedureInvoke(self, inputObject, errorHandler):
		prim = TorusPrimitive( self.plane, self.majorSegments, self.minorSegments, self.majorRadius, self.minorRadius )
		mesh = GSProductMesh()
		verts = MeshVertexList()
		prim.createTorus( mesh, verts )
		verts.transform( Matrix4.translate( self.position.toVector3() ), mesh )
		return mesh



class TorusAdjustableInvoker (GSProcedureAdjustableInvoker):
	functionClass = ProcTorusPrimitive

	def initInvoke(self):
		self.verts = MeshVertexList()

	def main(self):
		prim = TorusPrimitive( self._proc.plane, self._proc.majorSegments, self._proc.minorSegments, self._proc.majorRadius, self._proc.minorRadius )
		self.verts.clear()
		self.mesh = GSProductMesh()
		prim.createTorus( self.mesh, self.verts )
		return self.mesh

	def save(self):
		self.verts.savePositions( self.mesh )

	def restore(self):
		self.verts.restorePositions( self.mesh )

	def adjust(self):
		self.verts.transform( Matrix4.translate( self._proc.position.toVector3() ), self.mesh )

	def getResult(self):
		return self.mesh




class ToolTorusPrimitive (GSProcedureTool):
	row = RowRefColumn( ProcTorusPrimitive )

	functionClass = ProcTorusPrimitive
	adjustableInvoker = TorusAdjustableInvoker
	pageTitle = _( 'Torus' )
	titleText = _( 'Create Torus' )

	planeRadio = AxisColumnEditRadioGroup( row.plane )
	majorSegmentsEntry = IntColumnEditEntry( row.majorSegments, 3, 1048576, 1, 10 )
	minorSegmentsEntry = IntColumnEditEntry( row.minorSegments, 3, 1048576, 1, 10 )
	majorRadiusEntry = FloatColumnEditEntry( row.majorRadius, 0.0, 1.0e8, 0.1, 10.0, 4 )
	minorRadiusEntry = FloatColumnEditEntry( row.minorRadius, 0.0, 1.0e8, 0.1, 10.0, 4 )
	positionEntry = Point3ColumnEditEntry( row.position, -1.0e8, 1.0e8, 1.0, 10.0, 4 )

	positionManip = Point3ColumnManipulator( row.position, snapping=ColumnManipulatorSnappingDistance() )
	positionManip.enable()

	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Torus plane:' )
	layout.row()  <<  planeRadio.x  <<  planeRadio.y  <<  planeRadio.z
	layout.row()
	layout.row()  <<  _( 'Major segs.:' )  <<  majorSegmentsEntry.entry
	layout.row()  <<  _( 'Minor segs.:' )  <<  minorSegmentsEntry.entry
	layout.row()
	layout.row()  <<  _( 'Major radius:' )  <<  majorRadiusEntry.entry
	layout.row()  <<  _( 'Minor radius:' )  <<  minorRadiusEntry.entry
	layout.row()
	layout.row()  <<  _( 'Position' )
	layout.row()  <<  'X:'  <<  positionEntry.x
	layout.row()  <<  'Y:'  <<  positionEntry.y
	layout.row()  <<  'Z:'  <<  positionEntry.z
	layout.row()
	layout.finalise()
