##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import copy, deepcopy

from gSculpt.Kernel.DB import FieldColumnInterface, FieldColumn, RowClass, Row

from gSculpt.ProceduralCore import GSFunctionCommandTracker






class GSFunctionClass (RowClass):
	def __init__(cls, clsName, bases, clsDict):
		super( GSFunctionClass, cls ).__init__( clsName, bases, clsDict )

		if not clsDict.has_key( 'description' )  and  not clsDict.has_key( 'getDescription' ):
			raise TypeError, 'function description not supplied'






class GSFunction (Row):
	__metaclass__ = GSFunctionClass
	trackerClass = GSFunctionCommandTracker.GSFunctionCommandTracker
	description = 'Abstract function'


	def __init__(self, src=None):
		super( GSFunction, self ).__init__( src )

		self._adjustableInvokers = []



	@classmethod
	def getDescription(cls):
		return cls.description



	def _p_attachAdjustableInvoker(self, invoker):
		self._adjustableInvokers.append( invoker )


	def _p_detachAdjustableInvoker(self, invoker):
		self._adjustableInvokers.remove( invoker )


	def _f_onFieldModify(self, fieldName):
		super( GSFunction, self )._f_onFieldModify( fieldName )
		for invoker in self._adjustableInvokers:
			invoker.onFunctionArgumentModify( fieldName )





if __name__ == '__main__':
	class SimpleClass (GSFunction):
		a = FieldColumn( int, 1 )

	def onValue(value):
		print '**change** %d' % ( value, )

	x = SimpleClass()
	print x.a
	print '>>x.a=2'
	x.a = 2
	print x.a
	print '>>f = x.field_a'
	f = x.field_a
	print x.a
	print f.value
	print '>>x.a = 3'
	print dir(x)
	x.a = 3
	print x.a
	print f.value
	print '>>del f'
	del f
	gc.collect()
	print x.a

	print '>>f = x.field_a'
	f = x.field_a
	print x.a

	print '>>f.changedSignal.connect( onChange )'
	f.valueSignal.connect( onValue )
	print '>>x.a = 4'
	x.a = 4
	print '>>del f'
	del f
	gc.collect()
	print x.a

	print '>>f = x.field_a'
	f = x.field_a
	print '>>f.changedSignal.disconnect( onChange )'
	f.valueSignal.disconnect( onValue )
	print '>>del f'
	del f
	gc.collect()

	print dir(x)













