##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.CommandHistory import CommandHistory
from gSculpt.CommandHistory import CommandTracker

from GSFunctionCommandTracker import GSFunctionCommandTracker
from GSPlanCommandTracker import GSPlanCommandTracker
import GSFunctionGraph






class GSFunctionGraphAddNodeCommand (CommandHistory.Command):
	def __init__(self, functionGraph, node):
		super( GSFunctionGraphAddNodeCommand, self ).__init__()
		self._functionGraph = functionGraph
		self._node = node


	def execute(self):
		self._functionGraph.nodes.append( self._node )

	def unexecute(self):
		self._functionGraph.nodes.remove( self._node )




class GSFunctionGraphRemoveNodeCommand (CommandHistory.Command):
	def __init__(self, functionGraph, node):
		super( GSFunctionGraphRemoveNodeCommand, self ).__init__()
		self._functionGraph = functionGraph
		self._node = node



	def execute(self):
		self._functionGraph.nodes.remove( self._node )

	def unexecute(self):
		self._functionGraph.nodes.append( self._node )




class GSFunctionGraphNodeCommandTracker (GSFunctionCommandTracker):
	def track(self, node):
		super( GSFunctionGraphNodeCommandTracker, self ).track( node )
		assert isinstance( node, GSFunctionGraph.GSFunctionGraphNode )

		self._commandHistory.track( node.graphNode )


	def stopTracking(self, node):
		assert isinstance( node, GSFunctionGraph.GSFunctionGraphNode )

		self._commandHistory.stopTracking( node.graphNode )

		super( GSFunctionGraphNodeCommandTracker, self ).stopTracking( node )





class GSFunctionGraphCommandTracker (CommandTracker.CommandTracker):
	def track(self, functionGraph):
		super( GSFunctionGraphCommandTracker, self ).track( functionGraph )
		assert isinstance( functionGraph, GSFunctionGraph.GSFunctionGraph )

		self._commandHistory.track( functionGraph.graph )

		for node in functionGraph.nodes:
			self._commandHistory.track( node )


	def stopTracking(self, functionGraph):
		assert isinstance( functionGraph, GSFunctionGraph.GSFunctionGraph )

		self._commandHistory.stopTracking( functionGraph.graph )

		for node in functionGraph.nodes:
			self._commandHistory.stopTracking( node )

		super( GSFunctionGraphCommandTracker, self ).stopTracking( functionGraph )



	def _f_onFunctionGraphNodeAppended(self, functionGraph, node):
		self._commandHistory.track( node )
		self._commandHistory.addCommand( GSFunctionGraphAddNodeCommand( functionGraph, node ) )


	def _f_onFunctionGraphNodeRemoved(self, functionGraph, node):
		self._commandHistory.addCommand( GSFunctionGraphRemoveNodeCommand( functionGraph, node ) )
		self._commandHistory.stopTracking( node )





