##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.CommandHistory import CommandHistory
from gSculpt.CommandHistory.CommandTracker import CommandTracker

from GSProcedureCommandTracker import GSProcedureCommandTracker

import GSPlan




class GSPlanAppendCommand (CommandHistory.Command):
	def __init__(self, plan, proc):
		super( GSPlanAppendCommand, self ).__init__()
		self._plan = plan
		self._proc = proc


	def execute(self):
		self._plan.append( self._proc )

	def unexecute(self):
		self._plan.remove( self._proc )



class GSPlanInsertCommand (CommandHistory.Command):
	def __init__(self, plan, index, proc):
		super( GSPlanInsertCommand, self ).__init__()
		self._plan = plan
		self._index = index
		self._proc = proc


	def execute(self):
		self._plan.insert( self._index, self._proc )

	def unexecute(self):
		self._plan.remove( self._proc )



class GSPlanSetCommand (CommandHistory.Command):
	def __init__(self, plan, index, oldProc, newProc):
		super( GSPlanSetCommand, self ).__init__()
		self._plan = plan
		self._index = index
		self._oldProc = oldProc
		self._newProc = newProc


	def execute(self):
		self._plan[index] = newProc

	def unexecute(self):
		self._plan[index] = oldProc



class GSPlanRemoveCommand (CommandHistory.Command):
	def __init__(self, plan, index, proc):
		super( GSPlanRemoveCommand, self ).__init__()
		self._plan = plan
		self._index = index
		self._proc = proc


	def execute(self):
		del self._plan[self._index]

	def unexecute(self):
		self._plan.insert( self._index, self._proc )



class GSPlanSwapCommand (CommandHistory.Command):
	def __init__(self, plan, index1, index2):
		super( GSPlanSwapCommand, self ).__init__()
		self._plan = plan
		self._index1 = index1
		self._index2 = index2


	def execute(self):
		self._plan.swap( self._index1, self._index2 )

	def unexecute(self):
		self._plan.swap( self._index1, self._index2 )







class GSPlanCommandTracker (CommandTracker):
	def track(self, plan):
		super( GSPlanCommandTracker, self ).track( plan )
		assert isinstance( plan, GSPlan.GSPlan )

		for proc in plan:
			self._commandHistory.track( proc )



	def stopTracking(self, plan):
		assert isinstance( plan, GSPlan.GSPlan )

		for proc in plan:
			self._commandHistory.stopTracking( proc )

		super( GSPlanCommandTracker, self ).stopTracking( plan )




	def _f_onPlanProcAppended(self, plan, index, proc):
		self._commandHistory.track( proc )
		self._commandHistory.addCommand( GSPlanAppendCommand( plan, proc ) )

	def _f_onPlanProcInserted(self, plan, index, proc):
		self._commandHistory.track( proc )
		self._commandHistory.addCommand( GSPlanInsertCommand( plan, index, proc ) )

	def _f_onPlanProcSet(self, plan, index, oldProc, newProc):
		self._commandHistory.stopTracking( oldProc )
		self._commandHistory.track( newProc )
		self._commandHistory.addCommand( GSPlanSetCommand( plan, index, oldProc, newProc ) )

	def _f_onPlanProcRemoved(self, plan, index, proc):
		self._commandHistory.addCommand( GSPlanRemoveCommand( plan, index, proc ) )
		self._commandHistory.stopTracking( proc )

	def _f_onPlanProcsSwapped(self, plan, index1, index2):
		self._commandHistory.addCommand( GSPlanSwapCommand( plan, index1, index2 ) )


