##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************



class GSFunctionAdjustableInvokerClass (type):
	def __init__(cls, clsName, bases, clsDict):
		super( GSFunctionAdjustableInvokerClass, cls ).__init__( clsName, bases, clsDict )

		try:
			functionClass = clsDict['functionClass']
		except KeyError:
			raise TypeError, 'functionClass not specified'








class GSFunctionAdjustableInvoker (object):
	__metaclass__ = GSFunctionAdjustableInvokerClass
	functionClass = None


	STAGE_NONE = 0
	STAGE_INITIALISED = 1
	STAGE_PRODUCTCREATED = 2
	STAGE_ADJUSTED = 3
	STAGE_MAX__ = 4


	def __init__(self, function):
		self._function = function

		self._stage = GSFunctionAdjustableInvoker.STAGE_NONE

		self._initParamNames = set()
		self._mainParamNames = set()
		self._saveParamNames = set()
		self._adjustParamNames = set()




	def attach(self):
 		self._function._p_attachAdjustableInvoker( self )


	def detach(self):
 		self._function._p_detachAdjustableInvoker( self )



	def _o_restart(self):
		self._stage = GSFunctionAdjustableInvoker.STAGE_NONE




	def onFunctionArgumentModify(self, name):
		if name in self._adjustParamNames:
			self._stage = min( self._stage, GSFunctionAdjustableInvoker.STAGE_PRODUCTCREATED )

		if name in self._saveParamNames:
			self._stage = min( self._stage, GSFunctionAdjustableInvoker.STAGE_INITIALISED )

		if name in self._mainParamNames:
			self._stage = min( self._stage, GSFunctionAdjustableInvoker.STAGE_INITIALISED )

		if name in self._initParamNames:
			self._stage = min( self._stage, GSFunctionAdjustableInvoker.STAGE_NONE )



	def requestResult(self):
		if self._stage < GSFunctionAdjustableInvoker.STAGE_INITIALISED:
			initFunction = self._p_getInitFunction()
			self._initParamNames = self._p_callFunction( initFunction )



		if self._stage < GSFunctionAdjustableInvoker.STAGE_PRODUCTCREATED:
			mainFunction = self._p_getMainFunction()
			self._mainParamNames = self._p_callFunction( mainFunction )

			saveFunction = self._p_getSaveFunction()
			self._saveParamNames = self._p_callFunction( saveFunction )


		if self._stage < GSFunctionAdjustableInvoker.STAGE_ADJUSTED:
			restoreFunction = self._p_getRestoreFunction()
			self._p_callFunction( restoreFunction )

			adjustFunction = self._p_getAdjustFunction()
			self._adjustParamNames = self._p_callFunction( adjustFunction )


		self._stage = GSFunctionAdjustableInvoker.STAGE_ADJUSTED

		resultFunction = self._p_getResultFunction()
		return resultFunction( self )



	def _p_callFunction(self, function):
		if function is not None:
			fieldNames = set()

			def onGet(row, fieldName):
				fieldNames.add( fieldName )

			self._function.addGetListener( onGet )
			function( self )
			self._function.removeGetListener( onGet )
			return fieldNames
		else:
			return set()



	@classmethod
	def _p_getInitFunction(cls):
		try:
			return cls.initInvoke
		except:
			return None


	@classmethod
	def _p_getMainFunction(cls):
		try:
			return cls.main
		except:
			return None


	@classmethod
	def _p_getSaveFunction(cls):
		try:
			return cls.save
		except:
			return None


	@classmethod
	def _p_getRestoreFunction(cls):
		try:
			return cls.restore
		except:
			return None


	@classmethod
	def _p_getAdjustFunction(cls):
		try:
			return cls.adjust
		except:
			return None


	@classmethod
	def _p_getResultFunction(cls):
		return cls.getResult

