##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Util.SignalSlot import *

from gSculpt.Kernel.CachedObjectSource import CachedObjectSource
from gSculpt.Kernel.DBEditorTool import RowEditorToolClass, RowEditorTool

from gSculpt.ProceduralCore.GSFunction import GSFunction, GSFunctionClass

from gSculpt.ProceduralTool.GSFunctionAdjustableInvoker import GSFunctionAdjustableInvoker








class FunctionToolProductView (object):
	def installEventHandlers(self, view):
		pass

	def uninstallEventHandlers(self, view):
		pass


	def installPainters(self, view):
		pass

	def uninstallPainters(self, view):
		pass


	def _p_getOutputProductSource(self):
		assert False, 'abstract; overload the "outputProductSource" property'

	def _p_getDisplayedProduct(self):
		assert False, 'abstract'


	outputProductSource = property( _p_getOutputProductSource )
	displayedProduct = property( _p_getDisplayedProduct )





g_functionToolClassTable = {}




class GSFunctionToolClass (RowEditorToolClass):
	def __init__(cls, clsName, bases, clsDict):
		global g_functionToolClassTable
		if not clsDict.has_key( 'functionClass' ):
			raise AttributeError, 'functionClass not specified'

		if not ( isinstance( cls.functionClass, GSFunctionClass )  and  issubclass( cls.functionClass, GSFunction ) ):
			raise TypeError, 'functionClass must be a subclass of GSFunction and an instance of GSFunctionClass'


		clsDict['rowClass'] = cls.functionClass
		cls.rowClass = cls.functionClass


		if clsDict.has_key( 'adjustableInvoker' )  and  cls.adjustableInvoker is not None  and  not issubclass( cls.adjustableInvoker, GSFunctionAdjustableInvoker ):
			raise TypeError, 'adjustableInvoker must be a subclass of GSFunctionAdjustableInvoker'

		if clsDict.has_key( 'adjustableInvoker' )  and  cls.adjustableInvoker is not None  and  cls.adjustableInvoker.functionClass is not cls.functionClass:
			raise TypeError, 'adjustableInvoker functionClass does not match tool functionClass'


		# Class table
		g_functionToolClassTable[cls.functionClass] = cls


		super( GSFunctionToolClass, cls ).__init__( clsName, bases, clsDict )







class GSFunctionTool (RowEditorTool):
	"Function tool base class"
	__metaclass__ = GSFunctionToolClass

	functionClass = GSFunction
	pageTitle = _( 'tool' )
	titleText = 'GSFunctionTool'
	adjustableInvoker = None



	def __init__(self):
		"Constructor"
		super( GSFunctionTool, self ).__init__()
		self._func = None
		self._productView = None
		self._adjustableInvoker = None
		self._painterFactory = None


	def attachFunction(self, func, view, editorSettings, parentWindow, commandHistory, painterFactory):
		"""Attach a GSFunction object to this tool
		@func - the function object to attach
		@view - the modeller view
		@commandHistory - the command history
		@painterFactory - painter factory"""
		super( GSFunctionTool, self ).attachRow( func, view, editorSettings, parentWindow, commandHistory )
		assert painterFactory is not None
		assert self._func is None, 'Function already attached'
		assert isinstance( func, self.functionClass ), '@func is not an instance of ' + self.functionClass.__name__
		self._func = func
		self._painterFactory = painterFactory

	def detachFunction(self):
		"""Detach the currently attach GSFunction object"""
		super( GSFunctionTool, self ).detachRow()
		self._func = None
		self._painterFactory = None




	def initialiseView(self):
		super( GSFunctionTool, self ).initialiseView()

		self._productView = self._p_createProductView()
		self._productView.installPainters( self._view )


	def shutdownView(self):
		super( GSFunctionTool, self ).shutdownView()

		self._productView.uninstallPainters( self._view )
		self._p_shutdownProductView( self._productView )
		self._productView = None



	def onPageSwitchIn(self):
		self._productView.installEventHandlers( self._view )
		super( GSFunctionTool, self ).onPageSwitchIn()


	def onPageSwitchOut(self):
		super( GSFunctionTool, self ).onPageSwitchOut()
		self._productView.uninstallEventHandlers( self._view )



	def _p_createProductView(self):
		assert False, 'abstract'

	def _p_shutdownProductView(self, productView):
		assert False, 'abstract'



	def _f_setAdjustableInvoker(self, invoker):
		self._adjustableInvoker = invoker

	def _f_getAdjustableInvokerClass(self):
		return type( self ).adjustableInvoker



	def _p_getOutputProductSource(self):
		return self._productView.outputProductSource

	def _p_getDisplayedProduct(self):
		return self._productView.displayedProduct

	def _p_getPainterFactory(self):
		return self._painterFactory

	outputProductSource = property( _p_getOutputProductSource )
	displayedProduct = property( _p_getDisplayedProduct )
	painterFactory = property( _p_getPainterFactory )





def createFunctionTool(function):
	global g_functionToolClassTable
	functionClass = function.__class__
	toolClass = g_functionToolClassTable[functionClass]
	return toolClass()





if __name__ == '__main__':
	from gSculpt.Kernel.DB import FieldColumn
	from gSculpt.Kernel.ColumnEditors import IntColumnEditEntry
	from gSculpt.UI.FormLayout import FormLayoutFactory

	class ExampleFunction (GSFunction):
		description = 'example'
		a = FieldColumn( int, 1 )

	class ExampleTool (GSFunctionTool):
		functionClass = ExampleFunction
		pageTitle = 'Ex'
		titleText = 'Example'
		aEdit = IntColumnEditEntry( ExampleFunction.a, 0, 1048576, 1, 10 )

		layout = FormLayoutFactory()
		layout.row()  <<  'A:'  <<  aEdit
		layout.row()
		layout.finalise()


