##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from gSculpt.Kernel.Variable import VarInterface, Variable
from gSculpt.Kernel.DB import Field, registerFieldType

from TextureRef import TextureRef
from TextureChooser import TextureChooser

from gSculpt.VarEdit.VarEdit import WidgetSingleVarEdit




class TextureRefVarInterface (VarInterface):
	valueClass = TextureRef



class TextureRefVar (Variable):
	valueClass = TextureRef



class TextureRefField (Field):
	valueClass = TextureRef
	defaultValue = TextureRef()


registerFieldType( TextureRef, TextureRefField )



class TextureRefVarEditChooser (WidgetSingleVarEdit):
	def __init__(self, parentWindow, dialogTitle):
		super( TextureRefVarEditChooser, self ).__init__()
		self.chooser = TextureChooser( None, parentWindow, dialogTitle )
		self.chooser.textureSignal.connect( self._p_onChooser )

		self._o_setUISensitivity( False )


	def attachVariable(self, var):
		super( TextureRefVarEditChooser, self ).attachVariable( var )
		if self._var.isValid():
			self._p_onVarValue( self._var.value )
		self._var.valueSignal.connect( self._p_onVarValue )

	def detachVariable(self):
		assert self._var is not None, 'no variable attached'
		self._var.valueSignal.disconnect( self._p_onVarValue )
		self.chooser.texture = None
		super( TextureRefVarEditChooser, self ).detachVariable()


	def show(self):
		self.chooser.getWidget().show()

	def hide(self):
		self.chooser.getWidget().hide()

	def _o_setUISensitivity(self, bSensitive):
		self.chooser.getWidget().set_sensitive( bSensitive )


	def _p_onChooser(self, event, texture):
		if event.bUserEvent:
			self._var.value = TextureRef( texture )

	def _p_onVarValue(self, value):
		if value is not None:
			self.chooser.texture = value.texture
		else:
			self.chooser.texture = None


