##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

import math

from gSculpt.Math.Math import Vector3, Matrix4

from gSculpt.UI.FormLayout import FormLayoutFactory
from gSculpt.UI.UIAction import UIAction

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Manipulator.ColumnManipulators import *
from gSculpt.Manipulator.RealManipulator import *

from gSculpt.TransformationTools.ToolTransformation import *
from gSculpt.TransformationTools.ToolTransformationBase import *
from gSculpt.TransformationTools.ToolRotatedTransformation import *
from gSculpt.TransformationTools import TransformationToolActions
from gSculpt.Transformation.Transformation import DentTransformation, MatrixTransformation, Radius2Transformation




def makeTransformation(model, target, xformRotation, dent):
	centre = model.getBBox( target == TransformationTarget.POINTS ).getCentre()

	rotationMatrix = Matrix4.rotate( xformRotation )
	modelToXformFrameMatrix = Matrix4.translate( -centre.toVector3() )  *  rotationMatrix.rigidBodyInverse()
	xformToModelFrameMatrix = modelToXformFrameMatrix.rigidBodyInverse()

	getRadiusX0 = MatrixTransformation( modelToXformFrameMatrix )
	getRadiusX1 = Radius2Transformation()
	getRadiusX0.chainTo( getRadiusX1 )

	box = model.getBBox( getRadiusX0, target == TransformationTarget.POINTS )
	radius = box.getUpper().z

	x0 = MatrixTransformation( modelToXformFrameMatrix )
	x1 = DentTransformation( radius, dent )
	x2 = MatrixTransformation( xformToModelFrameMatrix )
	x1.chainTo( x2 )
	x0.chainTo( x1 )

	return x0




class ProcTransformationDent (ProcRotatedTransformation):
	description = _( 'Dent' )

	dent = FieldColumn( float, 1.0 )


	def procedureInvoke(self, inputObject, errorHandler):
		x0 = makeTransformation( inputObject, self.target, self.xformRotation, self.dent )
		ProcTransformationDent.transform( inputObject, x0, self.target )
		return inputObject




class TransformationDentAdjustableInvoker (RotatedTransformationAdjustableInvoker):
	functionClass = ProcTransformationDent

	def adjust(self):
		x0 = makeTransformation( self.model, self._proc.target, self._proc.xformRotation, self._proc.dent )
		self.transform( x0, self._proc.target )




class ToolTransformationDent (ToolRotatedTransformation):
	row = RowRefColumn( ProcTransformationDent )

	functionClass = ProcTransformationDent
	pageTitle = _( 'Dent' )
	titleText = _( 'Dent' )
	adjustableInvoker = TransformationDentAdjustableInvoker


	dentManip = Real1ColumnManipulator( row.dent, -1.0e8, 1.0e8, RealManipulator.FixedScale( 0.005 ), snapping=ColumnManipulatorSnappingFraction() )

	rotationManip = RotationColumnManipulator( row.xformRotation, ToolRotatedTransformation.focalPoint, snapping=ColumnManipulatorSnappingAngle() )

	manipulatorSwitch = ColumnManipulatorSwitch( [ dentManip, rotationManip ] )
	manipulatorSwitch.enable()

	manipulatorSelector = ColumnManipulatorSwitchRadioGroup( manipulatorSwitch )
	dentButton = manipulatorSelector.addChoice( dentManip, TransformationToolActions.deformAction )
	rotationButton = manipulatorSelector.addChoice( rotationManip, TransformationToolActions.rotationAction )


	rotationEntry = QuaternionColumnEditEntry( row.xformRotation )
	rotationPopup = QuaternionColumnEditDirectionPopup( row.xformRotation, _( 'Set rotation' ) )
	dentEntry = FloatColumnEditEntry( row.dent, -1.0e8, 1.0e8, 0.01, 0.1, 4 )


	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Mouse function:' )
	layout.row()  <<  dentButton
	layout.row()  <<  rotationButton
	layout.row()
	layout.row()  <<  _( 'Rotation:' )
	layout.row()  <<  rotationPopup.popup
	layout.row()  <<  'X:'  <<  rotationEntry.x
	layout.row()  <<  'Y:'  <<  rotationEntry.y
	layout.row()  <<  'Z:'  <<  rotationEntry.z
	layout.row()
	layout.row()  <<  _( 'Dent:' )  <<  dentEntry.entry
	layout.row()
	layout.finalise()

