##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
from copy import deepcopy

import math

from gSculpt.Math.Math import Vector3, Matrix4, Vector2, Point3

from gSculpt.UI.FormLayout import FormLayoutFactory
from gSculpt.UI.UIAction import UIAction

from gSculpt.Kernel.DB import *
from gSculpt.ProceduralCore.GSProcedure import *

from gSculpt.ProceduralTool.GSProcedureTool import GSProcedureTool
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.Manipulator.ColumnManipulators import *
from gSculpt.Manipulator.RealManipulator import *

from gSculpt.TransformationTools.ToolTransformation import *
from gSculpt.TransformationTools.ToolTransformationBase import *
from gSculpt.TransformationTools.ToolRotatedTransformation import *
from gSculpt.TransformationTools import TransformationToolActions
from gSculpt.Transformation.Transformation import MatrixTransformation




def makeTransformation(model, target, xformRotation, shear):
	rotationMatrix = Matrix4.rotate( xformRotation )
	modelToXformFrameMatrix = rotationMatrix.rigidBodyInverse()
	xformToModelFrameMatrix = rotationMatrix

	box = model.getBBox( modelToXformFrameMatrix, target == TransformationTarget.POINTS )
	shear = shear  *  ( 1.0 / box.getDepth() )
	transformationMatrix = modelToXformFrameMatrix * Matrix4.translate( -box.getLower().toVector3() ) * Matrix4.shearZ( shear.x, shear.y ) *			\
					Matrix4.translate( box.getLower().toVector3() ) * xformToModelFrameMatrix

	return transformationMatrix




class ProcTransformationShear (ProcRotatedTransformation):
	description = _( 'Shear' )

	shear = FieldColumn( Vector2, Vector2( 0.0, 0.0 ) )


	def procedureInvoke(self, inputObject, errorHandler):
		x0 = makeTransformation( inputObject, self.target, self.xformRotation, self.shear )
		ProcTransformationShear.transform( inputObject, x0, self.target )
		return inputObject




class TransformationShearAdjustableInvoker (RotatedTransformationAdjustableInvoker):
	functionClass = ProcTransformationShear

	def adjust(self):
		x0 = makeTransformation( self.model, self._proc.target, self._proc.xformRotation, self._proc.shear )
		self.transform( x0, self._proc.target )




class ToolTransformationShear (ToolRotatedTransformation):
	row = RowRefColumn( ProcTransformationShear )

	functionClass = ProcTransformationShear
	pageTitle = _( 'Shear' )
	titleText = _( 'Shear' )
	adjustableInvoker = TransformationShearAdjustableInvoker


	row_xformRotation = ObjectSourceProxyColumn( row.xformRotation )

	def _p_shearFocalPoint(self):
		centre = self.inputProduct.getBBox( self.row_target == TransformationTarget.POINTS ).getCentre()

		rotationMatrix = Matrix4.rotate( self.row_xformRotation )
		modelToXformFrameMatrix = Matrix4.translate( -centre.toVector3() )  *  rotationMatrix.rigidBodyInverse()
		xformToModelFrameMatrix = modelToXformFrameMatrix.rigidBodyInverse()

		box = self.inputProduct.getBBox( modelToXformFrameMatrix, self.row_target == TransformationTarget.POINTS )
		return Point3( 0.0, 0.0, box.getUpper().z )  *  xformToModelFrameMatrix

	shearFocalPoint = FunctionColumn( _p_shearFocalPoint )



	shearManip = Translation2ColumnManipulator3d( row.shear, shearFocalPoint, row.xformRotation,
										snapping=ColumnManipulatorSnappingDistance() )

	rotationManip = RotationColumnManipulator( row.xformRotation, ToolRotatedTransformation.focalPoint, snapping=ColumnManipulatorSnappingAngle() )

	manipulatorSwitch = ColumnManipulatorSwitch( [ shearManip, rotationManip ] )
	manipulatorSwitch.enable()

	manipulatorSelector = ColumnManipulatorSwitchRadioGroup( manipulatorSwitch )
	shearButton = manipulatorSelector.addChoice( shearManip, TransformationToolActions.deformAction )
	rotationButton = manipulatorSelector.addChoice( rotationManip, TransformationToolActions.rotationAction )


	rotationEntry = QuaternionColumnEditEntry( row.xformRotation )
	rotationPopup = QuaternionColumnEditDirectionPopup( row.xformRotation, _( 'Set rotation' ) )
	shearEntry = Vector2ColumnEditEntry( row.shear, -1.0e18, 1.0e18, 0.1, 10.0, 4 )




	layout = FormLayoutFactory()
	layout.row()  <<  _( 'Mouse function:' )
	layout.row()  <<  shearButton
	layout.row()  <<  rotationButton
	layout.row()
	layout.row()  <<  _( 'Rotation:' )
	layout.row()  <<  rotationPopup.popup
	layout.row()  <<  'X:'  <<  rotationEntry.x
	layout.row()  <<  'Y:'  <<  rotationEntry.y
	layout.row()  <<  'Z:'  <<  rotationEntry.z
	layout.row()
	layout.row()  <<  _( 'Shear:' )
	layout.row()  <<  'U:'  <<  shearEntry.x
	layout.row()  <<  'V:'  <<  shearEntry.y
	layout.row()
	layout.finalise()

