##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
import os

from gSculpt.Math.Math import Colour3f

from gSculpt.Math import MathVariable, MathField

from gSculpt.Kernel.DB import Row, FieldColumn

from gSculpt.FileIO.IOXml import *

from gSculpt.Config import GSConfig

from gSculpt.Model.Model import ModelDrawColourPalette, setColourPalette

from gSculpt.Model import ModelDraw

from gSculpt.WorkArea import Viewport



class ViewDrawingSettings (Row):
	backgroundColour = FieldColumn( Colour3f, Colour3f( 0.0, 0.0, 0.0 ), bWriteIfDefault=False )


	gridLevel1Colour = FieldColumn( Colour3f, Colour3f( 0.0, 0.8, 0.0 ), bWriteIfDefault=False )
	gridLevel2Colour = FieldColumn( Colour3f, Colour3f( 0.0, 0.4, 0.0 ), bWriteIfDefault=False )
	gridLevel3Colour = FieldColumn( Colour3f, Colour3f( 0.0, 0.2, 0.0 ), bWriteIfDefault=False )


	unmarkedVertexColour = FieldColumn( Colour3f, Colour3f( 0.5, 0.3, 1.0 ), bWriteIfDefault=False )
	unmarkedVertexPointSize = FieldColumn( float, 5.0, bWriteIfDefault=False )
	markedVertexColour = FieldColumn( Colour3f, Colour3f( 1.0, 0.95, 0.5 ), bWriteIfDefault=False )
	markedVertexPointSize = FieldColumn( float, 5.0, bWriteIfDefault=False )
	reflectedVertexColour = FieldColumn( Colour3f, Colour3f( 0.6, 0.6, 0.6 ), bWriteIfDefault=False )
	reflectedVertexPointSize = FieldColumn( float, 5.0, bWriteIfDefault=False )


	polylineVertexSpecialHighlightPointSize = FieldColumn( float, 11.0, bWriteIfDefault=False )
	polylineNormalSharpHighlightColour = FieldColumn( Colour3f, Colour3f( 0.45, 0.0, 0.0 ), bWriteIfDefault=False )
	polylineTextureBreakHighlightColour = FieldColumn( Colour3f, Colour3f( 0.0, 0.0, 0.45 ), bWriteIfDefault=False )


	unmarkedEdgeColour = FieldColumn( Colour3f, Colour3f( 1.0, 0.15, 0.0 ), bWriteIfDefault=False )
	unmarkedEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	markedEdgeColour = FieldColumn( Colour3f, Colour3f( 0.0, 0.7, 0.4 ), bWriteIfDefault=False )
	markedEdgeLineWidth = FieldColumn( float, 3.0, bWriteIfDefault=False )
	backgroundEdgeColour = FieldColumn( Colour3f, Colour3f( 0.525, 0.525, 0.525 ), bWriteIfDefault=False )
	backgroundEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	transparentBackgroundEdgeColour = FieldColumn( Colour3f, Colour3f( 0.525 * 0.25, 0.525 * 0.25, 0.525 * 0.25 ), bWriteIfDefault=False )
	transparentBackgroundEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	reflectedEdgeColour = FieldColumn( Colour3f, Colour3f( 0.4, 0.8, 1.0 ), bWriteIfDefault=False )
	reflectedEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	liveSubdEdgeColour = FieldColumn( Colour3f, Colour3f( 0.43, 0.52, 0.47 ), bWriteIfDefault=False )
	liveSubdEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	uvEdgeColour = FieldColumn( Colour3f, Colour3f( 1.0, 1.0, 1.0 ), bWriteIfDefault=False )
	uvEdgeAlpha = FieldColumn( float, 1.0, bWriteIfDefault=False )


	splineEdgeColour = FieldColumn( Colour3f, Colour3f( 0.698, 0.486, 0.8 ), bWriteIfDefault=False )
	splineEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	backgroundSplineEdgeColour = FieldColumn( Colour3f, Colour3f( 0.612, 0.502, 0.667 ), bWriteIfDefault=False )
	backgroundSplineEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )
	reflectedSplineEdgeColour = FieldColumn( Colour3f, Colour3f( 0.667, 0.557, 1.0 ), bWriteIfDefault=False )
	reflectedSplineEdgeLineWidth = FieldColumn( float, 1.0, bWriteIfDefault=False )


	surfaceDiffuseColour = FieldColumn( Colour3f, Colour3f( 1.0, 1.0, 1.0 ), bWriteIfDefault=False )
	surfaceSpecularColour = FieldColumn( Colour3f, Colour3f( 0.0, 0.0, 0.0 ), bWriteIfDefault=False )
	transparentBackgroundSurfaceDiffuseColour = FieldColumn( Colour3f, Colour3f( 0.835 * 0.3, 0.114 * 0.3, 0.29 * 0.3 ), bWriteIfDefault=False )
	transparentBackgroundSurfaceSpecularColour = FieldColumn( Colour3f, Colour3f( 0.5 * 0.3, 0.5 * 0.3, 0.5 * 0.3 ), bWriteIfDefault=False )
	transparentBackgroundSurfaceAlpha = FieldColumn( float, 0.6, bWriteIfDefault=False )
	reflectedSurfaceFilterColour = FieldColumn( Colour3f, Colour3f( 0.82, 0.94, 1.0 ), bWriteIfDefault=False )
	liveSubdSurfaceDiffuseColour = FieldColumn( Colour3f, Colour3f( 0.7, 0.7, 0.7 ), bWriteIfDefault=False )
	liveSubdSurfaceSpecularColour = FieldColumn( Colour3f, Colour3f( 0.5, 0.5, 0.5 ), bWriteIfDefault=False )


	markedFaceColour = FieldColumn( Colour3f, Colour3f( 0.5, 0.3, 0.075 ), bWriteIfDefault=False )
	markedFaceAlpha = FieldColumn( float, 0.5, bWriteIfDefault=False )

	overlayFactor = FieldColumn( float, 0.35, bWriteIfDefault=False )

	mirrorPlaneColour = FieldColumn( Colour3f, Colour3f( 0.02, 0.067, 0.10 ), bWriteIfDefault=False )
	mirrorPlaneAlpha = FieldColumn( float, 0.7, bWriteIfDefault=False )

	drawingPlaneColour = FieldColumn( Colour3f, Colour3f( 0.15, 0.0, 0.15 ), bWriteIfDefault=False )
	drawingPlaneAlpha = FieldColumn( float, 0.7, bWriteIfDefault=False )




	vertexPickHighlightPointSize = FieldColumn( float, 8.0, bWriteIfDefault=False )
	edgePickHighlightLineWidth = FieldColumn( float, 8.0, bWriteIfDefault=False )
	targetHighlightColour = FieldColumn( Colour3f, Colour3f( 0.0, 0.7, 0.0 ), bWriteIfDefault=False )
	markedTargetHighlightColour = FieldColumn( Colour3f, Colour3f( 0.7, 0.7, 0.0 ), bWriteIfDefault=False )
	invalidTargetHighlightColour = FieldColumn( Colour3f, Colour3f( 0.65, 0.0, 0.0 ), bWriteIfDefault=False )
	warningTargetHighlightColour = FieldColumn( Colour3f, Colour3f( 0.7, 0.35, 0.0 ), bWriteIfDefault=False )
	pathHighlightColour = FieldColumn( Colour3f, Colour3f( 0.25, 0.1, 1.0 ), bWriteIfDefault=False )

	vertexReticleColour = FieldColumn( Colour3f, Colour3f( 1.0, 0.0, 1.0 ), bWriteIfDefault=False )
	modification1HighlightColour = FieldColumn( Colour3f, Colour3f( 0.5, 0.0, 1.0 ), bWriteIfDefault=False )
	modification2HighlightColour = FieldColumn( Colour3f, Colour3f( 0.0, 0.4, 0.8 ), bWriteIfDefault=False )
	modificationHighlightAlpha = FieldColumn( float, 0.55, bWriteIfDefault=False )

	tweakConstraintWidgetAlpha = FieldColumn( float, 0.6, bWriteIfDefault=False )
	tweakConstraintWidgetSize = FieldColumn( float, 0.0375, bWriteIfDefault=False )
	tweakConstraintWidgetBrightness = FieldColumn( float, 0.5, bWriteIfDefault=False )






	def applySettings(self, view):
		palette = ModelDrawColourPalette()

		palette.unmarkedVertexColour = self.unmarkedVertexColour
		palette.unmarkedVertexPointSize = self.unmarkedVertexPointSize
		palette.markedVertexColour = self.markedVertexColour
		palette.markedVertexPointSize = self.markedVertexPointSize
		palette.reflectedVertexColour = self.reflectedVertexColour
		palette.reflectedVertexPointSize = self.reflectedVertexPointSize


		palette.polylineVertexSpecialHighlightPointSize = self.polylineVertexSpecialHighlightPointSize
		palette.polylineNormalSharpHighlightColour = self.polylineNormalSharpHighlightColour
		palette.polylineTextureBreakHighlightColour = self.polylineTextureBreakHighlightColour


		palette.unmarkedEdgeColour = self.unmarkedEdgeColour
		palette.unmarkedEdgeLineWidth = self.unmarkedEdgeLineWidth
		palette.markedEdgeColour = self.markedEdgeColour
		palette.markedEdgeLineWidth = self.markedEdgeLineWidth
		palette.backgroundEdgeColour = self.backgroundEdgeColour
		palette.backgroundEdgeLineWidth = self.backgroundEdgeLineWidth
		palette.transparentBackgroundEdgeColour = self.transparentBackgroundEdgeColour
		palette.transparentBackgroundEdgeLineWidth = self.transparentBackgroundEdgeLineWidth
		palette.reflectedEdgeColour = self.reflectedEdgeColour
		palette.reflectedEdgeLineWidth = self.reflectedEdgeLineWidth
		palette.liveSubdEdgeColour = self.liveSubdEdgeColour
		palette.liveSubdEdgeLineWidth = self.liveSubdEdgeLineWidth
		palette.uvEdgeColour = self.uvEdgeColour
		palette.uvEdgeAlpha = self.uvEdgeAlpha


		palette.splineEdgeColour = self.splineEdgeColour
		palette.splineEdgeLineWidth = self.splineEdgeLineWidth
		palette.backgroundSplineEdgeColour = self.backgroundSplineEdgeColour
		palette.backgroundSplineEdgeLineWidth = self.backgroundSplineEdgeLineWidth
		palette.reflectedSplineEdgeColour = self.reflectedSplineEdgeColour
		palette.reflectedSplineEdgeLineWidth = self.reflectedSplineEdgeLineWidth


		palette.surfaceDiffuseColour = self.surfaceDiffuseColour
		palette.surfaceSpecularColour = self.surfaceSpecularColour
		palette.transparentBackgroundSurfaceDiffuseColour = self.transparentBackgroundSurfaceDiffuseColour
		palette.transparentBackgroundSurfaceSpecularColour = self.transparentBackgroundSurfaceSpecularColour
		palette.transparentBackgroundSurfaceAlpha = self.transparentBackgroundSurfaceAlpha
		palette.reflectedSurfaceFilterColour = self.reflectedSurfaceFilterColour
		palette.liveSubdSurfaceDiffuseColour = self.liveSubdSurfaceDiffuseColour
		palette.liveSubdSurfaceSpecularColour = self.liveSubdSurfaceSpecularColour


		palette.markedFaceColour = self.markedFaceColour
		palette.markedFaceAlpha = self.markedFaceAlpha

		palette.overlayFactor = self.overlayFactor

		palette.mirrorPlaneColour = self.mirrorPlaneColour
		palette.mirrorPlaneAlpha = self.mirrorPlaneAlpha

		setColourPalette( palette )



		ModelDraw.VERTEX_PICKHIGHLIGHT_POINT_SIZE = self.vertexPickHighlightPointSize
		ModelDraw.EDGE_PICKHIGHLIGHT_LINE_WIDTH = self.edgePickHighlightLineWidth

		ModelDraw.OVERLAY_FACTOR = self.overlayFactor

		ModelDraw.VERTEX_RETICLE_COLOUR = self.vertexReticleColour

		ModelDraw.TARGET_HIGHLIGHT_COLOUR = self.targetHighlightColour
		ModelDraw.MARKED_TARGET_HIGHLIGHT_COLOUR = self.markedTargetHighlightColour
		ModelDraw.INVALID_TARGET_HIGHLIGHT_COLOUR = self.invalidTargetHighlightColour
		ModelDraw.WARNING_TARGET_HIGHLIGHT_COLOUR = self.warningTargetHighlightColour
		ModelDraw.PATH_HIGHLIGHT_COLOUR = self.pathHighlightColour

		ModelDraw.TWEAK_CONSTRAINT_WIDGET_ALPHA = self.tweakConstraintWidgetAlpha
		ModelDraw.TWEAK_CONSTRAINT_WIDGET_SIZE = self.tweakConstraintWidgetSize
		ModelDraw.TWEAK_CONSTRAINT_WIDGET_BRIGHTNESS = self.tweakConstraintWidgetBrightness

		ModelDraw.MODIFICATION1_HIGHLIGHT_COLOUR = self.modification1HighlightColour
		ModelDraw.MODIFICATION2_HIGHLIGHT_COLOUR = self.modification2HighlightColour
		ModelDraw.MODIFICATION_HIGHLIGHT_ALPHA = self.modificationHighlightAlpha

		ModelDraw.DRAWING_PLANE_COLOUR = self.drawingPlaneColour
		ModelDraw.DRAWING_PLANE_ALPHA = self.drawingPlaneAlpha


		Viewport.CLEAR_COLOUR = self.backgroundColour


		view.gridColours = [ self.gridLevel3Colour, self.gridLevel2Colour, self.gridLevel1Colour ]




def readViewDrawingSettings(viewDrawingSettings, filename):
	if os.path.exists( filename ):
		f = open( filename, 'r' )
		if f is not None:
			doc = InputXmlDocument()
			doc.parseFile( f )
			contentNode = doc.getContentNode()
			if contentNode.isValid():
				contentNode  >>  viewDrawingSettings
	else:
		raise IOError, 'View drawing settings file \'%s\' does not exist'  %  ( filename, )



def writeViewDrawingSettings(viewDrawingSettings, filename):
	doc = OutputXmlDocument()
	doc.getContentNode()  <<  viewDrawingSettings
	f = open( filename, 'w' )
	if f is not None:
		doc.writeFile( f )
		f.close()




def readSystemViewDrawingSettings(viewDrawingSettings):
	try:
		readViewDrawingSettings( viewDrawingSettings, GSConfig.viewDrawingSettingsConfigPath )
	except IOError:
		print 'Could not read system view drawing settings file'



def writeSystemViewDrawingSettings(viewDrawingSettings):
	try:
		writeViewDrawingSettings( viewDrawingSettings, GSConfig.viewDrawingSettingsConfigPath )
	except IOError:
		print 'Could not write system view drawing settings file'

