##-*************************
##-* This program is free software; you can use it, redistribute it and/or modify it
##-* under the terms of the GNU General Public License version 2 as published by the
##-* Free Software Foundation. The full text of the GNU General Public License
##-* version 2 can be found in the file named 'COPYING' that accompanies this
##-* program. This source code is (C)copyright Geoffrey French 1999-2007.
##-*************************
import os

import pygtk
pygtk.require( '2.0' )
import gtk

from copy import *

from gSculpt.UI.FormLayout import FormLayoutFactory
from gSculpt.UI.MessageDialog import messageDialog
from gSculpt.UI.ConfirmOverwriteFileDialog import confirmOverwriteFileDialog

from gSculpt.Util.SignalSlot import ClassSignal

from gSculpt.Kernel.DB import RowRefColumn
from gSculpt.Kernel.DBEdit import RowEditor
from gSculpt.Kernel.ColumnEditors import *

from gSculpt.View.ViewDrawingSettings import ViewDrawingSettings, readViewDrawingSettings, writeViewDrawingSettings



class ViewDrawingSettingsEditor (RowEditor):
	row = RowRefColumn( ViewDrawingSettings )

	rowClass = ViewDrawingSettings



	backgroundColourButton = Colour3fColumnEditColourButton( row.backgroundColour )


	gridLevel1ColourButton = Colour3fColumnEditColourButton( row.gridLevel1Colour )
	gridLevel2ColourButton = Colour3fColumnEditColourButton( row.gridLevel2Colour )
	gridLevel3ColourButton = Colour3fColumnEditColourButton( row.gridLevel3Colour )


	unmarkedVertexColourButton = Colour3fColumnEditColourButton( row.unmarkedVertexColour )
	unmarkedVertexPointSizeEntry = FloatColumnEditEntry( row.unmarkedVertexPointSize, 0.0, 1024.0, 1.0, 10.0, 2 )
	markedVertexColourButton = Colour3fColumnEditColourButton( row.markedVertexColour )
	markedVertexPointSizeEntry = FloatColumnEditEntry( row.markedVertexPointSize, 0.0, 1024.0, 1.0, 10.0, 2 )
	reflectedVertexColourButton = Colour3fColumnEditColourButton( row.reflectedVertexColour )
	reflectedVertexPointSizeEntry = FloatColumnEditEntry( row.reflectedVertexPointSize, 0.0, 1024.0, 1.0, 10.0, 2 )


	unmarkedEdgeColourButton = Colour3fColumnEditColourButton( row.unmarkedEdgeColour )
	unmarkedEdgeLineWidthEntry = FloatColumnEditEntry( row.unmarkedEdgeLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )
	markedEdgeColourButton = Colour3fColumnEditColourButton( row.markedEdgeColour )
	markedEdgeLineWidthEntry = FloatColumnEditEntry( row.markedEdgeLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )
	backgroundEdgeColourButton = Colour3fColumnEditColourButton( row.backgroundEdgeColour )
	backgroundEdgeLineWidthEntry = FloatColumnEditEntry( row.backgroundEdgeLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )
	transparentBackgroundEdgeColourButton = Colour3fColumnEditColourButton( row.transparentBackgroundEdgeColour )
	transparentBackgroundEdgeLineWidthEntry = FloatColumnEditEntry( row.transparentBackgroundEdgeLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )
	reflectedEdgeColourButton = Colour3fColumnEditColourButton( row.reflectedEdgeColour )
	reflectedEdgeLineWidthEntry = FloatColumnEditEntry( row.reflectedEdgeLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )
	liveSubdEdgeColourButton = Colour3fColumnEditColourButton( row.liveSubdEdgeColour )
	liveSubdEdgeLineWidthEntry = FloatColumnEditEntry( row.liveSubdEdgeLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )


	surfaceDiffuseColourButton = Colour3fColumnEditColourButton( row.surfaceDiffuseColour )
	surfaceSpecularColourButton = Colour3fColumnEditColourButton( row.surfaceSpecularColour )
	transparentBackgroundSurfaceDiffuseColourButton = Colour3fColumnEditColourButton( row.transparentBackgroundSurfaceDiffuseColour )
	transparentBackgroundSurfaceSpecularColourButton = Colour3fColumnEditColourButton( row.transparentBackgroundSurfaceSpecularColour )
	transparentBackgroundSurfaceAlphaEntry = FloatColumnEditEntry( row.transparentBackgroundSurfaceAlpha, 0.0, 1.0, 0.01, 0.1, 2 )
	transparentBackgroundSurfaceAlphaSlider = FloatColumnEditSlider( row.transparentBackgroundSurfaceAlpha, 0.0, 1.0, 0.01, 0.1 )
	reflectedSurfaceFilterColourButton = Colour3fColumnEditColourButton( row.reflectedSurfaceFilterColour )
	liveSubdSurfaceDiffuseColourButton = Colour3fColumnEditColourButton( row.liveSubdSurfaceDiffuseColour )
	liveSubdSurfaceSpecularColourButton = Colour3fColumnEditColourButton( row.liveSubdSurfaceSpecularColour )


	markedFaceColourButton = Colour3fColumnEditColourButton( row.markedFaceColour )
	markedFaceAlphaEntry = FloatColumnEditEntry( row.markedFaceAlpha, 0.0, 1.0, 0.01, 0.1, 2 )
	markedFaceAlphaSlider = FloatColumnEditSlider( row.markedFaceAlpha, 0.0, 1.0, 0.01, 0.1 )


	overlayFactorEntry = FloatColumnEditEntry( row.overlayFactor, 0.0, 1.0, 0.01, 0.1, 2 )
	overlayFactorSlider = FloatColumnEditSlider( row.overlayFactor, 0.0, 1.0, 0.01, 0.1 )


	mirrorPlaneColourButton = Colour3fColumnEditColourButton( row.mirrorPlaneColour )
	mirrorPlaneAlphaEntry = FloatColumnEditEntry( row.mirrorPlaneAlpha, 0.0, 1.0, 0.01, 0.1, 2 )
	mirrorPlaneAlphaSlider = FloatColumnEditSlider( row.mirrorPlaneAlpha, 0.0, 1.0, 0.01, 0.1 )


	drawingPlaneColourButton = Colour3fColumnEditColourButton( row.drawingPlaneColour )
	drawingPlaneAlphaEntry = FloatColumnEditEntry( row.drawingPlaneAlpha, 0.0, 1.0, 0.01, 0.1, 2 )
	drawingPlaneAlphaSlider = FloatColumnEditSlider( row.drawingPlaneAlpha, 0.0, 1.0, 0.01, 0.1 )



	vertexPickHighlightPointSizeEntry = FloatColumnEditEntry( row.vertexPickHighlightPointSize, 0.0, 1024.0, 1.0, 10.0, 2 )
	edgePickHighlightLineWidthEntry = FloatColumnEditEntry( row.edgePickHighlightLineWidth, 0.0, 1024.0, 1.0, 10.0, 2 )
	targetHighlightColourButton = Colour3fColumnEditColourButton( row.targetHighlightColour )
	markedTargetHighlightColourButton = Colour3fColumnEditColourButton( row.markedTargetHighlightColour )
	invalidTargetHighlightColourButton = Colour3fColumnEditColourButton( row.invalidTargetHighlightColour )
	warningTargetHighlightColourButton = Colour3fColumnEditColourButton( row.warningTargetHighlightColour )
	pathHighlightColourButton = Colour3fColumnEditColourButton( row.pathHighlightColour )

	vertexReticleColourButton = Colour3fColumnEditColourButton( row.vertexReticleColour )
	modification1HighlightColourButton = Colour3fColumnEditColourButton( row.modification1HighlightColour )
	modification2HighlightColourButton = Colour3fColumnEditColourButton( row.modification2HighlightColour )
	modificationHighlightAlphaEntry = FloatColumnEditEntry( row.modificationHighlightAlpha, 0.0, 1.0, 0.01, 0.1, 2 )
	modificationHighlightAlphaSlider = FloatColumnEditSlider( row.modificationHighlightAlpha, 0.0, 1.0, 0.01, 0.1 )




	layout = FormLayoutFactory( bHomogeneous=False, xAttachOptions=gtk.FILL, yAttachOptions = gtk.SHRINK | gtk.FILL, xPadding=15 )
	layout.row()  <<  _( 'Background' )
	layout.row()  <<  _( 'Background colour:' )  <<  backgroundColourButton.colourButton  <<  None
	layout.row()  <<  ''
	layout.row()  <<  _( 'Grid' )
	layout.row()  <<  _( 'Grid level 1 colour:' )  <<  gridLevel1ColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Grid level 2 colour:' )  <<  gridLevel2ColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Grid level 3 colour:' )  <<  gridLevel3ColourButton.colourButton  <<  None
	layout.row()  <<  ''
	layout.row()  <<  _( 'Vertices' )
	layout.row()  <<  _( 'Unmarked vertex colour:' )  <<  unmarkedVertexColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Unmarked vertex point size:' )  <<  unmarkedVertexPointSizeEntry.entry  <<  None
	layout.row()  <<  _( 'Marked vertex colour:' )  <<  markedVertexColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Marked vertex point size:' )  <<  markedVertexPointSizeEntry.entry  <<  None
	layout.row()  <<  _( 'Reflected vertex colour:' )  <<  reflectedVertexColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Reflected vertex point size:' )  <<  reflectedVertexPointSizeEntry.entry  <<  None
	layout.row()  <<  ''
	layout.row()  <<  _( 'Edges' )
	layout.row()  <<  _( 'Unmarked edge colour:' )  <<  unmarkedEdgeColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Unmarked edge line width:' )  <<  unmarkedEdgeLineWidthEntry.entry  <<  None
	layout.row()  <<  _( 'Marked edge colour:' )  <<  markedEdgeColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Marked edge line width:' )  <<  markedEdgeLineWidthEntry.entry  <<  None
	layout.row()  <<  _( 'Background edge colour:' )  <<  backgroundEdgeColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Background edge line width:' )  <<  backgroundEdgeLineWidthEntry.entry  <<  None
	layout.row()  <<  _( 'Transparent background edge colour:' )  <<  transparentBackgroundEdgeColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Transparent background edge line width:' )  <<  transparentBackgroundEdgeLineWidthEntry.entry  <<  None
	layout.row()  <<  _( 'Reflected edge colour:' )  <<  reflectedEdgeColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Reflected edge line width:' )  <<  reflectedEdgeLineWidthEntry.entry  <<  None
	layout.row()  <<  _( 'Live subdivided edge colour:' )  <<  liveSubdEdgeColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Live subdivided edge line width:' )  <<  liveSubdEdgeLineWidthEntry.entry  <<  None
	layout.row()  <<  ''
	layout.row()  <<  _( 'Surface' )
	layout.row()  <<  _( 'Surface diffuse colour:' )  <<  surfaceDiffuseColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Surface specular colour:' )  <<  surfaceSpecularColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Transparent background diffuse colour:' )  <<  transparentBackgroundSurfaceDiffuseColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Transparent background specular colour:' )  <<  transparentBackgroundSurfaceSpecularColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Transparent background alpha:' )  <<  transparentBackgroundSurfaceAlphaEntry.entry  <<  transparentBackgroundSurfaceAlphaSlider.slider
	layout.row()  <<  _( 'Reflected surface filter colour:' )  <<  reflectedSurfaceFilterColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Live subdivided surface diffuse colour:' )  <<  liveSubdSurfaceDiffuseColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Live subdivided surface specular colour:' )  <<  liveSubdSurfaceSpecularColourButton.colourButton  <<  None
	layout.row()  <<  ''
	layout.row()  <<  _( 'Marked faces' )
	layout.row()  <<  _( 'Marked face colour:' )  <<  markedFaceColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Marked face alpha:' )  <<  markedFaceAlphaEntry.entry  <<  markedFaceAlphaSlider.slider
	layout.row()  <<  ''
	layout.row()  <<  _( 'Overlay' )
	layout.row()  <<  _( 'Overlay factor:' )  <<  overlayFactorEntry.entry  <<  overlayFactorSlider.slider
	layout.row()  <<  ''
	layout.row()  <<  _( 'Mirror plane' )
	layout.row()  <<  _( 'Mirror plane colour:' )  <<  mirrorPlaneColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Mirror plane alpha:' )  <<  mirrorPlaneAlphaEntry.entry  <<  mirrorPlaneAlphaSlider.slider
	layout.row()  <<  ''
	layout.row()  <<  _( 'Drawing plane' )
	layout.row()  <<  _( 'Drawing plane colour:' )  <<  drawingPlaneColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Drawing plane alpha:' )  <<  drawingPlaneAlphaEntry.entry  <<  drawingPlaneAlphaSlider.slider
	layout.row()  <<  ''
	layout.row()  <<  _( 'Highlights' )
	layout.row()  <<  _( 'Vertex pick highlight point size:' )  <<  vertexPickHighlightPointSizeEntry.entry  <<  None
	layout.row()  <<  _( 'Edge pick highlight line width:' )  <<  edgePickHighlightLineWidthEntry.entry  <<  None
	layout.row()  <<  _( 'Target highlight colour:' )  <<  targetHighlightColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Marked target highlight colour:' )  <<  markedTargetHighlightColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Invalid target highlight colour:' )  <<  invalidTargetHighlightColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Warning target highlight colour:' )  <<  warningTargetHighlightColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Path highlight colour:' )  <<  pathHighlightColourButton.colourButton  <<  None
	layout.row()  <<  ''
	layout.row()  <<  _( 'Modification highlights' )
	layout.row()  <<  _( 'Vertex reticle colour:' )  <<  vertexReticleColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Modification 1 highlight colour:' )  <<  modification1HighlightColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Modification 2 highlight colour:' )  <<  modification2HighlightColourButton.colourButton  <<  None
	layout.row()  <<  _( 'Modification highlight alpha:' )  <<  modificationHighlightAlphaEntry.entry  <<  modificationHighlightAlphaSlider.slider
	layout.finalise()





class ViewDrawingSettingsEditorWindow (object):
	closeSignal = ClassSignal()
	applySignal = ClassSignal()


	def __init__(self, view, parentWindow):
		self._drawingSettings = None
		self._view = view
		self._parentWindow = parentWindow


		titleLabel = gtk.Label()
		titleLabel.set_markup( '<span foreground="#000080"><b>'  +  _( 'View drawing settings' )  +  '</b></span>' )
		titleLabel.show()


		scrolledWindow = gtk.ScrolledWindow()
		scrolledWindow.set_policy( gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC )



		self._editor = ViewDrawingSettingsEditor()

		self._editor.createRowEditorUI()
		self._editor.layoutRowEditorUI()
		self._editor.getRowEditorWidget().show()
		scrolledWindow.add_with_viewport( self._editor.getRowEditorWidget() )



		scrolledWindow.show()


		buttonBoxSep = gtk.HSeparator()
		buttonBoxSep.show()

		loadButton = gtk.Button( stock=gtk.STOCK_OPEN )
		loadButton.connect( 'clicked', self._p_onOpen )
		loadButton.show()

		saveButton = gtk.Button( stock=gtk.STOCK_SAVE )
		saveButton.connect( 'clicked', self._p_onSave )
		saveButton.show()

		resetButton = gtk.Button( _( 'Reset' ) )
		resetButton.connect( 'clicked', self._p_onReset )
		resetButton.show()

		cancelButton = gtk.Button( stock=gtk.STOCK_CANCEL )
		cancelButton.connect( 'clicked', self._p_onCancel )
		cancelButton.show()

		applyButton = gtk.Button( stock=gtk.STOCK_APPLY )
		applyButton.connect( 'clicked', self._p_onApply )
		applyButton.show()

		okButton = gtk.Button( stock=gtk.STOCK_OK )
		okButton.connect( 'clicked', self._p_onOk )
		okButton.show()


		buttonBox = gtk.HBox( spacing=10 )
		buttonBox.pack_start( loadButton, False, False )
		buttonBox.pack_start( saveButton, False, False )
		buttonBox.pack_end( okButton, False, False )
		buttonBox.pack_end( applyButton, False, False )
		buttonBox.pack_end( cancelButton, False, False )
		buttonBox.pack_end( resetButton, False, False )
		buttonBox.show()


		windowBox = gtk.VBox()
		windowBox.pack_start( titleLabel, False, False, 10 )
		windowBox.pack_start( scrolledWindow, True, True, 0 )
		windowBox.pack_start( buttonBoxSep, False, False, 10 )
		windowBox.pack_start( buttonBox, False, False, 10 )
		windowBox.show()


		self._window = gtk.Window( gtk.WINDOW_TOPLEVEL )
		self._window.set_title( _( 'View drawing settings' ) )
		self._window.set_transient_for( parentWindow )
		self._window.add( windowBox )
		self._window.set_border_width( 20 )
		self._window.set_size_request( -1, 500 )
		self._window.connect( 'delete-event', self._p_onWindowDelete )




	def start(self, drawingSettings):
		self._drawingSettings = copy( drawingSettings )
		self._editor.attachRow( self._drawingSettings, self._view, None, self._parentWindow, None )
		self._editor.initialiseRowEditorUI()
		self._window.show()


	def stop(self):
		self._editor.shutdownRowEditorUI()
		self._editor.detachRow()
		self._window.hide()
		self.closeSignal.emit()
		self._drawingSettings = None




	def _p_onOpen(self, button):
		gsviewFilter = gtk.FileFilter()
		gsviewFilter.set_name( _( 'gSculpt View drawing settings (*.gsview)' ) )
		gsviewFilter.add_pattern( '*.gsview' )

		openDialog = gtk.FileChooserDialog( _( 'Open' ), self._window, gtk.FILE_CHOOSER_ACTION_OPEN,
										( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK ) )
		openDialog.set_current_folder( os.getcwd() )
		openDialog.add_filter( gsviewFilter )
		openDialog.show()
		response = openDialog.run()
		filename = openDialog.get_filename()
		openDialog.destroy()
		if response == gtk.RESPONSE_OK:
			if filename is not None:
				try:
					readViewDrawingSettings( self._drawingSettings, filename )
				except IOError:
					messageDialog( _( 'Could not read file' ), self._window, _( 'Could not read \'%s\'' )  %  ( filename, ) )
				else:
					self._p_apply()


	def _p_onSave(self, button):
		gsviewFilter = gtk.FileFilter()
		gsviewFilter.set_name( _( 'gSculpt View drawing settings (*.gsview)' ) )
		gsviewFilter.add_pattern( '*.gsview' )

		saveDialog = gtk.FileChooserDialog( _( 'Save' ), self._window, gtk.FILE_CHOOSER_ACTION_SAVE,
									( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK ) )
		saveDialog.add_filter( gsviewFilter )
		saveDialog.set_current_folder( os.getcwd() )
		saveDialog.show()
		response = saveDialog.run()
		filenameFromDialog = saveDialog.get_filename()
		saveDialog.destroy()
		filename = None
		if response == gtk.RESPONSE_OK:
			if filenameFromDialog is not None:
				if os.path.exists( filenameFromDialog ):
					if confirmOverwriteFileDialog( filenameFromDialog, self._window ):
						filename = filenameFromDialog
				else:
					filename = filenameFromDialog
		if filename is not None:
			try:
				writeViewDrawingSettings( self._drawingSettings, filename )
			except IOError:
				messageDialog( _( 'Could not write file' ), self._window, _( 'Could not write \'%s\'' )  %  ( filename, ) )






	def _p_apply(self):
		self.applySignal.emit( self._drawingSettings )


	def _p_onReset(self, widget):
		dialog = gtk.Dialog( _( 'Confirm' ), self._window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, ( gtk.STOCK_YES, gtk.RESPONSE_ACCEPT, gtk.STOCK_NO, gtk.RESPONSE_REJECT ) )
		dialog.vbox.pack_start( gtk.Label( _( 'Are you sure you wish to reset the view drawing settings?' ) ) )
		dialog.show_all()
		response = dialog.run()
		dialog.destroy()
		if response == gtk.RESPONSE_ACCEPT:
			self._editor.shutdownRowEditorUI()
			self._editor.detachRow()
			self._drawingSettings = ViewDrawingSettings()
			self._editor.attachRow( self._drawingSettings, self._view, None, self._parentWindow, None )
			self._editor.initialiseRowEditorUI()
			self._p_apply()



	def _p_onCancel(self, widget):
		self.stop()


	def _p_onApply(self, widget):
		self._p_apply()


	def _p_onOk(self, widget):
		self._p_apply()
		self.stop()

	def _p_onWindowDelete(self, widget, event):
		self.stop()
		# Don't allow the window to be deleted
		return True
